var arr = [];
var loadGraph  = function(data, months){
    var options = {
        chart: {
            height: 300,
            type: 'line',
            toolbar: {
                show: false
            },
            zoom: {
                enabled: false
            }
        },
        colors: ['#0080ff'],
        series: [{
            name: "Student Minutes",
            data: data
        }],
        dataLabels: {
            enabled: false
        },
        stroke: {
            curve: 'straight'
        },
        grid: {
            row: {
                colors: ['transparent', 'transparent'], opacity: .2
            },
            borderColor: 'rgba(0,0,0,0.05)'
        },
        xaxis: {
            categories: months,
            axisBorder: {
                show: true, 
                color: 'rgba(0,0,0,0.05)'
            },
            axisTicks: {
                show: true, 
                color: 'rgba(0,0,0,0.05)'
            }
        }
    }
    var chart = new ApexCharts(
        document.querySelector("#apex-line-chart"),
        options
    );
    chart.render();
}

var load = function(){
    api.home.get(undefined, function(success) {
        if(success && success.counts){
            var result = success.counts;
            $('#users').html((result.users ? result.users : 0));
            $('#departments').html((result.departments ? result.departments : 0));
            $('#p_courses').html((result.published_courses ? result.published_courses : 0));
            $('#campaigns').html((result.campaigns ? result.campaigns : 0));
            $('#quizzes').html((result.quizzes ? result.quizzes : 0));
            $('#skills').html((result.skills ? result.skills : 0));
            $('#recent_campaigns').html('');
            if(result.recent_campaigns && result.recent_campaigns.length > 0)
                result.recent_campaigns.forEach(function(recent_campaign) {
                    $('#recent_campaigns').append('<tr><td>'+capitalizeFirstLetter(recent_campaign.name)+'</td> <td>'+dataFormat(recent_campaign.createdAt)+'</td> <td>'+(recent_campaign.Simulations.length > 0 ? (recent_campaign.Simulations[0].type == 'quiz' ? 'Assessment' : capitalizeFirstLetter(recent_campaign.Simulations[0].type)) : '-----')+'</td> </tr>');
                });
            else  $('#recent_campaigns').html('No Campaign Found');
        }
    });

    api.home.types(undefined, function(success) {
        if(success && success.counts){
            var result = success.counts;
            var percentCourses =  Math.round(parseFloat((result.courses / result.total) * 100).toFixed(2));
            var percentQuizzes =  Math.round(parseFloat((result.quizzes / result.total) * 100).toFixed(2));
            var percentAnnouncements =  Math.round(parseFloat((result.announcements / result.total) * 100).toFixed(2));
            var averageCampaign =  (result.courses + result.quizzes + result.announcements) / result.total;
            $('#average_campaigns').val((averageCampaign ?  Math.ceil(averageCampaign * 100) : '0'))
            $('#course-percent').html((percentCourses ? percentCourses : '0')+'%');
            $('#quizzes-percent').html((percentQuizzes ? percentQuizzes : '0')+ '%');
            $('#announcement-percent').html((percentAnnouncements ? percentAnnouncements : '0')+ '%');
            $('#course-percent-progress').css({ 'width': (percentCourses ? percentCourses : '0')+ '%' });
            $('#quizzes-percent-progress').css({ 'width': (percentQuizzes ? percentQuizzes : '0')+ '%' });
            $('#announcement-percent-progress').css({ 'width': (percentAnnouncements ? percentAnnouncements : '0')+ '%' });
        }
    });

    api.home.graph(undefined, function(success) {
        var months = []; var data = [];
        var tempDate = new Date();
        tempDate.setMonth(tempDate.getMonth()-6);
        for(var i=0;i<7;i++){
            var monthString = moment(tempDate).format("MMM");
            months.push(monthString);
            if(success.graph[monthString]) data.push(Math.round(success.graph[monthString] / 60));
            else data.push(0);
            tempDate.setMonth(tempDate.getMonth()+1);
        }
        loadGraph(data, months);
    });

    api.home.activity(undefined, function(success) {
       $('#recent-activity').html('');
       if(success && success.data && success.data.length > 0){
           success.data.forEach(function(activity) {
               $('#recent-activity').append(' <div class="activities-history-list"> <div class="activities-history-item"> <h6>'+activity.text+'</h6> <p class="mb-0">'+dataFormat(activity.createdAt)+'</p> </div> </div>');
           })
       }else $('#recent-activity').html('No Activity Found');
    },function(error){
        $('#recent-activity').html('No Activity Found');
    });

    api.home.topCourses(undefined,function(success){
        $('#topCourses').html('');
        if(success && success.data && success.data.length > 0){
            success.data.forEach(function(course) {
                var p =Math.round(course.courseCount/course.campaignCount * 100);
                $('#topCourses').append('<p>'+course.name+' <span class="float-right">'+p+'%</span></p> <div class="progress mb-4" style="height: 4px;"> <div class="progress-bar bg-success" role="progressbar" style="width: '+p+'%;" aria-valuenow="'+p+'" aria-valuemin="0" aria-valuemax="100"></div> </div>');
            })
        }else $('#topCourses').html('No Courses Found');
    },function(error) {
        $('#topCourses').html('No Courses Found');
    });
}
function setTypeFunction(type){
    window.location.href = "/course/create?type="+type;
}


$(document).ready(function(){
    load();
});