import { db } from '../../../models';
import Blowfish from 'egoroof-blowfish';
import config from '../../../config';
import mailer from '../../../mailer';
import { queue } from '../../../kue';
import {differentDateFormat} from '../../../functions';
import fs from 'fs';

const bf = new Blowfish(toString(config.app.secret), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);
const fromHexString = hexString => new Uint8Array(hexString.match(/.{1,2}/g).map(byte => parseInt(byte, 16)));

var dcryptPassword = (password) => {
    return new Promise((resolve, reject) => {
        if (password && password.length) resolve(bf.decode(fromHexString(password)));
        else resolve(false);
    })
}    

function retryArray(dataArr,inputPath){
    return new Promise((resolve,reject) => {
        let retryArr = []
        let batching = 30;
        let count = 0;
        if(dataArr.length > 0){
        return dataArr.reduce((ac,element, i) => {
            let obj = {};
            obj.email = element.split(',')[0];
            obj.courseId = element.split(',')[1];
            obj.simulationId = element.split(',')[2];
            obj.remainderId = element.split(',')[3];
            retryArr.push(obj)
            fs.writeFile(inputPath, `Email,Course,Simulation,Reminder`+ "\n", function(){})
            if ((i + 1) % batching == 0 || i == dataArr.length -1){
                queue.create('retry-remainder', {
                    retries: retryArr
                }).delay((count++) * 1000).removeOnComplete(true).save();
                retryArr = [];
            }
        },Promise.resolve(resolve(true)))
    }else reject(false)
    })
}

export default {
    async index(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '%%';
        let sort = ['createdAt', 'DESC'];
        if (req.query.limit != undefined) {
            limit = parseInt(req.query.limit);
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        if (req.query.search) {
            search = '%' + req.query.search + '%';
        }
        db.Course.count({
            where: { status: 'Published', companyId: req.user.companyId, courseType: 'streaming', name: { like: search } }
        })
            .then((count) => {
                let pages = Math.ceil(count / limit);
                offset = limit * (page - 1);
                return db.Course.findAll({
                    attributes: ['id', 'name', 'totalDuration', 'createdAt'],
                    where: { status: 'Published', companyId: req.user.companyId, courseType: 'streaming', name: { like: search } },
                    include: [{
                        model: db.MemberCourse,
                        attributes: ['id', 'courseId', 'sessionId'],
                        include: [{
                            model: db.Session,
                            attributes: ['id', 'memberId'],
                            include: [{ model: db.Group }]
                        }]
                    }],
                    limit: limit,
                    offset: offset,
                    order: [sort]
                }).then(r => [r, count, pages])
            })
            .then(([course, count, pages]) => {
                return res.status(200).json({ course: course, 'count': count, 'pages': pages });
            })
            .catch(err => {
                console.log(err);
                next();
            })
    },

    async list(req, res, next) {
        db.Course.findAll({
            attributes: ['id', 'name'],
            order: [['id', 'desc']],
            where: { status: 'Published', 'companyId': req.user.companyId, 'courseType': 'streaming' }
        })
            .then(course => {
                if (course)
                    return res.status(200).json({ course: course });
                else throw new RequestError("server Error.");
            })
            .catch(err => {
                //  console.log(err);
            })
    },

    async courseData(req, res, next) {
        db.Course.findOne({
            attributes: ['id', 'name', 'companyId', 'status'],
            where: { id: req.query.courseId, companyId: req.user.companyId, status: 'Published' },
            include: [{
                model: db.CourseSection,
                include: [{
                    model: db.CourseSectionLecture, group: ['id'],
                    include: [{ model: db.SessionCourseLectureTracking, }]
                }]
            }],
            order: [[db.CourseSection, db.CourseSectionLecture, 'id', 'ASC']]
        })
            .then(course => {
                if (course) {
                    return db.MemberCourse.findAll({
                        attributes: ['id', 'courseId', 'sessionId'],
                        where: { courseId: course.id },
                        include: [{
                            model: db.Session,
                            attributes: ['id', 'memberId'],
                            include: [{ model: db.Group }, { model: db.SessionCourseLectureTracking, }]
                        }]
                    }).then(r => [r, course])
                } else throw new RequestError("Course Not Found");
            })
            .then(([memberCourse, course]) => {
                return res.status(200).json({ course: course, memberCourse: memberCourse });
            })
            .catch(err => {
                console.log(err);
            })
    },
    async studentList(req, res, next) {
        var condition;
        var condition2 = { courseId: req.query.courseId };
        let limit = 10;
        let offset = 0;
        let page = 1;
        let sort = ['createdAt', 'DESC'];
        let order_status = [];
        if (req.query.limit != undefined) {
            limit = parseInt(req.query.limit);
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1) page = 1;
        }
        if (req.query.search) {
            var search = '%' + req.query.search + '%';
            condition = { firstName: { like: search } };
        }
        if (req.query && req.query.hasOwnProperty('sort')) {
            order_status = req.query.sort == 'name' ? [[db.Member, 'firstName', 'Asc'], sort] : [sort];
        }
        if (req.query.percentage) {
            var percent = req.query.percentage;
            console.log(req.query.percentage)
            condition2['percent'] = { $between: [percent.start, percent.end] }
        }
        if (req.query.type == 'CSV') {
            var uaParser = require('ua-parser-js');
            var ip = req.headers["x-real-ip"] || req.ip;
            var detail = {};
            detail.user_agent = req.headers['user-agent'];
            detail.systemDetail = uaParser(req.headers['user-agent']);
            detail.ip = ip;
            var info = { info: detail };
            var metaData = JSON.stringify(info);
            var username = req.user.User.firstName + ' ' + req.user.User.lastName;
            var userId = req.user.User.id;
            var companyId = req.user.companyId;
            var type = 'course';
            var text = 'CSV file of Student list is downloaded by ' + username;
            db.RecentActivity.create({ userId, companyId, type, text, metaData });
        }

        db.Course.findOne({ where: { id: req.query.courseId, companyId: req.user.companyId } })
            .then(course => {
                if (course)
                    return db.MemberCourse.count({
                        where: condition2,
                        include: [{
                            model: db.Member,
                            where: condition,
                        }]
                    })
                else throw new RequestError("Course Not Found");
            })
            .then(count => {
                let pages = Math.ceil(count / limit);
                offset = limit * (page - 1);
                if (req.query.type == undefined) {
                    return db.MemberCourse.findAll({
                        where: condition2,
                        include: [{
                            model: db.Member,
                            where: condition
                        }],
                        limit: limit,
                        offset: offset,
                        order: order_status
                    })
                        .then(r => [r, count, pages]
                        )
                } else {
                    return db.MemberCourse.findAll({
                        where: condition2,
                        include: [{
                            model: db.Member,
                            where: condition,
                        }],
                        order: order_status
                    })
                        .then(r => [r, count, pages])
                }
            })
            .then(([members, count, pages]) => {
                if (members)
                    return res.status(200).json({ courseMembers: members, 'count': count, 'pages': pages });
                else throw new RequestError("No members found");
            })
            .catch(err => {
                next(err);
            })

    },
    async courseGraph(req, res) {
        var condition;
        var tempDate = new Date();
        var courseId = req.query.courseId;
        if (courseId)
            condition = { id: courseId };
        tempDate.setMonth(tempDate.getMonth() - 7);
        db.SessionCourseLectureTracking.findAll({
            attributes: [[db.sequelize.fn('SUM', db.sequelize.col('SessionCourseLectureTracking.duration')), 'total'], [db.sequelize.literal("CASE month(`SessionCourseLectureTracking`.`createdAt`) WHEN 1 THEN 'Jan' WHEN 2 THEN 'Feb' WHEN 3 THEN 'Mar' WHEN 4 THEN 'Apr' WHEN 5 THEN 'May' WHEN 6 THEN 'Jun' WHEN 7 THEN 'Jul' WHEN 8 THEN 'Aug' WHEN 9 THEN 'Sept' WHEN 10 THEN 'Oct' WHEN 11 THEN 'Nov' WHEN 12 THEN 'Dec' END"), 'months']],
            where: { createdAt: { $gte: tempDate } },
            order: ['createdAt'],
            group: ['months'],
            include: [{
                model: db.Session,
                where: { companyId: req.user.companyId }
            },
            {
                model: db.CourseSectionLecture, required: true,
                include: [
                    {
                        model: db.CourseSection, required: true,
                        include: [
                            {
                                model: db.Course,
                                where: condition,
                                required: true
                            }
                        ]
                    }
                ]
            }
            ]
        })
            .then(graph => {
                graph = json(graph);
                var finalData = {};
                for (var i = 0; i < graph.length; i++)
                    finalData[graph[i]['months']] = graph[i].total;
                return res.status(200).json({ graph: finalData });
            })
            .catch(err => {
                console.log(err);
                throw new RequestError('Fetching Error Counts');
            })
    },

    async lectureList(req, res, next) {
        // var condition;
        var condition2 = { courseId: req.query.courseId };
        let limit = 10;
        let offset = 0;
        let page = 1;
        let sort = ['createdAt', 'ASC'];
        // if (req.query.limit != undefined) {
        //     limit = parseInt(req.query.limit);
        // }
        // if (req.query.page) {
        //     page = req.query.page;
        //     if (page < 1) page = 1;
        // }
        // if (req.query.search){
        //     var search = '%' + req.query.search + '%';
        //     condition = {  firstName : { like: search } };
        // }
        // if (req.query.sort) {
        //     if (req.query.sort == 'name') {
        //         sort = ['name'];
        //     }
        // }
        // if (req.query.percentage) {
        //     var percent = req.query.percentage;
        //     console.log(req.query.percentage)
        //     condition2['percent'] = { $between: [percent.start, percent.end] }
        // }
        db.Course.findOne({ where: { id: req.query.courseId, companyId: req.user.companyId } })
            .then(course => {
                if (course)
                    return db.CourseSectionLecture.count({
                        where: { type: 'quiz' },
                        include: [{ model: db.CourseSection, where: { courseId: req.query.courseId }, required: true, }]
                    }).then(r => [r, course]);
                else throw new RequestError("Invalid Course Found");
            })
            .then(([count, course]) => {
                let pages = Math.ceil(count / limit);
                offset = limit * (page - 1);
                return db.CourseSectionLecture.findAll({
                    where: { type: 'quiz' },
                    include: [{ model: db.CourseSection, where: { courseId: req.query.courseId } },/* { model:db.CourseLectureQuestion, } */],
                    limit: limit,
                    offset: offset,
                    order: [sort],
                }).then(r => [r, count, pages, course])
            })
            .then(([lectures, count, pages, course]) => {
                if (lectures)
                    return res.status(200).json({ course: course, lectures: lectures, 'count': count, 'pages': pages });
                else throw new RequestError("No members found");
            })
            .catch(err => {
                next(err);
            })
    },

    async studentQuizResult(req, res, next) {
        var condition;
        var condition2 = { courseSectionLectureId: req.query.lectureId };
        let limit = 10;
        let offset = 0;
        let page = 1;
        let sort = ['createdAt', 'DESC'];
        let maxMarks = 0;
        let bool = false;
        if (req.query.type == 'CSV') {
            var uaParser = require('ua-parser-js');
            var ip = req.headers["x-real-ip"] || req.ip;
            var detail = {};
            detail.user_agent = req.headers['user-agent'];
            detail.systemDetail = uaParser(req.headers['user-agent']);
            detail.ip = ip;
            var info = { info: detail };
            var metaData = JSON.stringify(info);
            var username = req.user.User.firstName + ' ' + req.user.User.lastName;
            var userId = req.user.User.id;
            var companyId = req.user.companyId;
            var type = 'course';
            var text = 'CSV file of Student Quiz Result is downloaded by ' + username;
            db.RecentActivity.create({ userId, companyId, type, text, metaData });
        }
        if (req.query.limit != undefined) {
            limit = parseInt(req.query.limit);
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1) page = 1;
        }
        if (req.query.search) {
            var search = '%' + req.query.search + '%';
            condition = {
                $or: [
                  db.sequelize.where(db.sequelize.fn('concat', db.sequelize.col('Member.firstName'), ' ', db.sequelize.col('Member.lastName')), {
                    like: search
                  }),
                    { email: { $like: search } }, 
                ]
              }
        }
        // if (req.query.sort) {
        //     if (req.query.sort == 'name') {
        //         sort = ['name'];
        //     }
        // }
        if (req.query.percentage) {
            var percent = req.query.percentage;
            condition2['percent'] = { $between: [percent.start, percent.end] }
            bool = true;
        }
        db.CourseSectionLecture.findOne({ where: { id: req.query.lectureId }, include: [{ model: db.CourseSection, include: [{ model: db.Course, where: { companyId: req.user.companyId } }] }] })
            .then(lecutre => {
                if (lecutre){
                    maxMarks = (lecutre.questionCount)*5
                    return db.MemberCourse.count({
                        where:{courseId:lecutre.CourseSection.Course.id}
                    }).then(r=>[r,lecutre])
                }
                else throw new RequestError("Lecutre Not Found");
            })
            .then(([count,lecutre]) => {
                let pages = Math.ceil(count / limit);
                offset = limit * (page - 1);
                if (req.query.type == undefined) {
                    return db.MemberCourse.findAll({
                        where:{courseId:lecutre.CourseSection.Course.id},
                        include: [{model: db.Session, required:true,include:[{model: db.CourseUserQuizDetail,required: bool,where:condition2}] }, 
                                    { model: db.Member, required: true, where: condition }]
                        })
                        .then(r => [r, count, pages])
                } else {
                    return db.MemberCourse.findAll({
                        where:{courseId:lecutre.CourseSection.Course.id},
                        include: [{model: db.Session, required:true,include:[{model: db.CourseUserQuizDetail, required: bool,where:condition2}] }, 
                                    { model: db.Member, required: true, where: condition }]
                        })
                        .then(r => [r, count, pages])
                }
            })
            .then(([members, count, pages]) => {
                if (members)
                    return db.CourseSectionLecture.findOne({ where: { id: req.query.lectureId }, include: [{ model: db.CourseSection, include: [{ model: db.Course }] }] }).then(r => [r, members, count, pages])
                else throw new RequestError("No members found");
            })
            .then(([course, members, count, pages]) => {
                if (course)
                    return res.status(200).json({ course: course, courseMembers: members, 'count': count, 'pages': pages,'maxMarks':maxMarks });
                else throw new RequestError("Invalid Course found");
            })
            .catch(err => {
                next(err);
            })
    
    },

    async resendFailureReminder(req, res, next) {
        let type = 'quiz_fail'
        db.Course.findOne({ attributes: ['id', 'name', 'logo', 'status', 'companyId', 'setting'], where: { id: req.body.cId, companyId: req.user.companyId, status: 'Published' } })
        .then(course => {
            if (course)
                return db.CourseSectionLecture.findOne({ attributes: ['id', 'name', 'courseSectionId'], where: { id: req.body.lId }, include: [{ model: db.CourseSection, attributes: ['id', 'courseId', 'name'], required: true, where: { courseId: req.body.cId } }] }).then(r => [r, course])
            else throw new RequestError("Invalid Course found");
        })
        .then(([lecture, course]) => {
            if (lecture) {
                var quiz_passing_percentage = JSON.parse(course.setting).quiz_passing_percentage;
                return db.CourseUserQuizDetail.count({ attributes: ['id', 'courseSectionLectureId', 'sessionId', 'percent', 'attempts'], where: { courseSectionLectureId: lecture.id, percent: { $lt: quiz_passing_percentage + '.00' }, attempts: { $gte: 1 } } }).then(r => [r, course, lecture.name, quiz_passing_percentage])
            } else throw new RequestError("Invalid lecture found");
        })
        .then(([quizCount, course, lecturename, quiz_passing_percentage]) => {
            return db.Company.findOne({ where: { id: req.user.companyId }, attributes: [['id', 'companyId'], 'name', 'logo', 'smtpStatus'], include: [{ model: db.MetaData, attributes: ['key', 'value'], where: { key: 'SDOMAIN' }, required: false }] }).then(r => [r, quizCount, course, lecturename, quiz_passing_percentage]);
        })
        .then(([company, quizCount, course, lecturename, quiz_passing_percentage]) => {
            return db.Templates.findOne({ where: { name: mailer.getTemplateType[type] } }).then(r => [r, company, quizCount, course, lecturename, quiz_passing_percentage])
        })
        .then(([template, company, quizCount, course, lecturename, quiz_passing_percentage]) => {
            if (template){
                if (company && company.smtpStatus == 'custom')
                    return db.Sender.findOne({ where: { companyId: req.user.companyId }, order: [['id', 'DESC']] }).then(r => [r, template, company, quizCount, course, lecturename, quiz_passing_percentage])
                else return db.Sender.findOne({ where: { companyId: null } }).then(r => [r, template, company, quizCount, course, lecturename, quiz_passing_percentage])
            }else throw new RequestError('Invalid Template received');
        })
        .then(([sender, template, company, quizCount, course, lecturename, quiz_passing_percentage]) => {
            if (sender) {
                var emailconfig = {
                    tls: { rejectUnauthorized: false },
                    host: sender.host.split(':')[0],
                    port: sender.host.split(':')[1],
                };
                if (sender.requiretls) {
                    emailconfig['requireTLS'] = sender.requiretls
                } else {
                    if ((sender.ssl != null) && (sender.ssl == true || sender == false)) {
                        emailconfig['ssl'] = sender.ssl
                    }
                }
                if (sender.username) {
                    emailconfig['auth'] = { user: sender.username, }
                    if (sender.password)
                        return dcryptPassword(sender.password).then(r => [r, template, company, emailconfig, sender.fromAddress, sender.delay, sender.limit, quizCount, course, lecturename, quiz_passing_percentage])
                }
                return [false, template, company, emailconfig, sender.fromAddress, sender.delay, sender.limit, quizCount, course, lecturename, quiz_passing_percentage];
            } else reject(new RequestError('SMTP Profile is not set.'));
        })
        .then(([password, template, company, emailconfig, fromAddress,delay,limit,quizCount,course,lecturename,quiz_passing_percentage]) => {
            if(password) emailconfig.auth['pass'] = password;
            return db.MetaData.findOne({ where: { companyId: company.companyId, key: template.name}, attributes:['value'] }).then(r =>[r, template, company, emailconfig, fromAddress,delay,limit,quizCount,course,lecturename,quiz_passing_percentage])
        })
        .then(([metadata, template, company, emailconfig, fromAddress, delay, limit, quizCount, course, lecturename, quiz_passing_percentage]) => {
            let delayVal = config.mail.delay, batchingRangeVal = config.mail.batching;
            if (company && company.smtpStatus == 'custom') {
                if (delay && delay != null)
                    delayVal = delay;
                if (limit && limit != null)
                    batchingRangeVal = limit;
            }
            let miliseconds = 0;
            for (let i = 0; i < Math.ceil(quizCount / parseInt(batchingRangeVal)); i++) {
                miliseconds = parseInt(delayVal) * i;
                let offset = i * parseInt(batchingRangeVal);
                queue.create('remainder-resolve-error', {
                    limit: batchingRangeVal,
                    offset: offset,
                    quizCount: quizCount,
                    companyId: req.user.companyId,
                    type: type,
                    ltd: req.body.lId,
                    quiz_passing_percentage: quiz_passing_percentage,
                    course: course,
                    lecturename: lecturename,
                    metadata: metadata,
                    template: template,
                    company: company,
                    emailconfig: emailconfig,
                    fromAddress: fromAddress
                }).delay(miliseconds).removeOnComplete(true).save();
            }
            return true;
        })
        .then(result => {
            return res.status(200).json({ success: true, members: result });
        })
        .catch(err => {
            next(err);
        })
    },

    async showtotalcount(req, res, next) {
        db.Course.findAll({
            attributes: ['id'],
            where: { status: 'Published', companyId: req.user.companyId, courseType: 'streaming' }
        })
            .then(result => {
                let active_learner = [];
                if (result) {
                    result.reduce((p, a) => {
                        active_learner.push(a.id)
                    }, Promise.resolve())
                    return db.MemberCourse.count({ where: { courseId: { $in: active_learner } } })
                }
                else throw new RequestError("Course Not Found");
            })
            .then(activeLearner => {
                if (activeLearner) {
                    return db.Group.findAll({
                        attributes: [[db.sequelize.fn('SUM', db.sequelize.literal('(SELECT DISTINCT COUNT(DISTINCT `Sessions`.`groupId`) FROM `Sessions` WHERE `Sessions`.`groupId` = `Group`.`id`)')), 'departmentcountNumber']],
                        where: { companyId: req.user.companyId }
                    }).then(result => [result, activeLearner])
                }
                else throw new RequestError("Course Not Found");
            })
            .then(([departmentcount, activeLearner]) => {
                return db.SessionCourseLectureTracking.findAll({
                    attributes: [[db.sequelize.fn('SUM', db.sequelize.col('SessionCourseLectureTracking.duration')), 'total']],
                    include: [{
                        model: db.Session, attributes: [],
                        where: { companyId: req.user.companyId }
                    },
                    {
                        model: db.CourseSectionLecture, attributes: [], required: true,
                        include: [
                            {
                                model: db.CourseSection, attributes: [], required: true,
                                include: [
                                    {
                                        model: db.Course, attributes: [],
                                        required: true
                                    }
                                ]
                            }
                        ]
                    }
                    ]
                }).then(r => [r, departmentcount, activeLearner])
            })
            .then(([mintaught, departmentcount, activeLearner]) => {
                if (mintaught) {
                    departmentcount = JSON.parse(JSON.stringify(departmentcount[0]))
                    departmentcount = departmentcount.departmentcountNumber
                    mintaught = JSON.parse(JSON.stringify(mintaught[0]))
                    mintaught = mintaught.total;
                    mintaught = Math.floor(mintaught / 60);
                    res.status(200).json({ 'success': true, 'mintaught': mintaught, 'departmentcount': departmentcount, 'activeLearner': activeLearner })
                } else throw new RequestError("Course Not Found");
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async userTrackingList(req,res,next){
        if (req.query.type == 'CSV') {
            let uaParser = require('ua-parser-js');
            let ip = req.headers["x-real-ip"] || req.ip;
            let detail = {};
            detail.user_agent = req.headers['user-agent'];
            detail.systemDetail = uaParser(req.headers['user-agent']);
            detail.ip = ip;
            let info = { info: detail };
            let metaData = JSON.stringify(info);
            let username = req.user.User.firstName + ' ' + req.user.User.lastName;
            let userId = req.user.User.id;
            let companyId = req.user.companyId;
            let type = 'course';
            let text = 'CSV file of Student list is downloaded by ' + username;
            db.RecentActivity.create({ userId, companyId, type, text, metaData });
        }
                db.Course.findOne({
                    attributes: ['id','setting'],
                    where: { id: req.query.courseId, companyId: req.user.companyId, status: 'Published' },
                    include: [{
                        model: db.CourseSection,attributes:['id','name'],
                        include: [{
                            model: db.CourseSectionLecture, attributes: ['id','courseSectionId','name'],
                            include: [{ model: db.SessionCourseLectureTracking,attributes:['percent'],where:{sessionId:req.query.sessionId},required:false},{ model: db.CourseUserQuizDetail,attributes:['percent','maxMarks','obtainedMarks','attempts'],where:{sessionId:req.query.sessionId},required:false}]
                        }]
                    }]
                })    
            .then(result=>{
                if(result)
                res.status(200).json({'success':true,finalresult:result})
                else throw new RequestError("Course Not Found");
            })
    },

    async retryFailRemainder(req, res, next) {
        let inputPath = appRoot +`/public/error/error-${differentDateFormat(3, undefined)}.csv`;
        if(fs.existsSync(inputPath)){
            try{
                fs.readFile(inputPath, 'utf8', function (err, data) {
                    let dataArray = data.split(/\r?\n/);
                    dataArray.shift();
                    if(dataArray.length > 0){
                        retryArray(dataArray,inputPath)
                        .then(data => {
                            res.json({ 'success':true })
                        })
                        .catch(err=>{
                            console.log(err);
                            next(err);
                        })
                    }else res.json({ 'success':true, 'message': 'No data found.' });
                });
            }catch(err){
                console.log(err);
                next(err);
            } 
        }else res.json({ 'success':true, 'message': 'No reminder error file generated.' });
    },
    
    };