import bcrypt from 'bcrypt-nodejs';
import { db } from '../../../models';
import mailer from '../../../mailer';
import { makenumerickey , makealphakey, differentDateFormat } from '../../../functions';
import { queue } from '../../../kue';

export default {

    async index(req, res, next) {
        db.Company.findOne({ where: { id: req.user.companyId } })
        .then(company =>{
            if(company)
                return db.UserPackage.findAll({ where: { companyId: company.id } }).then(r => [company, r])
            else throw new RequestError('Error fetching Company',404);
        })
        .then(([company, userPack]) => { 
            if(userPack){
               return db.AppsumoCode.count({ where:{ userId: company.referralId } }).then(r => [r, userPack])
            }else throw new RequestError('Invalid Code',403);
        })
        .then(([code, userPack]) => {
            if(code){
                var count = code + 1;
                if(count < 5)
                    return db.AppsumoPlan.findOne({ where: { code:count } }).then(r => [r, code, userPack])
                else return [false, code, userPack];
            } else throw new RequestError('Error',500);
        })
        .then(([nextPack, code, userPack]) =>{
            return db.GroupMember.count({ include:[{ model:db.Group, where: { companyId: req.user.companyId} }] }).then(r => [r, nextPack, code, userPack])
        })
        .then(([count, nextPack, code, userPack]) =>{
            return db.User.findAll({ where:{ companyId: req.user.companyId } }).then(r => [r, count, nextPack, code, userPack ])
        })
        .then(([r, count, nextPack, code, userPack]) =>{
            return res.status(200).json({ status: true, count, code, userPack, nextPack, r:r.length });
        })
        .catch(err => {
            console.log(err);
            next(err);
        });
    },

    async check(req, res, next) {
        if(req.query.code){
            db.AppsumoCode.findOne({ where: { name: req.query.code, status: 1 } })
            .then(code =>{
                if(code)
                    return res.status(200).json({ status: true });
                else throw new RequestError('Invalid Code',404);
            })
            .catch(err => {
                console.log(err);
                next(err);
            });
        }else{
            next(new RequestError('Invalid Data received',403));
        }
    },

    async redeem(req, res, next) {
        if(req.query.code){
            db.AppsumoCode.findOne({ where: { name: req.query.code, status: 1 } })
            .then(code =>{
                if(code){
                    return db.AppsumoCode.count({ where: { status: 0, userId: req.user.userId } })
                } else throw new RequestError('Invalid Code',404);
            })
            .then(count => {
                count = count + 1;
                if(count < 5)
                    return db.AppsumoPlan.findOne({ where: { code: count } })
                else throw new RequestError('Not Allowed to redeem this code',403);
            })
            .then(appsumoplan =>{
                return db.UserPackage.destroy({ where: { companyId: req.user.companyId} }).then(r => appsumoplan)
            })
            .then(appsumoplan => {
                var obj = [];
                obj.push({ companyId: req.user.companyId, name:'users', value: appsumoplan.users });
                obj.push({ companyId: req.user.companyId, name:'storage', value: appsumoplan.storage });
                obj.push({ companyId: req.user.companyId, name:'dataTransfer', value: appsumoplan.dataTransfer });
                return db.UserPackage.bulkCreate(obj, {returning: true});
            })
            .then(r => {
                return db.AppsumoCode.update({ status: 0, userId: req.user.userId  }, { where: { name: req.query.code, status:1 } })
            })
            .then(r => {
                return res.status(200).json({ status: true });
            })
            .catch(err => {
                console.log(err);
                next(err);
            });
        }else{
            next(new RequestError('Invalid Data received',403));
        }
    },

    async signup(req, res, next) {
        const { firstName, lastName, email, password} = req.body;
        var alias = makealphakey();
        var key = makenumerickey();
        var passwordHash = bcrypt.hashSync(password);
        db.AppsumoCode.findOne({ where: { name: req.body.code, status: 1 } })
        .then(code => { 
            if(code){
                var nowDate = differentDateFormat(5);
                var expiredDate = differentDateFormat(5, code.expiredAt);
                if(nowDate > expiredDate) throw new RequestError('Code has been Expired.',401);
                else return db.User.findOne({ where : { email }}).then(r => [r, code])
            }else throw new RequestError('Invalid Code',403);
        })
        .then(([user, code]) => {
            if(user) throw new RequestError('Email already exists',409);
            return db.Company.create({ alias, key }).then(r => [r, code])
        })
        .then(([company, code])=>{
            if(company) return  db.User.findOrCreate({where: {email: email}, defaults: { firstName , lastName, companyId : company.id }}).then(r=>[r[0], code, company])
            throw new RequestError('Server error',500);
        })
        .then(([user, code, company]) => {
            if(user)  return db.MetaAuth.findOne({ where : { email : user.email } }).then(r => [r, user.id, code, company])
            throw new RequestError('Server error',500);
        })
        .then(([metaauth, userId, code, company]) => {
            if(metaauth)
                return db.MetaAuth.update({ userId: userId }, { where:{ id:metaauth.id } }).then(r => [r, code, userId, company])
            else return db.MetaAuth.findOrCreate( { where : { email : email } ,defaults: { password : passwordHash, userId : userId, status:'active', valid:1 } }).then(r=>[r[0], code, userId, company])
        })
        .then(([result, code, userId, company]) => {
            if(result) return db.AppsumoCode.update({ status: 0, userId: userId },{ where : {id : code.id}}).then(r => [userId, company]);
            else throw new RequestError('Server Error',500);
        })
        .then(([userId, company]) => {
            if(company) return db.Company.update({ referralId: userId },{ where : {id : company.id}}).then(r => [r, userId, company.id]);
            else throw new RequestError('Server Error',500);
        })
        .then(([result, userId, companyId]) => {
            if(result){
                queue.create('appsumo-user-package', {
                    companyId: companyId,
                    code: 1
                }).save();
                mailer.sendAppSumoUsers(userId, req.body.code);
                mailer.sendAppSumoWelcome(req.body);
                return res.status(200).json({ status: true });
            } else throw new RequestError('Error creating User',500);
        })
        .catch(err => {
            console.log(err);
            next(err);
        });
    },
};