import express from 'express';
import authController from './auth.controller';
import { localStrategy, jwtStrategy ,rootLoginStrategy} from '../../../middleware/strategy';
import { sanitize } from '../../../middleware/sanitizer';
import { validateBody, schemas } from '../../../middleware/validator';


export const authRouter = express.Router();

const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
    windowMs: 1 * 60 * 1000, // 1 minutes
    max: 100,
    message: "Too many accounts created from this IP, please try again after an hour"
  });
authRouter.route('/register').post(sanitize(), validateBody(schemas.registerSchema),apiLimiter, authController.register);
authRouter.route('/free-trial').post(sanitize(), validateBody(schemas.registerSchema),apiLimiter, authController.free_trial);
authRouter.route('/user').post(sanitize(), validateBody(schemas.rootCheckSchema),apiLimiter, authController.rootUserCheck);
authRouter.route('/login').post(sanitize(), validateBody(schemas.loginSchema),apiLimiter, rootLoginStrategy, authController.login);
authRouter.route('/changePassword').post(sanitize(), validateBody(schemas.changePassword),apiLimiter, jwtStrategy, authController.changePassword);
authRouter.route('/setPassword').post(sanitize(), validateBody(schemas.setPassword),apiLimiter, authController.setPassword);
authRouter.route('/sendReset').post(sanitize(), validateBody(schemas.sendResetPassword),apiLimiter, authController.sendReset);
authRouter.route('/reset').post(sanitize(), validateBody(schemas.resetPassword),apiLimiter, authController.resetPassword);
authRouter.route('/profile').patch(sanitize(), validateBody(schemas.userProfile),apiLimiter, jwtStrategy, authController.profileUpdate);
// big blue button APIs
authRouter.route('/create').get(sanitize(),apiLimiter,  authController.create);
authRouter.route('/joinmoderator').get(sanitize(),apiLimiter,  authController.joinModerator);
authRouter.route('/joinuser').get(sanitize(),apiLimiter,  authController.joinUser);
authRouter.route('/ismeetingrunning').get(sanitize(),apiLimiter,  authController.isMeetingRunning);
authRouter.route('/getmeetinginfo').get(sanitize(),apiLimiter,  authController.getMeetingInfo);
authRouter.route('/getmeetings').get(sanitize(),  authController.getMeetings);
authRouter.route('/endmeeting').get(sanitize(),  authController.end);