import { db } from '../../../models';
import Sequelize from 'sequelize';
import mailer from '../../../mailer';
import config from '../../../config';
import { queue } from '../../../kue';
import stringifyCSV from 'csv-stringify';
import fs from 'fs';

function sendCSVforDownload(items, job){
    return new Promise((resolve, reject) => {
        stringifyCSV(items, function (err, output) {
            if (err) reject(err);
            let fileName = Date.now();
            fs.writeFile(appRoot + '/public/downloads/' + fileName + '.csv', output, function (err) {
                if (err){ reject(err); throw err; };
                var data ={
                    fileName: fileName + '.csv'
                }  
                resolve(data);
            });
        })
    })
}

export default {
    async list(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '%%';
        let sort = ['createdAt', 'DESC'];
        if (req.query.limit != undefined) {
            limit = parseInt(req.query.limit);
        }
        if (req.query.search) {
            search = '%' + req.query.search + '%';
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        if (req.query.sort) {
            if (req.query.sort == 'name') {
                sort = ['name'];
            }
        }
        var condition = { companyId: req.user.companyId, name: { like: search } };
        if (req.query.status) {
            condition['status'] = req.query.status;
        }
        if (req.query.fromDate && req.query.toDate) {
            let endDate = new Date(req.query.toDate);
            endDate.setDate(endDate.getDate() + 1);
            condition['createdAt'] = { $between: [new Date(req.query.fromDate), endDate] }
        }
        db.Campaign.count({
            where: condition,
        })
            .then((count) => {
                let pages = Math.ceil(count / limit);
                offset = limit * (page - 1);
                db.Campaign.findAll({
                    attributes: ['id', 'name', 'memberCount', 'departmentCount', 'courseCount', 'createdAt', 'status'],
                    where: condition,
                    limit: limit,
                    offset: offset,
                    order: [sort],
                })
                    .then(campaign => {
                        res.status(200).json({ 'campaigns': campaign, 'count': count, 'pages': pages });
                    })
                    .catch(function (error) {
                        console.log(error);
                        res.status(500).json({ 'errors': ['Error fetching course.'] });
                    });
            })
            .catch(function (error) {
                console.log("TCL: list -> error", error)
                res.status(500).json({ 'errors': ['Error fetching course.'] });
            });

    },
    async delete(req, res, next) {
        db.Campaign.findOne({
            where: { companyId: req.user.companyId, id: req.params.id }
        })
            .then(campaign => {
                if (campaign) {
                    return db.Campaign.destroy({ where: { id: req.params.id } });
                } else {
                    throw {
                        name: 'PRAException',
                        msg: 'Campaign doesn\'t Exists'
                    }
                }
            })
            .then(r => {
                return res.status(200).json({ success: true });
            })
            .catch(err => {
                next(err);
            })
    },
    async deleteAll(req, res, next) {
        var campaignIds = req.body;
        db.Campaign.findAll({
            where: { companyId: req.user.companyId, id: campaignIds }
        })
            .then(campaign => {
                if (campaign) {
                    return campaign.reduce((p, data) => {
                        return db.Campaign.destroy({ where: { id: data.id } });
                    }, Promise.resolve());
                } else {
                    throw new RequestError('Campaign doesn\'t Exists')
                }
            })
            .then(r => {
                res.status(200).json({ success: true });
            })
            .catch(err => {
                return res.status(500).json({ 'errors': ['Error deleting template'] })
            })
    },

    async result(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '%%';
        let sort = ['createdAt', 'DESC'];
        var filter = '';
        if (req.query.limit != undefined) {
            limit = parseInt(req.query.limit);
        }
        if (req.query.search) {
            search = '%' + req.query.search + '%';
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        if (req.query.sort) {
            if (req.query.sort == 'name') {
                sort = ['name'];
            }
        }
        if (req.query.filterBy != 'all')
            filter = req.query.filterBy;

        if (req.query.type == 'CSV') {
            var uaParser = require('ua-parser-js');
            var ip = req.headers["x-real-ip"] || req.ip;
            var detail = {};
            detail.user_agent = req.headers['user-agent'];
            detail.systemDetail = uaParser(req.headers['user-agent']);
            detail.ip = ip;
            var info = { info: detail };
            var metaData = JSON.stringify(info);
            var username = req.user.User.firstName + ' ' + req.user.User.lastName;
            var userId = req.user.User.id;
            var companyId = req.user.companyId;
            var type = 'campaign';
            var text = 'CSV file of Campaign Result is downloaded by ' + username;
            db.RecentActivity.create({ userId, companyId, type, text, metaData });
        }
        db.Campaign.findOne({ where: { id: req.query.campaignId, companyId: req.user.companyId } })
        .then(campaign => {
            if (campaign) {
                return db.Campaign.findOne({
                    include: [
                        {
                            model: db.CampaignCourse, include: [{ model: db.Course, attributes: ['id', 'name'] }, { model: db.SimulationReminder, attributes: ['type', 'scheduledAt', 'percent', 'executedAt'] }]
                        }, {
                            model: db.CampaignQuiz, include: [{ model: db.Quiz, attributes: ['id', 'name'] }, { model: db.SimulationReminder, attributes: ['type', 'scheduledAt', 'percent', 'executedAt'] }]
                        }, {
                            model: db.CampaignGroup, include: [{ model: db.Group }]
                        }, {
                            model: db.Simulation
                        }],
                    where: { id: campaign.id }
                });
            } else
                throw new RequestError('Campaign not found');
        })
        .then(campaignData => {
            if (campaignData)
                return db.Session.findAndCountAll({
                    where: {
                        status: (filter == '' ? ['Pending', 'Email Sent', 'Email Opened', 'Link Clicked', 'Error', 'Unsubscribed'] : filter), simulationId: campaignData.Simulations[0].id, $or: [
                            Sequelize.literal("`Session`.`firstName` LIKE '%" + search + "%'"),
                            Sequelize.literal("`Session`.`lastName` LIKE '%" + search + "%'"),
                            Sequelize.literal("`Session`.`email` LIKE '%" + search + "%'"),
                        ]
                    }

                }).then(sessionCount => [sessionCount, campaignData])
            else
                throw new RequestError('No Data found');
        })
        .then(([sessionCount, campaignData]) => {
            if (sessionCount) {
                let pages = Math.ceil(sessionCount.count / limit);
                offset = limit * (page - 1);
                if (req.query.type == undefined) {
                    return db.Session.findAll({
                        include: [{ model: db.Member, required: true }],
                        where: {
                            status: (filter == '' ? ['Pending', 'Email Sent', 'Email Opened', 'Link Clicked', 'Error', 'Unsubscribed'] : filter), simulationId: campaignData.Simulations[0].id, $or: [
                                Sequelize.literal("`Session`.`firstName` LIKE '%" + search + "%'"),
                                Sequelize.literal("`Session`.`lastName` LIKE '%" + search + "%'"),
                                Sequelize.literal("`Session`.`email` LIKE '%" + search + "%'"),
                            ]
                        },
                        limit: limit,
                        offset: offset
                    })
                        .then(r => [r, sessionCount, pages, campaignData])
                } else {
                    return db.Session.findAll({
                        include: [{ model: db.Member, required: true }],
                        where: {
                            status: (filter == '' ? ['Pending', 'Email Sent', 'Email Opened', 'Link Clicked', 'Error', 'Unsubscribed'] : filter), simulationId: campaignData.Simulations[0].id, $or: [
                                Sequelize.literal("`Session`.`firstName` LIKE '%" + search + "%'"),
                                Sequelize.literal("`Session`.`lastName` LIKE '%" + search + "%'"),
                                Sequelize.literal("`Session`.`email` LIKE '%" + search + "%'"),
                            ]
                        },
                    })
                        .then(r => [r, sessionCount, pages, campaignData])
                }
            }
        })
        .then(([campaign, sessionCount, pages, campaignData]) => {
            if (campaign) return db.SessionEvent.findAll({ attributes: [[db.sequelize.fn('DISTINCT', db.sequelize.col('message')), 'message'], 'sessionId'], where: { simulationId: campaignData.Simulations[0].id } }).then(r => [r, campaign, sessionCount, pages, campaignData])
            else throw new RequestError('No Data found');
        })
        .then(([events, campaign, sessionCount, pages, campaignData]) => {
            if (events) res.status(200).json({ 'data': campaignData, 'events': events, 'memberList': campaign, 'count': sessionCount.count, 'pages': pages });
            else throw new RequestError('No Data found');
        })
        .catch(err => {
            next(err);
        })
    },

    async errorResult(req, res, next) {
        let campaignName = '';
        db.Campaign.findOne({ where: { id: req.query.campaignId, companyId: req.user.companyId }, include:[{ model:db.Simulation }] })
        .then(campaign => {
            if (campaign) {
                campaignName = campaign.name;
                return db.Session.findAll({
                    attributes:['status'],
                    include: [{ model: db.Member, attributes:['firstName', 'lastName', 'email', 'phone', 'designation'], required: true }, { model:db.SessionEvent, attributes:['message', 'details'] }],
                    where: { status: req.query.filterBy, simulationId: campaign.Simulations[0].id  },
                    order:[[db.SessionEvent, 'id', 'DESC']]
                })
            } else throw new RequestError('Campaign not found');
        })
        .then(data => {
            var items = [];
            items.push([
                'Id',
                'Name',
                'Email',
                'Phone',
                'Position',
                'Status',
                'Details'
            ]);
            var count = 1;
            for(var i=0; i<data.length;i++){
                items.push([
                    count++,
                    (data[i].Member.firstName ? data[i].Member.firstName : '') +' '+(data[i].Member.lastName ?data[i].Member.lastName : ''),
                    (data[i].Member.email ? data[i].Member.email : '-'),
                    (data[i].Member.phone ?  data[i].Member.phone : '-'),
                    (data[i].Member.designation ? data[i].Member.designation : '-'),
                    (data[i].status ? data[i].status : '-'),
                    (data[i].SessionEvents && data[i].SessionEvents[0].details ? JSON.parse(JSON.stringify(data[i].SessionEvents[0].details)) : '-')
                ]);
            }
            return sendCSVforDownload(items)
        })
        .then(file => {
            var options = {
                root: appRoot + '/public/downloads/',
                dotfiles: 'deny',
                headers: {
                    'x-timestamp': Date.now(),
                    'x-sent': true,
                }
            };
            res.setHeader('Content-Disposition', 'attachment; filename=' + campaignName+'.csv');
            res.sendFile(file.fileName, options, function (err) {
                if (err) {
                    console.log(err);
                    next(err);
                } else {
                    console.log('Sent:', file.fileName);
                }
            });
        })
        .catch(err => {
            next(err);
        })
    },

    async resultCount(req, res, next) {
        var result = {}; var simulationId = 0;
        var campaignId = req.query.campaignId;
        db.Simulation.findOne({ include: [{ model: db.Campaign, where: { id: campaignId, companyId: req.user.companyId }, required: true }], })
            .then(simulation => {
                if (simulation) {
                    simulationId = simulation.id;
                    return db.Session.count({ where: { simulationId: simulationId } })
                } else throw new RequestError('Campaign not found');
            })
            .then(data => {
                result['Total'] = data;
                return db.Session.count({ where: { status: "Pending", simulationId: simulationId } })
            })
            .then(data => {
                result['Pending'] = data;
                return db.Session.count({ where: { status: "Unsubscribed", simulationId: simulationId } })
            })
            .then(data => {
                result['Unsubscribed'] = data;
                return db.Session.count({ where: { status: "Error", simulationId: simulationId } })
            })
            .then(data => {
                result['Error'] = data;
                return db.SessionEvent.findAll({ attributes: [[db.sequelize.fn('DISTINCT', db.sequelize.col('message')), 'message'], 'sessionId'], where: { simulationId: simulationId, message: 'Email Sent' } })
            })
            .then(data => {
                result['Email Sent'] = (data && data.length > 0 ? data.length : 0);
                return db.SessionEvent.findAll({ attributes: [[db.sequelize.fn('DISTINCT', db.sequelize.col('message')), 'message'], 'sessionId'], where: { simulationId: simulationId, message: 'Email Opened' } })
            })
            .then(data => {
                result['Email Opened'] = (data && data.length > 0 ? data.length : 0);
                return db.SessionEvent.findAll({ attributes: [[db.sequelize.fn('DISTINCT', db.sequelize.col('message')), 'message'], 'sessionId'], where: { simulationId: simulationId, message: 'Link Clicked' }, distinct: true })
            })
            .then(data => {
                result['Link Clicked'] = (data && data.length > 0 ? data.length : 0);
                return res.status(200).json({ success: true, data: result });
            })
            .catch(err => {
                next(err);
            })
    },

    async courseList(req, res, next) {
        var statusValues = ['Published', 'Active'];
        db.Course.findAll({ where: { status: { $in: statusValues }, companyId: req.user.companyId, courseType: 'streaming' }, order: [['id', 'DESC']] })
            .then(r => {
                if (r)
                    return res.status(200).json({ success: true, course: r });
                else
                    throw new RequestError('No Data found');
            })
            .catch(err => {
                next(err);
            })

    },
    async quizList(req, res, next) {
        var statusValues = ['Published', 'Active'];
        db.Quiz.findAll({ where: { status: { $in: statusValues }, companyId: req.user.companyId }, order: [['id', 'desc']] })
            .then(r => {
                if (r)
                    return res.status(200).json({ success: true, quizzes: r });
                else
                    throw new RequestError('No Data found');
            })
            .catch(err => {
                next(err);
            })

    },
    async fetcherror(req, res, next) {
        var simulationId = 0;
        var campaignId = 0;
        var type = '';
        db.Campaign.findOne({ where: { id: req.params.id, companyId: req.user.companyId }, include: [{ model: db.Simulation }] })
            .then(campaign => {
                if (campaign) {
                    campaignId = campaign.id;
                    simulationId = campaign.Simulations[0].id;
                    type =  campaign.Simulations[0].type;
                    return db.Session.count({ attributes: ['id', 'simulationId', 'companyId'],  where: { status: 'Error', companyId:req.user.companyId }, include: [{ model: db.Simulation, where: { campaignId: campaign.id } }] })
                } else throw new RequestError('Campaign not found');
            })
            .then(sessionCount =>{
                return db.Sender.findOne({ where:{ companyId:req.user.companyId }, include:[{ model:db.Company, attributes:['smtpStatus', 'skipLogin'], required:true }], order:[['id', 'DESC']] }).then(r =>[r, sessionCount])
            })
            .then(([sender, sessionCount]) =>{
                var delay = config.mail.delay, batchingRange = config.mail.batching;
                if(sender && sender.Company.smtpStatus == 'custom'){
                    if(sender.delay && sender.delay != null)
                        delay = sender.delay;
                    if(sender.limit && sender.limit != null)
                        batchingRange = sender.limit;
                }
                console.log(delay, batchingRange);
                var miliseconds = 0;
                for(var i=0; i < Math.ceil(sessionCount/batchingRange); i++){
                    let offset = i*batchingRange;
                    miliseconds = delay*i;
                    queue.create('session-resolve-error', {
                        campaignId: campaignId,
                        simulationId:simulationId,
                        // offset: offset,
                        limit: batchingRange,
                        sessionCount:sessionCount,
                        companyId: req.user.companyId,
                        type:type
                    }).delay(miliseconds).removeOnComplete(true).save();
                };
                return true;
            })
            .then(data => {
                if (data)
                    res.status(200).json({ 'success': true })
                else throw new RequestError('No Data found');
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },
};
