import { db } from '../../../models';
import mailer from '../../../mailer';
import { createChecksum,sanitize } from './../../../functions';
import bigblue from '../../../krademy/bigbluebutton';
import config from '../../../config';

function makeid() {
    var result = '', characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789', charactersLength = 62;
    for ( var i = 0; i < 30; i++ ) { result += characters.charAt(Math.floor(Math.random() * charactersLength)) }
    return result;
}

function replaceAll(str, find, replace) {
    return str.replace(new RegExp(find, 'g'), replace);
  }

function generate(n) {
    var add = 1,
      max = 12 - add;  
    if (n > max)
      return generate(max) + generate(n - max);  
    max = Math.pow(10, n + add);
    var min = max / 10; // Math.pow(10, n) basically 
    var number = Math.floor(Math.random() * (max - min + 1)) + min;
    return ("" + number).substring(add);
  }

export default {
    async list(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '%%';
        var langCondition = { };
        var skillCondition = { };
        let sort = ['createdAt', 'DESC'];
        if(req.query.limit != undefined){
            limit = parseInt(req.query.limit);
        }
        if (req.query.search) {
            search = '%' + req.query.search + '%';
        }
        if(req.query.language) {
            langCondition = { name : { like: req.query.language }};
        }
        if(req.query.skill) {
            skillCondition = { skillId : req.query.skill };
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        if (req.query.sort) {
            if (req.query.sort == 'name') {
                sort = ['name'];
            }
        }
        var condition = { companyId: req.user.companyId, courseType: 'streaming', name: { like: search } };
        if (req.query.fromDate && req.query.toDate) {
            let endDate = new Date(req.query.toDate);
            endDate.setDate(endDate.getDate()+1);
            condition['createdAt'] = { $between: [new Date(req.query.fromDate), endDate] }
        }
        db.Language.findOne({ where : { name : req.query.language } })
        .then(language=>{
            if(language && language.id) condition['languageId'] = language.id;
            return db.Course.findAll({
                where: condition,
                attributes: ['id'],
                include: [{
                    model: db.CourseSkill,
                    attributes: [],
                    where: skillCondition,
                }]
            })
        })
        .then((count) => {
            count = json(count).length;
            let pages = Math.ceil(count / limit);
            offset = limit * (page - 1);
            db.Course.findAll({
                attributes: ['id', 'name', 'logo' , 'description' , 'totalDuration', 'totalLecture','totalSection', 'status','courseType', 'instructorDetail','createdAt' ],
                where: condition,
                include: [
                {
                    model: db.Language,
                    attributes: ['name'],
                    where: langCondition,
                    required: true
                },{
                    model: db.CourseSkill,
                    attributes: ['skillId', 'courseId'],
                    where: skillCondition,
                    required: true,
                    include:[{ model: db.Skill, attributes: ['id', 'name']}]
                }],
                limit: limit,
                offset: offset,
                order: [sort],
            })
            .then(course => {
                var fullName = req.user.User.firstName+(req.user.User.lastName ? "+"+req.user.User.lastName : '');
                res.status(200).json({ 'course': course, 'count': count, 'pages': pages, 'userName':fullName });
            })
            .catch(function (error) {
                console.log(error)
                res.status(500).json({ 'errors': ['Error fetching course.'] });
            });
        })
        .catch(function (error) {
            console.log("TCL: list -> error", error)
            res.status(500).json({ 'errors': ['Error fetching course.'] });
        });
        
    }, 
    async fetch(req,res,next) {
        db.Course.findOne({
            where: { id: req.query.id },
            include: [{
                model: db.CourseSkill,
                required: true,
                where : {courseId : req.query.id},
                include:[{ model: db.Skill, attributes: ['id', 'name']}]
            },{
                model: db.CourseSection, 
                include: [{
                    model: db.CourseSectionLecture,
                    required : false,
                    include: [{model: db.Content },{model : db.SessionCourseLectureTracking }],
                    subQuery: false
                }]
            }],
            // subQuery: false
        })
        .then(course=>{
            return res.status(200).json({ success: true,'course' : course })
        })
        .catch(err=>{
            next(err)
        })
    },
    async fetchLanguages(req, res, next) {
        db.Language.findAll({ attributes : [ 'name' , 'id' ]} )
        .then(languages=>{
           return res.status(200).json({ languages })
        })
        .catch(e=>{
            next(e);
        })
    },
    async store(req, res, next) {
        var {  name  , description , languageName, courseType, instructorDetail } = req.body;
        var uaParser = require('ua-parser-js');
        var ip = req.headers["x-real-ip"] || req.ip;
        var detail = {};
        detail.user_agent = req.headers['user-agent'];
        detail.systemDetail = uaParser(req.headers['user-agent']);
        detail.ip = ip;
        var info = { info: detail};
        var userId = req.user.User.id;
        var type = 'course';
        var metaData = JSON.stringify(info);
        var companyId = req.user.companyId;
        var username = req.user.User.firstName+' '+req.user.User.lastName;
        var text = name+' course created by '+username;
        db.Course.findOne({ where : { name, companyId }})
        .then(course=>{
            if(course) throw new RequestError("Course name already exists.");
            return db.Language.findOne({ attributes : ['name','id'],where : { id : languageName } });
        })
        .then(language=>{
            if(language) {
                if(req.file){
                    req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                    var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                    if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                        throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                    }
                }
                name =sanitize(name);
                db.RecentActivity.create({userId, companyId, type, text, metaData});
                return db.Course.create({ name ,logo: req.file?req.file.filename:null, description , languageId : language.id, courseType : courseType? courseType:null, instructorDetail : instructorDetail? instructorDetail:null, companyId:companyId, totalLecture : 0 })
            }
            throw new RequestError("Invalid language selected.");
        })
        .then(course=>{
            if(course){
                var skills = [];
                var skill = req.body.skills.split(',');
              for(var i=0;i< skill.length; i++){
                skills.push({
                    'courseId' : course.id ,
                    'skillId' : skill[i]
                })
              }
              return db.CourseSkill.bulkCreate(skills).then(r => course);     
            } 
            else throw new RequestError('Error creating course!');
        })
        .then(course => {
            if(course)
                return res.status(200).json({ success : 1 ,id: course.id });
        })
        .catch(err=>{
            next(err);
        })
    },
    async fetchSection(req, res, next) {
        var {  courseId } = req.query;
        db.CourseSection.findAll({
            //, required: true
            include : [{model : db.CourseSectionLecture, include : [{model : db.Content}]}],
            where : { courseId : courseId } 
        })
        .then(cSection=>{
            if(cSection)
                return res.status(200).json({ success : 1 , section: cSection});
        })
        .catch(err=>{
            next(err);
        }) 
    },
    async storeSection(req, res, next) {
        var {  name  , courseId } = req.body;
        db.Course.findOne({ where : { id : courseId, companyId: req.user.companyId } })
        .then(course=>{
            if(!course)
                 throw new RequestError("Course does not exists.");
            return db.CourseSection.findOne({where:{name : name,courseId :courseId}}).then(r=>[r,course])
        })
        .then(([sectionname,course]) => {
            name=sanitize(name);
            if(!sectionname)  
               return  db.CourseSection.create( {name ,  courseId} ).then(r=>[r,course.totalSection])             
            else throw new RequestError("Section name already exists.");
        })
        .then(([section, totalSection])=>{
            if(section){
                db.Course.update({ totalSection : Math.floor(totalSection+1) },{ where : { id : courseId, companyId: req.user.companyId }})
                return res.status(200).json({ success : 1 ,sectionId : section.id});
            } 
            else throw new RequestError("server Error.");
        })
        .catch(err=>{
            next(err);
        }) 
    },
    async updateSection(req, res, next) {
        var {  name  , sectionId ,courseId } = req.body;
        db.CourseSection.findOne({ where : { id : sectionId}})
        .then(section=>{
            if(!section) throw new RequestError("Invalid Section selected.");
                return db.CourseSection.findOne({ where : {$not : { id : sectionId},name:name,courseId:courseId}})
        })
        .then(checkSectionName => {
            name=sanitize(name);
            if(!checkSectionName)
                return db.CourseSection.update({ name },{where : {id : sectionId}});
            else throw new RequestError("Section name already exists.");

        })
        .then(cSection=>{
            if(cSection) return res.status(200).json({ success : 1 });
            throw new RequestError("Server Error.");
        })
        .catch(err=>{
            next(err);
        })
    },
    async deleteSection(req, res, next) {
        var {  courseId,sectionId } = req.body;
        db.CourseSection.findOne({ where : { id : sectionId } })
        .then(cSection=>{
            if(cSection)
               return db.CourseSectionLecture.findOne({where : {courseSectionId: sectionId}});                
        })
        .then(lectureCount => {
            if(!lectureCount){
                return db.CourseSection.destroy({where : {id: sectionId}});   
            }else throw new RequestError("Section containing lectures cannot be deleted.");
        })
        .then(result =>{
            return db.Course.findOne({where: {id : courseId, companyId: req.user.companyId}})
        })
        .then(courseData => {
            var data = JSON.parse(JSON.stringify(courseData));
            if(courseData)
                return db.Course.update({ totalSection : Math.floor(data.totalSection-1) },{ where : { id : courseId, companyId: req.user.companyId}})
            else throw new RequestError("Error deleting section.");
        })
        .then(r =>{
            return res.status(200).json({ success : 1 , data: 'Section Deleted'});
        })
        .catch(err=>{
            next(err);
        }) 
    },
    async update(req, res, next) {
        var {  name  , description , languageName, courseType, instructorDetail } = req.body;
        var meetingId ;
        var expired ;
        var instructorData;
        if(courseType == 'instructor')
            instructorData = JSON.parse(instructorDetail);
        db.Course.findOne({ where : { id : req.params.id, companyId: req.user.companyId }})
        .then(course =>{
            if(JSON.parse(course.instructorDetail) && courseType == 'instructor'){
                meetingId = JSON.parse(course.instructorDetail).meetingId;
                expired = JSON.parse(course.instructorDetail).expired;
            }
            if(!course) throw new RequestError('Invalid Course Exists.');
            return db.Course.findOne({ where: { name: req.body.name, companyId: req.user.companyId, id: { $not : req.params.id }}}).then(r => [r, course])
        })
        .then(([result, course]) => {
            if(result) throw new RequestError('Course with same name exists.');
            return db.Language.findOne({ where : { id : languageName } }).then(r => [r, course])
        })
        .then(([clanguage, course])=>{
            if(!clanguage) throw new RequestError("Invalid language selected.");
            if(req.file){
                req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                    throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                }
            }
            return db.CourseSkill.destroy({where : {courseId : course.id}}).then(r => [clanguage, course])    
        })
        .then(([clanguage, course]) => {
            var skills = [];
            var skill = req.body.skills.split(',');
            for(var i=0;i< skill.length; i++)
                skills.push({ 'courseId' : course.id, 'skillId' : skill[i] });
            return db.CourseSkill.bulkCreate(skills, { updateOnDuplicate: ["courseId", "skillId"] }).then(r => clanguage);
        })
        .then(clanguage => {
            if(req.file)
                return db.Course.update({logo: (req.file.filename ? sanitize(req.file.filename) : null)},{where : {id:req.body.courseId, companyId: req.user.companyId}});
            if(meetingId != undefined){
                instructorData["meetingId"] = meetingId;
                instructorData["expired"] = expired;
                instructorData = JSON.stringify(instructorData)
            }else
                instructorData = instructorDetail
           return db.Course.update({ name: sanitize(name), languageId: clanguage.id , description : description,instructorDetail : instructorData},{where : {id:req.body.courseId, companyId: req.user.companyId}})
        })
        .then(course=>{
            return res.status(200).json({ success : true ,data : "Course Updated"})
        })
        .catch(err => {
            next(err);
        })
    },

    async storeSectionLecture(req,res,next){
        var { courseId, sectionId, name, description} = req.body;
        db.CourseSection.findOne({ where : { id : sectionId}})
        .then(courseSection=>{
            if(courseSection){
                return db.CourseSection.findAll({attributes:['id'],where:{courseId :courseId}})
            }
            else throw new RequestError("Invalid Section Id.");
        })
        .then(courseSectionList=>{
            var courseSectionIds = [];
            for(var i=0; i<courseSectionList.length; i++){
                courseSectionIds.push(courseSectionList[i].id);
            }
            if(courseSectionList){
                return db.CourseSectionLecture.findOne({ where : {courseSectionId : {$in : courseSectionIds}, name : name}})
            }
            else throw new RequestError("Invalid Section Id.");
        })
        .then(checkLectureName => {
            if(!checkLectureName)
                return db.CourseSectionLecture.create({
                        courseSectionId:sectionId,
                        name:sanitize(name),
                        description:(description != '' ? description : null)
                    });
            else throw new RequestError("Lecture name already exists.");
        })
        .then(courseSectionLecture => {
            if(courseSectionLecture)
                return db.CourseSection.findOne({where : {id : sectionId}})
                .then(CourseSection=>[CourseSection,courseSectionLecture])
        })
        .then(([CourseSection,courseSectionLecture])=>{
            return db.Course.findOne({where : {id : CourseSection.courseId, companyId: req.user.companyId}})
                .then(Course=>[Course,CourseSection,courseSectionLecture])
        })
        .then(([Course,CourseSection,courseSectionLecture])=>{
            if(courseSectionLecture){
                db.CourseSection.update({totalLecture :Math.floor(CourseSection.totalLecture+1)},{where : {id : sectionId}});
                db.Course.update({totalLecture :Math.floor(Course.totalLecture+1)},{where : {id : Course.id, companyId: req.user.companyId}});
                return res.status(200).json({ success : 1, lectureId : courseSectionLecture.id});
            }
            else throw new RequestError("Failed to create.");
        })
        .catch(err=> {
            next(err);
        })        
    },
    async updateSectionLecture(req,res,next){
        var { courseId, lectureId , name } = req.body;
        db.CourseSectionLecture.findOne({ where : { id : lectureId}})
        .then(courseSection=>{
            if(courseSection){
                return db.CourseSection.findAll({attributes:['id'],where:{courseId :courseId}})
            }
            else throw new RequestError("Invalid Section Id.");
        })
        .then(courseSectionList=>{
            var courseSectionIds = [];
            for(var i=0; i<courseSectionList.length; i++){
                courseSectionIds.push(courseSectionList[i].id);
            }
            if(courseSectionList){
                return db.CourseSectionLecture.findOne({ where : {$not : {id : lectureId},courseSectionId : {$in : courseSectionIds}, name : name}})
            }
            else throw new RequestError("Invalid Lecture Id.");
        })
        .then(checkLectureName => {
            if(!checkLectureName)
                 return db.CourseSectionLecture.update({
                    name:sanitize(name),
                    description:(req.body.description ? req.body.description : null)
                },{where : {id : lectureId}});
            else throw new RequestError("Lecture name already exists.");

        })
        .then((courseSectionLecture)=>{
            if(courseSectionLecture)
            return res.status(200).json({ success : 1, data : 'Lecture Updated'});
            else throw new RequestError("Failed to Update.");
        })
        .catch(err=> {
            next(err);
        })        
    },
    async deleteLecture(req, res, next) {
        var { sectionId,lectureId,courseId} = req.body;
        var courseTotalDuration;
        db.CourseSectionLecture.findOne({ where : { id : lectureId } })
        .then(cSection=>{
            if(cSection)
                return db.CourseSectionLecture.destroy({where : {id: lectureId}}).then(r => cSection)
            
        })
        .then(cSection => {
            return db.CourseSection.findOne({where: {id : sectionId}}).then(r => [r, cSection])
        })
        .then(([section, cSection])=>{
            return db.Course.findOne({where: {id : section.courseId, companyId: req.user.companyId}}).then(course => [course,section, cSection])
        })
        .then(([course,section, cSection]) => {
            if(section){
                db.Course.update({totalLecture :Math.floor(course.totalLecture-1)},{where : {id : course.id, companyId: req.user.companyId}});
                return  db.CourseSection.update({totalDuration:  Math.floor(section.totalDuration-cSection.totalDuration), totalLecture : Math.floor(section.totalLecture-1) },{ where : { id : sectionId }}).then(r => [r,cSection])
            }
        }) 
        .then((updateDuration) =>{
            return db.CourseSection.findAll({
                attributes: [
                [db.sequelize.fn('SUM', db.sequelize.col('CourseSection.totalDuration')), 'durationSum'],
            ],
            where : {courseId : courseId}})
        })
        .then((courseDuration) =>{
            var data = JSON.parse(JSON.stringify(courseDuration));
            if(data && data.length > 0){
                courseTotalDuration = data[0].durationSum;
                return db.Course.update({totalDuration : data[0].durationSum},{where : {id : courseId, companyId: req.user.companyId}})
            }
        })
        .then(r =>{
            return db.SessionCourseLectureTracking.destroy({ where :{ courseSectionLectureId : lectureId } })
        })
        .then(course => {           
            return db.MemberCourse.findAll({
                attributes : ['id','memberId','sessionId'],
                group: ['id'],
                 include: [{
                   model: db.Session, attributes : ['id'],
                    required: true,
                    include:[{
                        model: db.SessionCourseLectureTracking ,attributes : [[db.sequelize.literal("(select sum(duration) from SessionCourseLectureTrackings where SessionCourseLectureTrackings.sessionId = Session.id AND SessionCourseLectureTrackings.deletedAt IS NULL)"), 'durationSum']]                
                    }]
                 }],
                 where: { courseId: courseId} });
        })
        .then(allData => {
            var data = JSON.parse(JSON.stringify(allData))
            if(data){
                data.reduce((a,c)=>{
                    var percent = 0;
                    if(c.Session.SessionCourseLectureTrackings[0] && c.Session.SessionCourseLectureTrackings[0].durationSum){
                        percent = Math.round((c.Session.SessionCourseLectureTrackings[0].durationSum / courseTotalDuration) * 100);
                        if (percent >= 100){ 
                            percent = 100;
                            db.MemberCourse.update({percent : percent},{where : {id : c.id}})
                        }else db.MemberCourse.update({percent : percent, completedAt: null, issuedDate:null },{where : {id : c.id}})
                   }else db.MemberCourse.update({percent : percent, completedAt: null, issuedDate:null },{where : {id : c.id}})
                },Promise.resolve());
            }
        })
        .then(updateCourseDuration => {
             return res.status(200).json({ success : 1 , data: 'Lecture Deleted'});
        })
        .catch(err=>{
            next(err);
        }) 
    },
    async uploadLectureFile(req, res, next){
        var {sectionId, type ,courseId ,duration, thumbnail} = req.body;
        var companyId = req.user.companyId;
        var thumbnail = (type =="video" ? req.files.thumbnail[0].filename : null);
        var duration = (type =="video" ? req.body.duration : 60);
        var key = makeid();
        var courseTotalDuration;
        db.CourseSectionLecture.findOne({ where : { id: req.body.lectureId} })
        .then(lecture => {
            if(lecture){
                if(req.files && req.files.filename[0]){
                    req.files.filename[0].originalname = sanitize((req.files.filename[0].originalname ? req.files.filename[0].originalname : ''));
                    var fileExt=req.files.filename[0].originalname.split('.').pop().toLowerCase();
                    var ft = ''; var qt = ''
                    if(req.body.type == 'pdf')
                        ft = 'pdf';
                    if((req.body.type == 'doc'  && fileExt =='doc')  || ( req.body.type == 'doc' && fileExt =='docx'))
                        ft = fileExt;
                    if((req.body.type == 'xls' && fileExt =='xls') || (req.body.type == 'xls' && fileExt =='xlsx'))
                        ft = fileExt;
                    if((req.body.type == 'ppt' && fileExt =='ppt') || (req.body.type == 'ppt' && fileExt =='pptx'))
                        ft = fileExt;
                    if(req.body.type == 'video' && fileExt =='mp4')
                        ft = 'mp4';
                    if (fileExt != ft) 
                        throw new RequestError('Only .'+req.body.type+' file format allowed!',403); 
                }
                if(lecture.type == 'quiz')
                    return  db.CourseLectureQuestion.destroy({ where : { courseSectionLectureId : req.body.lectureId } }).then(r=> lecture);
                else lecture;
            }else throw new RequestError("Invalid lecture received.!");
        })
        .then(lecture => {
            return db.Content.create({
                type:req.body.type,
                duration:duration,
                mimeType:req.files.filename[0].mimetype,
                originalName:req.files.filename[0].originalname,
                name:req.files.filename[0].filename,
                thumbnail:thumbnail,
                companyId:companyId,
                key:key
            })
        })
        .then(content => {
            if(content)
                return db.CourseSectionLecture.update({contentId : content.id, type : req.body.type, totalDuration:duration},{where : {id : req.body.lectureId}})
            else throw new RequestError('Failed To Add Content');
        })
        .then(courseSectionLecture => {
            if(courseSectionLecture){
                return  db.SessionCourseLectureTracking.destroy({where : {courseSectionLectureId : req.body.lectureId}});    
            }
        })
        .then(data => {
            return db.CourseSectionLecture.findAll({
                attributes: [
                [db.sequelize.fn('SUM', db.sequelize.col('CourseSectionLecture.totalDuration')), 'durationSum'],
            ],
            where : {courseSectionId : sectionId}})
        })
        .then((sectionDuration) =>{
            var data = JSON.parse(JSON.stringify(sectionDuration));
            if(data && data.length > 0){
                return db.CourseSection.update({totalDuration : data[0].durationSum},{where : {id : sectionId}})
            }
        })
        .then((updateDuration) =>{
            return db.CourseSection.findAll({
                attributes: [
                [db.sequelize.fn('SUM', db.sequelize.col('CourseSection.totalDuration')), 'durationSum'],
            ],
            where : {courseId : courseId}})
        })
        .then((courseDuration) =>{
            var data = JSON.parse(JSON.stringify(courseDuration));
            if(data && data.length > 0){
                courseTotalDuration = data[0].durationSum;
                return db.Course.update({totalDuration : data[0].durationSum},{where : {id : courseId, companyId: companyId}})
            }
        })
        .then(course => {           
            return db.MemberCourse.findAll({
                attributes : ['id','memberId','sessionId'],
                group: ['id'],
                 include: [{
                   model: db.Session, attributes : ['id'],
                    required: true,
                    include:[{
                        model: db.SessionCourseLectureTracking ,attributes : [[db.sequelize.literal("(select sum(duration) from SessionCourseLectureTrackings where SessionCourseLectureTrackings.sessionId = Session.id AND SessionCourseLectureTrackings.deletedAt IS NULL)"), 'durationSum']]                
                    }]
                 }],
                 where: { courseId: courseId} });
        })
        .then(allData => {
            var data = JSON.parse(JSON.stringify(allData))
            if(data){
                data.reduce((a,c)=>{
                    var percent = 0;
                    if(c.Session.SessionCourseLectureTrackings[0] && c.Session.SessionCourseLectureTrackings[0].durationSum){
                        percent = Math.round((c.Session.SessionCourseLectureTrackings[0].durationSum / courseTotalDuration) * 100);
                        if (percent >= 100){ 
                            percent = 100;
                            db.MemberCourse.update({percent : percent},{where : {id : c.id}})
                        }else db.MemberCourse.update({percent : percent, completedAt: null, issuedDate:null },{where : {id : c.id}})
                   }else db.MemberCourse.update({percent : percent, completedAt: null, issuedDate:null },{where : {id : c.id}})
                },Promise.resolve());
            }

        })
        .then(data => {
            return res.status(200).json({ success : 1 , data: 'File Uploaded Successfully', filename : req.files.filename[0].filename});
        })
        .catch(err => {
            next(err);
        })

    },

    // async storeSectionLecture(req,res,next){
    //     console.log(req.body);
    //     var { courseId , sectionId , name , contentId ,type} = req.body;
    //     db.Content.findOne({ where : { id : contentId}})
    //     .then(content=>{
    //         if(content)return db.Course.findOne({ where : { id : courseId}}).then(r=>[r,r.totalDuration,content.duration]);
    //         else throw new RequestError("Invalid Course selected.");
    //     })
    //     .then(([course,courseDuration,contentDuration])=>{
    //         if(course) return db.CourseSection.findOne({ where : { id : sectionId }}).then(r=>[r,r.totalDuration,r.totalLecture,courseDuration,contentDuration]);
    //         else throw new RequestError("Invalid Course selected.");
    //     })
    //     .then(([content,totalLecture,sectionDuration,courseDuration,contentDuration])=>{
    //         if(content) return db.CourseSectionLecture.create({ courseSectionId : sectionId , name, type , contentId ,totalDuration : contentDuration }).then(r=>[r,sectionDuration,totalLecture,courseDuration,contentDuration])
    //         else throw new RequestError("Invalid Course Section selected.");
    //     })
    //     .then(([lecture,sectionDuration,totalLecture,courseDuration,contentDuration])=>{
    //         db.Course.update({totalDuration:Math.floor(courseDuration+contentDuration) },{ where : { id : contentId}});
    //         db.CourseSection.update({ totalDuration : Math.floor(sectionDuration+contentDuration) , totalLecture: Math.floor(totalLecture +1 )},{where: { id : sectionId }})
    //         return res.status(200).json({ success : 1, data : lecture});
    //     })
    // },

    async fetchMemberCourse(req,res,next) {
        var coursePercent = 0;
        var memberCourseId = '';
        db.MemberCourse.findOne({ where : { courseId : req.query.id, memberId:req.user.memberId }, include:[ { model : db.Course, required: true} ]  })
        .then(course => {
            if(course){
                memberCourseId = JSON.parse(JSON.stringify(course)).id;
                coursePercent = JSON.parse(JSON.stringify(course)).percent;
                return db.Session.findOne({ where : { sessionKey: req.query.sKey } })
            }
            else throw new RequestError("Course is not Assigned.");
        })
        .then(session => {
            if(session)
                return db.Course.findOne({
                    where: { id: req.query.id },
                    include: [{
                        model: db.CourseSection, 
                        include: [{
                            model: db.CourseSectionLecture,
                            required : false,
                            include: [{model : db.CourseUserQuizDetail, attributes:['courseSectionLectureId', 'percent', 'status'], where : { sessionId : session.id}, required: false }, {model: db.Content },{model : db.SessionCourseLectureTracking, where : { sessionId : session.id}, required: false }, {model: db.CourseLectureQuestion, include:[{ model: db.CourseQuizQuestion, include:[{ model: db.CourseQuizQuestionOption, attributes:['id', 'title'] }] }] }, ],
                            subQuery: false
                        }]
                    },{model : db.Language},{model : db.CourseSkill,include : {model : db.Skill} },  ],
                    order:[[db.CourseSection, db.CourseSectionLecture, 'id', 'ASC']],
                }).then(r => [r, session])
            else throw new RequestError("Invalid Token received.");
        })
        .then(([result, session])=>{
            return res.status(200).json({ success: true,'course' : result, coursePercent :coursePercent, memberCourseId : memberCourseId, userName:session.firstName+(session.lastName ? "+"+session.lastName : '')})
        })
        .catch(err=>{
            next(err)
        })
    },

    async previewCourse(req,res,next) {
        if(req.query.id > 0)
            db.Course.findOne({
                where: { id: req.query.id },
                include: [{
                    model: db.CourseSection,
                    include: [{
                        model: db.CourseSectionLecture,
                        required : false,
                        include: [{model: db.Content },{model: db.CourseLectureQuestion, include:[{ model: db.CourseQuizQuestion, include:[{ model: db.CourseQuizQuestionOption, attributes:['id', 'title','isCorrect'] }] }] } ],
                        subQuery: false
                    },]
                },{model : db.Language},{model : db.CourseSkill,include : {model : db.Skill}}]
            })
            .then(result=>{
                return res.status(200).json({ success: true, 'course' : result })
            })
            .catch(err=>{
                next(err)
            })
        else throw new RequestError("Invalid Data received.");
    },
    
    async tracker(req, res, next) {
        var date = '';
        var completedAt = '';
        var memberCourseId = '';
        var { courseSectionLectureId, courseId, duration } = req.body;
        db.MemberCourse.findOne({ where: { courseId: courseId, memberId: req.user.memberId } })
            .then(memberCourse => {
            if (memberCourse){
                memberCourseId = JSON.parse(JSON.stringify(memberCourse)).id;
                return db.CourseSectionLecture.findOne({ where: { id: courseSectionLectureId } }).then(r => [r.totalDuration, memberCourse])
            } 
            else throw new RequestError("Invalid Course selected.");
        })
        .then(([totalDuration, memberCourse]) => {
            if (totalDuration) return db.SessionCourseLectureTracking.findOne({ where: { sessionId: memberCourse.sessionId, courseSectionLectureId: courseSectionLectureId } }).then(r => [r, totalDuration, memberCourse])
            else throw new RequestError("Invalid lecture selected.");
        })
        .then(([session, totalDuration, memberCourse]) => {
            var percent = 0;
            if(totalDuration){
                percent = Math.floor((duration / totalDuration) * 100);
                if (percent >= 100) {percent = 100;}
            }
            if (session) {
                var oldDuration = session.duration;
                if (oldDuration <= duration) 
                    return db.SessionCourseLectureTracking.update({ duration: Math.floor(duration), percent: percent }, { where: {  courseSectionLectureId: courseSectionLectureId , sessionId: memberCourse.sessionId} }).then(r => [r, percent, memberCourse])
                else{
                    percent = Math.floor(oldDuration/totalDuration*100);
                    if (percent >= 100) {percent = 100;}
                    return [1, percent, memberCourse];
                }
            }else 
                return db.SessionCourseLectureTracking.create({ duration: Math.floor(duration), sessionId: memberCourse.sessionId, percent: percent, courseSectionLectureId: courseSectionLectureId }).then(r => [r, percent, memberCourse])
        })
        .then(([tracking, percent , memberCourse]) => {
            return db.Course.findOne({ where: { id: courseId } }).then(r => [r, percent, memberCourse]);
        })
        .then(([course, percent, memberCourse]) => {
            return db.sequelize.query('select SUM(slt.duration) as duration from CourseSections cs, CourseSectionLectures l, SessionCourseLectureTrackings slt where cs.id = l.courseSectionId and l.id = slt.courseSectionLectureId and cs.courseId = '
            +courseId+' and slt.sessionId = '+ memberCourse.sessionId+ ' and cs.deletedAt IS NULL and l.deletedAt IS NULL and slt.deletedAt IS NULL', { type: db.sequelize.QueryTypes.SELECT}).then(r => [r, course, percent])
        })
        .then(([totalDurations, course, percent]) => {
            if(totalDurations){
                var tduration = totalDurations[0].duration;
                var coursePercentage = Math.floor(tduration/course.totalDuration*100);
                if (coursePercentage >= 100){
                    coursePercentage = 100;
                    date = new Date();
                    completedAt = date.getFullYear()+'-'+(date.getMonth()+1)+'-'+date.getDate()
                    mailer.sendCourseCompletion(coursePercentage, completedAt, courseId, req.user.memberId, 'completion')
                    return db.MemberCourse.update({ percent: 100}, { where: { courseId: courseId, memberId: req.user.memberId } }).then(r => [percent,coursePercentage]);
                }else 
                    return db.MemberCourse.update({ percent: coursePercentage}, { where: { courseId: courseId, memberId: req.user.memberId } }).then(r => [percent,coursePercentage]);
            }
        })
        .then(([percent,coursePercentage]) => {
            return res.status(200).json({ percent: percent,coursePercentage : coursePercentage,memberCourseId : memberCourseId})
        })
        .catch(err => {
            console.log("TCL: tracker -> err", err)
            next(err);
        })
    },

    async checkCurriculum(req, res, next){
        var courseType;
        db.Course.findOne({where: {id : req.query.courseId}})
        .then(course => {
            if(course){
                courseType = course.courseType;
                return db.CourseSection.findOne({where: {courseId : req.query.courseId}})
            }
        })
        .then(CourseSectionData => {
            if(CourseSectionData){
                return db.CourseSectionLecture.findOne({where : {courseSectionId : CourseSectionData.id}}).then(r => [r, CourseSectionData]);
            }else return [false, false];
        })
        .then(([CourseLectureData, CourseSectionData]) => {
            if(CourseLectureData && courseType){
                if(courseType == "streaming" || courseType == null){
                    if(CourseLectureData.type == 'quiz'){
                        return db.CourseSectionLecture.findOne({where : {courseSectionId : CourseSectionData.id, $not : {  questionCount: 0, } }});
                    }else  return db.CourseSectionLecture.findOne({where : {courseSectionId : CourseSectionData.id, $not : {  contentId: null, } }});
                }else return CourseLectureData;
            }else return false;
        })
        .then(CourseSectionLectureData => {
            if(CourseSectionLectureData)
                return res.status(200).json({"success" : true})
            return res.status(200).json({"success" : false})          
        })
        .catch(err => {
            next(err);
        })
    },

    async createMeeting(req, res, next){
        db.Course.findOne({where : {id : req.body.courseId, companyId: req.user.companyId}})
        .then(course => {
            if(course){
                var instructorDetail = JSON.parse(course.instructorDetail);
                if(instructorDetail.expired == 0){
                     throw new RequestError("Meeting not expired yet")
                }
                var meetingId = '';
                var name = course.name;
                name = name.replace(/ /g, "+");
                var instructorData = (course.instructorDetail ? JSON.parse(course.instructorDetail): null);
                meetingId = instructorData.meetingId;
                if(instructorData.meetingId == undefined)
                    meetingId = 'LMS-'+generate(8);
                var num = Math.floor(Math.random() * (99999 - 10000 + 1)) + 10000;
                var data = "attendeePW=ap&meetingID="+meetingId+"&moderatorPW=mp&name="+name+"&voiceBridge="+num+"&welcome=%3Cbr%3EWelcome+to+%3Cb%3E"+name+"%3C%2Fb%3E%21";
                data = data+'&logoutURL='+encodeURIComponent(config.app.url+'/bigbluebutton/sessionExpired');
                data = data+'&meta_endCallbackUrl='+encodeURIComponent(config.app.url+'/meetingEnd?meetingId='+meetingId+'&courseId='+req.body.courseId);
                var instructorSetting= JSON.parse(req.body.settings);
                if(instructorSetting.muteOnStart)
                    data = data+'&muteOnStart=true';
                if(instructorSetting.guestPolicy)
                    data = data+'&guestPolicy=ASK_MODERATOR';
                var checksum = createChecksum('create', data);
                data = data+'&checksum='+checksum;
                return bigblue.createMeeting(data).then(r => [course, meetingId])
            }else throw new RequestError("Invalid Course");
        })
        .then(([course, meetingId]) => {
            var instructorData = (course.instructorDetail ? JSON.parse(course.instructorDetail): null);
            if(course.courseType == "instructor" && instructorData.meetingId == undefined){
                instructorData["meetingId"] = meetingId;
                instructorData["expired"] = 0;
            }
            instructorData["expired"] = 0;
            return db.Course.update({setting : req.body.settings, instructorDetail : JSON.stringify(instructorData)},{where : {id : req.body.courseId, companyId: req.user.companyId }, returning:true })
        })
        .then(courseUpdate => {
            if(courseUpdate)
                return res.status(200).json({"success" : true});
            else
                throw new RequestError("Failed . Try again !");
        })
        .catch(err => {
            next(err);
        })
     },

    async settings(req, res, next){
        db.Course.findOne({where : {id : req.query.courseId, companyId: req.user.companyId}})
        .then(course => {
            if(course){
                var instructorDetail = JSON.parse(course.instructorDetail);
               if(course.courseType == 'instructor' && (instructorDetail.expired == undefined || instructorDetail.expired == 1)){
                    throw new RequestError("Create meeting first")
               }else
                return db.Course.update({setting : req.query.settings, 'status' : req.query.status},{where : {id : req.query.courseId, companyId: req.user.companyId }, returning:true });
           }else throw new RequestError("Invalid Course");
       })
       .then(courseUpdate => {
           if(courseUpdate)
             return res.status(200).json({"success" : 1});
            else
             throw new RequestError("Failed . Try again !");
       })
       .catch(err => {
           next(err);
       })
    },

    async courseStatus(req, res, next){
        db.Course.findOne({where : {id : req.query.courseId, companyId: req.user.companyId}})
        .then(course => {
            if(course){
              return db.Course.update({status : req.query.status},{where : {id : req.query.courseId, companyId: req.user.companyId}})
            }else throw new RequestError("Invalid Course");
        })
        .then(courseUpdate => {
            if(courseUpdate)
              return res.status(200).json({"success" : 1});
             else
              throw new RequestError("Failed . Try again !");
        })
        .catch(err => {
            next(err);
        })
    },

    async storeQuestion(req, res, next) {
        var sectionId;  var courseId; var courseTotalDuration;
        db.CourseSectionLecture.findOne({ where : { id: req.body.lectureId}, required:true, include:[{ model: db.CourseSection }] })
        .then(lecture => {
            if(lecture){
                courseId = lecture.CourseSection.courseId;
                sectionId = lecture.CourseSection.id;
                return db.CourseQuizQuestion.findOne({where: { title: req.body.title, companyId:req.user.companyId } }).then(r => [r, lecture])
            }else throw new RequestError("Invalid lecture received.!");
        })
        .then(([checkSameName, lecture]) => {
            /* if(!checkSameName){ */
                var { title } = req.body;
                if(req.file){
                    req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                    var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                    if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                        throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                    }
                }
                var data = { title: sanitize(title), marks:5, thumbnail: (req.file ? req.file.filename : null), companyId:req.user.companyId}
                return db.CourseQuizQuestion.create(data).then(r => [r, lecture]);
            /* }else throw new RequestError("Question title already exists.!"); */
        })
        .then(([quizQuestion, lecture]) => {
            if(quizQuestion){
                var options = JSON.parse(req.body.options);
                var opts = [];
                if(options.length > 0){
                    for(var i=0;i < options.length; i++)
                        opts.push({ 'courseQuizQuestionId' : quizQuestion.id, 'title' : sanitize(options[i].title), isCorrect: options[i].isCorrect })
                    return db.CourseQuizQuestionOption.bulkCreate(opts).then(r => [r, lecture, quizQuestion]);  
                }else throw new RequestError("Invalid Option data received.!");
            }else throw new RequestError("Error creating question!");
        })
        .then(([quizQuestionOption, lecture, quizQuestion]) => {
            if(quizQuestionOption)
                return db.CourseLectureQuestion.create({ courseSectionLectureId: lecture.id, courseQuizQuestionId:quizQuestion.id }).then(r => [r, lecture])
            else throw new RequestError("Error creating question option!");
        })
        .then(([saveLectureQuestion, lecture]) => {
            if(saveLectureQuestion){
                var questionCount = lecture.questionCount + 1;
                return db.CourseSectionLecture.update({ type: 'quiz', questionCount:questionCount, totalDuration: questionCount, contentId: null }, { where :{ id: lecture.id}, returning:true }).then(r => [r, questionCount])
            }else throw new RequestError("Error creating question lecture!");
        })
        .then(([courseSectionLecture, questionCount]) => {
            if(courseSectionLecture){
                return  db.SessionCourseLectureTracking.destroy({where : {courseSectionLectureId : req.body.lectureId}}).then(r => questionCount)  
            }
        })
        .then(questionCount => {
            return db.CourseSectionLecture.findAll({
                attributes: [
                [db.sequelize.fn('SUM', db.sequelize.col('CourseSectionLecture.totalDuration')), 'durationSum'],
            ],
            where : {courseSectionId : sectionId}}).then(r => [r, questionCount]) 
        })
        .then(([sectionDuration, questionCount]) =>{
            var data = JSON.parse(JSON.stringify(sectionDuration));
            if(data && data.length > 0){
                return db.CourseSection.update({totalDuration : data[0].durationSum},{where : {id : sectionId}}).then(r => questionCount)
            }
        })
        .then(questionCount =>{
            return db.CourseSection.findAll({
                attributes: [
                [db.sequelize.fn('SUM', db.sequelize.col('CourseSection.totalDuration')), 'durationSum'],
            ],
            where : {courseId : courseId}}).then(r => [r, questionCount]) 
        })
        .then(([courseDuration, questionCount]) =>{
            var data = JSON.parse(JSON.stringify(courseDuration));
            if(data && data.length > 0){
                courseTotalDuration = data[0].durationSum;
                return db.Course.update({totalDuration : data[0].durationSum},{where : {id : courseId, companyId: req.user.companyId}}).then(r => questionCount)
            }else return questionCount;
        })
        .then(questionCount => {           
            return db.MemberCourse.findAll({
                attributes : ['id','memberId','sessionId'],
                group: ['id'],
                include: [{
                    model: db.Session, attributes : ['id'],
                    required: true,
                    include:[{
                        model: db.SessionCourseLectureTracking ,attributes : [[db.sequelize.literal("(select sum(duration) from SessionCourseLectureTrackings where SessionCourseLectureTrackings.sessionId = Session.id AND SessionCourseLectureTrackings.deletedAt IS NULL)"), 'durationSum']]                
                    }]
                }],
                where: { courseId: courseId} }).then(r => [r, questionCount]) 
        })
        .then(([allData, questionCount]) => {
            var data = JSON.parse(JSON.stringify(allData))
            if(data.length > 0){
                return new Promise ((resolve, reject) =>{
                    data.reduce((a,c)=>{
                        var percent = 0;
                        if(c.Session.SessionCourseLectureTrackings[0] && c.Session.SessionCourseLectureTrackings[0].durationSum){
                            percent = Math.round((c.Session.SessionCourseLectureTrackings[0].durationSum / courseTotalDuration) * 100);
                            if (percent >= 100){ 
                                percent = 100;
                                db.MemberCourse.update({percent : percent},{where : {id : c.id}})
                            }else db.MemberCourse.update({percent : percent, completedAt: null, issuedDate:null },{where : {id : c.id}})
                       }else db.MemberCourse.update({percent : percent, completedAt: null, issuedDate:null },{where : {id : c.id}})
                    },Promise.resolve(resolve(questionCount)))
                })
            }else return questionCount;
        })
        .then(questionCount => {
            return res.status(200).json({ "success" : true, "questionCount": questionCount });
        })
        .catch(err=>{
            console.log(err)
            next(err);
        }) 
    },

    async updateQuestion(req, res, next) {
        db.CourseSectionLecture.findOne({ where : { id: req.body.lectureId} })
        .then(lecture => {
            if(lecture){
                return db.CourseQuizQuestion.findOne({where: { title: req.body.title, id: { $not : req.body.id }, companyId: req.user.companyId } })
            }else throw new RequestError("Invalid lecture received.!");
        })
        .then(checkSameName => {
            /* if(!checkSameName){ */
                if(req.file){
                    req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                    var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                    if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                        throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                    }
                    return db.CourseQuizQuestion.update({'title' :sanitize(req.body.title),thumbnail:req.file.filename},{where : { id : req.body.id, companyId:req.user.companyId }})
                }else
                    return db.CourseQuizQuestion.update({ 'title' :sanitize(req.body.title) },{where : { id : req.body.id, companyId:req.user.companyId }})
            /* }else throw new RequestError("Question title already exists.!"); */
        })
        .then(isUpdated => {
            var opt = JSON.parse(req.body.options);
            Object.keys(opt).reduce((p,a)=>{
                if(opt[a]['mode'] == 'update')
                    db.CourseQuizQuestionOption.update({'isCorrect' : opt[a]['isCorrect'], 'title' : sanitize(opt[a]['title'])},{where : {'id' : opt[a]['id']}});
            },Promise.resolve())
         }) 
         .then(optionUpdated=>{
            var opt = JSON.parse(req.body.options);
            var newOpts= [];
            Object.keys(opt).reduce((p,a)=>{
                if(opt[a]['mode'] == 'add')
                    newOpts.push({'courseQuizQuestionId': req.body.id, 'isCorrect' : opt[a]['isCorrect'], 'title' : sanitize(opt[a]['title'])})
            },Promise.resolve())
          return db.CourseQuizQuestionOption.bulkCreate(newOpts);  
         })
        .then(res => {
            if(res)
                return  db.CourseLectureQuestion.findOne({ where : { courseSectionLectureId : req.body.lectureId }, attributes:[], include:[{ model: db.CourseQuizQuestion, attributes:['id', 'title', 'marks', 'thumbnail'], where: { companyId: req.user.companyId}, required:true, include:[{ model: db.CourseQuizQuestionOption, attributes:['id', 'title', 'isCorrect'], required:true }] }] })
            else throw new RequestError("Error creating question!");
        })
        .then(result => {
            if(result)
                return res.status(200).json({"success" : true, 'result': result});
            else throw new RequestError("Error creating question!");
        })
        .catch(err=>{
            console.log(err)
            next(err);
        }) 
    },

    async lectureQuestions(req, res, next){
        db.CourseSectionLecture.findOne({ where : { id: req.query.wQ3LG, courseSectionId: req.query.rT5Sd} })
        .then(lecture => {
            if(lecture)
                return  db.CourseLectureQuestion.findAll({ where : { courseSectionLectureId : lecture.id }, attributes:[], include:[{ model: db.CourseQuizQuestion, attributes:['id', 'title', 'marks', 'thumbnail'], where: { companyId: req.user.companyId}, required:true, include:[{ model: db.CourseQuizQuestionOption, attributes:['id', 'title', 'isCorrect'], required:true }] }] })
            else
                throw new RequestError("Error fetching questions");
        })
        .then(lectureQuestions => {
            if(lectureQuestions)
                return res.status(200).json({success : true, result: lectureQuestions});
            else
                throw new RequestError("Error fetching questions");
        })
        .catch(err => {
            next(err);
        })
    },

    async deleteLectureQuestions(req, res, next){
        var sectionId;  var courseId; var courseTotalDuration;
        db.CourseSectionLecture.findOne({ where : { id: req.params.lId}, required:true, include:[{ model: db.CourseSection }] })
        .then(lecture => {
            if(lecture){
                courseId = lecture.CourseSection.courseId;
                sectionId = lecture.CourseSection.id;
                return db.CourseQuizQuestion.findOne({ where : { id: req.params.id, companyId: req.user.companyId } }).then(r => [r, lecture])
            }else throw new RequestError("Invalid lecture received.!");
        })
        .then(([courseQuizQuestion, lecture]) => {
            if(courseQuizQuestion){
                return  db.CourseQuizQuestion.destroy({ where : { id: req.params.id } }).then(r => [r, lecture])
            }else throw new RequestError("Invalid quiz question received.!");
        })
        .then(([deleteQuizQuestion, lecture]) => {
            if(deleteQuizQuestion)
                return db.CourseLectureQuestion.destroy({ where: { courseSectionLectureId:req.params.lId , courseQuizQuestionId:req.params.id } }).then(r => [r, lecture])
            else throw new RequestError("Error deleting questions");
        })
        .then(([deleteLectureQuestion, lecture]) => {
            if(deleteLectureQuestion){
                var questionCount = lecture.questionCount - 1;
                return db.CourseSectionLecture.update({ questionCount: questionCount, totalDuration: questionCount}, { where: { id: req.params.lId }, returning:true }).then(r => questionCount)
            }else throw new RequestError("Error deleting questions");
        })
        .then(questionCount => {
            return db.CourseSectionLecture.findAll({
                attributes: [
                [db.sequelize.fn('SUM', db.sequelize.col('CourseSectionLecture.totalDuration')), 'durationSum'],
            ],
            where : {courseSectionId : sectionId}}).then(r => [r, questionCount]) 
        })
        .then(([sectionDuration, questionCount]) =>{
            var data = JSON.parse(JSON.stringify(sectionDuration));
            if(data && data.length > 0){
                return db.CourseSection.update({totalDuration : data[0].durationSum},{where : {id : sectionId}}).then(r => questionCount)
            }
        })
        .then(questionCount =>{
            return db.CourseSection.findAll({
                attributes: [
                [db.sequelize.fn('SUM', db.sequelize.col('CourseSection.totalDuration')), 'durationSum'],
            ],
            where : {courseId : courseId}}).then(r => [r, questionCount]) 
        })
        .then(([courseDuration, questionCount]) =>{
            var data = JSON.parse(JSON.stringify(courseDuration));
            if(data && data.length > 0){
                courseTotalDuration = data[0].durationSum;
                return db.Course.update({totalDuration : data[0].durationSum},{where : {id : courseId, companyId: req.user.companyId}}).then(r => questionCount)
            }else return questionCount;
        })
        .then(questionCount => {           
            return db.MemberCourse.findAll({
                attributes : ['id','memberId','sessionId'],
                group: ['id'],
                include: [{
                    model: db.Session, attributes : ['id'],
                    required: true,
                    include:[{
                        model: db.SessionCourseLectureTracking ,attributes : [[db.sequelize.literal("(select sum(duration) from SessionCourseLectureTrackings where SessionCourseLectureTrackings.sessionId = Session.id AND SessionCourseLectureTrackings.deletedAt IS NULL)"), 'durationSum']]                
                    }]
                }],
                where: { courseId: courseId} }).then(r => [r, questionCount]) 
        })
        .then(([allData, questionCount]) => {
            var data = JSON.parse(JSON.stringify(allData))
            if(data.length > 0){
                return data.reduce((a,c)=>{
                    var percent = 0;
                    if(c.Session.SessionCourseLectureTrackings[0] && c.Session.SessionCourseLectureTrackings[0].durationSum){
                       percent = Math.round((c.Session.SessionCourseLectureTrackings[0].durationSum / courseTotalDuration) * 100);
                        if (percent >= 100) {percent = 100;}
                            db.MemberCourse.update({percent : percent},{where : {id : c.id}})
                    }
                },Promise.resolve()).then(r => [r, questionCount]);
            }else return [true, questionCount];
        })
        .then(([result, questionCount]) => {
            console.log(result, questionCount);
            if(result)
                return res.status(200).json({ success : true, questionCount: questionCount });
            else throw new RequestError("Error deleting questions");
        })
        .catch(err => {
            next(err);
        })
    },

    async deleteQuizQuestionOptions(req, res, next){
        db.CourseQuizQuestionOption.findOne({ where : { id: req.params.id } })
        .then(courseQuizQuestionOption => {
            if(courseQuizQuestionOption){
                return db.CourseQuizQuestionOption.destroy({ where : { id: req.params.id } });
            }else throw new RequestError("Invalid quiz question option received.!");
        })
        .then(deleteQuizQuestionOption => {
            if(deleteQuizQuestionOption)
                return res.status(200).json({ success : true });
            else throw new RequestError("Error deleting questions");
        })
        .catch(err => {
            next(err);
        })
    },
}; 
