import express from 'express';
import groupController from './group.controller';
import { sanitize } from '../../../middleware/sanitizer';
import multer from 'multer';
import { jwtStrategy } from '../../../middleware/strategy';
import { userPermission } from '../../../middleware/userPermission';
import path from 'path';
var attachmentDir = path.join(path.dirname(require.main.filename), 'public', 'groupscsv')
var storage = multer.diskStorage({
    destination: function (req, file, cb) {
      cb(null, attachmentDir)
    },
    filename: function (req, file, cb) {
      cb(null, Date.now() + path.extname(file.originalname))
    }
})
var uploadAttachment = multer({ storage: storage, limits:{ fileSize: 1024 * 1024 * 1024 }})

export const groupRouter = express.Router();
const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
  windowMs: 1 * 60 * 1000, // 15 minutes
  max: 100, 
  message: "Too many accounts created from this IP, please try again after an hour"
});
groupRouter.route('/').get(sanitize(),apiLimiter, jwtStrategy, userPermission('department-list'),groupController.index);
groupRouter.route('/:id').get(sanitize(),apiLimiter, jwtStrategy, userPermission('department-read'),groupController.find);
groupRouter.route('/sub/group/:id').get(sanitize(),apiLimiter, jwtStrategy, userPermission('department-read'),groupController.fetchAll);
groupRouter.route('/existing/group').get(sanitize(),apiLimiter, jwtStrategy, userPermission('department-read'),groupController.fetchAllExistGroups);
groupRouter.route('/').post(sanitize(), jwtStrategy, userPermission('department-write'),uploadAttachment.single('file'), groupController.create);
groupRouter.route('/:id').patch(sanitize(), jwtStrategy, userPermission('department-write'),uploadAttachment.single('file'), groupController.update);
groupRouter.route('/check').post(sanitize(), jwtStrategy, userPermission('department-write'),uploadAttachment.single('file'), groupController.checkPermission);
groupRouter.route('/:id').delete(sanitize(), jwtStrategy, userPermission('department-delete'),groupController.delete);
groupRouter.route('/active/directory').post(sanitize(), jwtStrategy, groupController.saveAD);
groupRouter.route('/check/AD').post(sanitize(), jwtStrategy, groupController.checkPassword);
groupRouter.route('/AD/password').post(sanitize(), jwtStrategy, groupController.loginPassword);
groupRouter.route('/active/directory/groups').get(sanitize(), jwtStrategy,  groupController.loadGroupUsers);
groupRouter.route('/active/directory/users').get(sanitize(), jwtStrategy,  groupController.loadUsers);
groupRouter.route('/active-directory/user-groups').post(sanitize(), jwtStrategy,  groupController.createUserGroups);
groupRouter.route('/hierarchy/data/department').get(sanitize(),apiLimiter, jwtStrategy,userPermission('department-hierarchy'),groupController.hierarchyData);
