import express from 'express';
import iamUserController from './iam-user.controller';
import { sanitize } from '../../../middleware/sanitizer';
import { jwtStrategy } from '../../../middleware/strategy';
import { validateBody, schemas } from '../../../middleware/validator';
import { userPermission } from '../../../middleware/userPermission';

export const iamUserRouter = express.Router();
iamUserRouter.route('/list').get(jwtStrategy,userPermission('user-read'),iamUserController.index);
iamUserRouter.route('/addEmployee').post(sanitize(),validateBody(schemas.addUserSchema),jwtStrategy,userPermission('user-write'), iamUserController.addEmployee);
iamUserRouter.route('/employeePolicyList/:id').get(jwtStrategy,userPermission('user-read'),iamUserController.employeePolicyList);
iamUserRouter.route('/show/:userId').get(jwtStrategy,userPermission('user-policy-read'),iamUserController.showUserPolicy);
iamUserRouter.route('/delete/:policyId/:userId').get(jwtStrategy,userPermission('user-policy-delete'),iamUserController.policyUserDelete);//delete and update does same work need to change user-policy-assign
iamUserRouter.route('/delete/:userId').delete(jwtStrategy,userPermission('user-delete'),iamUserController.userDelete);
iamUserRouter.route('/employeePolicyUpdate/:id').patch(sanitize(),validateBody(schemas.employeePolicyUpdateSchema),jwtStrategy,userPermission('user-policy-assign'),iamUserController.employeePolicyUpdate);//aleast one policy to be selected
iamUserRouter.route('/resetPassword').post(sanitize(),validateBody(schemas.employeeResetPassword),jwtStrategy,userPermission('user-write'),iamUserController.employeeResetPassword);
