import express from 'express';
import learnerController from './learner.controller';
import { sanitize } from '../../../middleware/sanitizer';
import { jwtStrategy } from '../../../middleware/strategy';
import { validateBody, schemas } from '../../../middleware/validator';
import { userPermission } from '../../../middleware/userPermission';

export const learnerRouter = express.Router();
const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
    windowMs: 1 * 60 * 1000, // 15 minutes
    max: 50, 
    message: "Too many accounts created from this IP, please try again after an hour"
  });
learnerRouter.route('/list').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.index);
learnerRouter.route('/quiz/list').get(jwtStrategy,apiLimiter, userPermission('learner-read'),learnerController.quiz);
learnerRouter.route('/list/ongoing').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.onGoingCourses);
learnerRouter.route('/list/download-certificate').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.downloadCertificate);
learnerRouter.route('/startquiz').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.startQuizAuthCheck);
learnerRouter.route('/startQuizQuestion').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.startQuizQuestion);
learnerRouter.route('/saveAnswer').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.saveAnswer);
learnerRouter.route('/finishTest').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.finishTest);
learnerRouter.route('/submitTest').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.submitTest);
learnerRouter.route('/resetQuiz').get(jwtStrategy,apiLimiter,userPermission('learner-read'),learnerController.resetQuiz);
learnerRouter.route('/quizResult').get(jwtStrategy,userPermission('learner-read'),learnerController.quizResult);
learnerRouter.route('/course-quiz-result').get(sanitize(), jwtStrategy,apiLimiter, learnerController.courseQuizResult);
