import { db } from '../../../models';
import Sequelize from 'sequelize';
import mailer from '../../../mailer';

export default {

    async index(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '';
        if(req.query.limit != undefined){
            limit = parseInt(req.query.limit);
        }
        if(req.query.search != undefined){
            search = req.query.search;
        }
        if(req.query.page != undefined){
            page = req.query.page;
            if(page < 1)
                page = 1;
        }
        db.GroupMember.findAndCountAll({
            attributes: ['groupId', 'memberId', 'isManager'],
            include: [{
                model: db.Member
            }, {
                model: db.Group, where :{ companyId: req.user.companyId}, required:true
            }],
            where: { groupId: req.query.groupId, $or: [ 
                Sequelize.literal("`Member`.`firstName` LIKE '%"+search+"%'"),
                Sequelize.literal("`Member`.`lastName` LIKE '%"+search+"%'"),
                Sequelize.literal("`Member`.`email` LIKE '%"+search+"%'"),
            ]}
        })
        .then((data) => {
            if(data){
                data = JSON.parse(JSON.stringify(data));
                let pages = Math.ceil( data.count/ limit);
                offset = limit * (page - 1);
                db.GroupMember.findAll({
                    attributes: ['groupId', 'memberId', 'isManager'],
                    include: [{
                        model: db.Member,
                        attributes: ['id', 'firstName', 'lastName', 'email', 'phone', 'designation'],
                    },{
                        model: db.Group, where :{ companyId: req.user.companyId}, required:true
                    }],
                    where: { groupId: req.query.groupId, $or: [ 
                        Sequelize.literal("`Member`.`firstName` LIKE '%"+search+"%'"),
                        Sequelize.literal("`Member`.`lastName` LIKE '%"+search+"%'"),
                        Sequelize.literal("`Member`.`email` LIKE '%"+search+"%'"),
                    ] },
                    limit: limit,
                    offset: offset,
                    order: [['isManager', 'DESC'], ['id', 'ASC']],
                })
                .then((groupMembers) => {
                    res.status(200).json({'targets': groupMembers, 'count': data.count, 'pages': pages});
                })
                .catch(function (error) {
                    console.log(error);
                    next(new RequestError('Error fetching member'));
                });
            }else{
                throw null;
            }
        })
        .catch(function (error) {
            console.log(JSON.stringify(error));
            next(new RequestError('Error fetching members'));
        });
    },

    async storeUserCourseQuiz(req, res, next){
        var courseId; var lectureName = ''; var quiz_passing_percentage;
        var percentage = 0;
        db.CourseSectionLecture.findOne({ where : { id: req.body.lectureId }, required:true, include:[{ model:db.CourseSection, include:[{ model:db.Course }] }] })
        .then(CourseSectionLecture => {
            if(CourseSectionLecture){
                lectureName = CourseSectionLecture.name
                courseId = CourseSectionLecture.CourseSection.courseId;
                quiz_passing_percentage = JSON.parse(CourseSectionLecture.CourseSection.Course.setting).quiz_passing_percentage;
                return  db.Session.findOne({ where : { sessionKey : req.body.sessionKey } })
            }else throw new RequestError("Invalid Lecture received.!");
        })
        .then(Session => {
            if(Session)
                return  db.CourseQuizQuestion.findAll({ attributes:['id'], required:true, include:[{ model: db.CourseQuizQuestionOption, attributes:['id', 'title', 'isCorrect'], required:true }, { model: db.CourseLectureQuestion, attributes:['CourseSectionLectureId'], where : { courseSectionLectureId : req.body.lectureId }, required:true }]  }).then(r => [r, Session.id])
            else throw new RequestError("Invalid Session Key received.!");
        })
        .then(([CourseLectureQuestion, sessionId]) => {
            if(CourseLectureQuestion){
                var correctOption = [];
                if(CourseLectureQuestion && CourseLectureQuestion.length > 0){
                    for(var i = 0; i < CourseLectureQuestion.length; i++){
                        var options = CourseLectureQuestion[i].CourseQuizQuestionOptions;
                        if(options.length > 0){
                            for(var j = 0; j < options.length; j++){
                                if(options[j].isCorrect)
                                    correctOption.push(options[j].id)
                            }
                        }
                    }
                }
                return [correctOption, sessionId, CourseLectureQuestion.length];
            }else throw new RequestError("Invalid Lecture received.!");
        })
        .then(([correctOption, sessionId, noOfquestions]) => {
            if(correctOption && correctOption.length > 0){
                var userAnswers = JSON.parse(req.body.results);  var dataAll = [];
                if(userAnswers && userAnswers.length > 0){
                    var marks = 0;
                    for(var z = 0; z < userAnswers.length; z++){
                        if(correctOption.includes(parseInt(userAnswers[z].optionId))){
                            marks = marks + 5;
                            dataAll.push({ "courseSectionLectureId":req.body.lectureId, "courseQuizQuestionId":userAnswers[z].questionId, "courseQuizQuestionOptionId":userAnswers[z].optionId, "sessionId":sessionId, "courseId":req.body.courseId, "marks":5, "isCorrect":1 });
                        }else
                            dataAll.push({ "courseSectionLectureId":req.body.lectureId, "courseQuizQuestionId":userAnswers[z].questionId, "courseQuizQuestionOptionId":userAnswers[z].optionId, "sessionId":sessionId, "courseId":req.body.courseId, "marks":0, "isCorrect":0 });
                    }
                    var totalMarks = (correctOption.length) * 5;
                    percentage =  Math.ceil((marks/totalMarks)* 100);
                    if (percentage >= 100) percentage = 100;
                }
                return db.CourseUserQuizAnswer.bulkCreate(dataAll).then(r => [r, percentage, totalMarks, marks, sessionId, noOfquestions]);
            }else throw new RequestError("Error submiting quiz.!");
        })
        .then(([res, percentage, totalMarks, marks, sessionId, noOfquestions]) => {
            if(res)
                return db.CourseUserQuizDetail.findOne({ where: { "courseSectionLectureId": req.body.lectureId, "sessionId":sessionId }}).then(r => [r, percentage, totalMarks, marks, sessionId, noOfquestions]);
            else throw new RequestError("Error submiting quiz.!");
        })
        .then(([findUserDetails, percentage, totalMarks, marks, sessionId, noOfquestions]) => {
            if(!findUserDetails)
                return db.CourseUserQuizDetail.create({ "courseSectionLectureId": req.body.lectureId, "sessionId":sessionId, "attempts":1, "percent":percentage, "status":1, "maxMarks": totalMarks, "obtainedMarks":marks}).then(r => [r, sessionId, noOfquestions, percentage]);
            else return db.CourseUserQuizDetail.update({ "attempts":(findUserDetails.attempts + 1), "percent":percentage, "status":1, "maxMarks": totalMarks, "obtainedMarks":marks, }, {where: {"courseSectionLectureId": req.body.lectureId, "sessionId":sessionId}, returning:true }).then(r => [r, sessionId, noOfquestions, percentage]);
        })
        .then(([res, sessionId, noOfquestions, percentage]) => {
            if(res)
                return db.SessionCourseLectureTracking.findOne({ where: { "courseSectionLectureId": req.body.lectureId, "sessionId":sessionId }}).then(r => [r, sessionId, noOfquestions, percentage]);
            else throw new RequestError("Error submiting quiz.!");
        })
        .then(([findtrack, sessionId, noOfquestions, percentage]) => {
            var percent = 100;
            if(parseInt(percentage) < parseInt(quiz_passing_percentage)){
                noOfquestions = 0;
                percent = 0
            }
            if(!findtrack)
                return db.SessionCourseLectureTracking.create({ "courseSectionLectureId": req.body.lectureId, "sessionId":sessionId, "duration":noOfquestions, "percent":percent}).then(r => [r, sessionId]);
            else return db.SessionCourseLectureTracking.update({ "duration":noOfquestions, "percent":percent }, {where: {"courseSectionLectureId": req.body.lectureId, "sessionId":sessionId}, returning:true }).then(r => [r, sessionId]);
        })
        .then(([result, sessionId]) => {
            if(result)
                return db.Course.findOne({ where: { id: courseId }, attributes:['id', 'name', 'logo', 'status', 'courseType', 'totalDuration', 'setting' ] }).then(r => [r, sessionId]);
            else  throw new RequestError("Error submiting quiz.!");
        })
        .then(([course, sessionId]) => {
            return db.sequelize.query('select SUM(slt.duration) as duration from CourseSections cs, CourseSectionLectures l, SessionCourseLectureTrackings slt where cs.id = l.courseSectionId and l.id = slt.courseSectionLectureId and cs.courseId = '
            +course.id+' and slt.sessionId = '+ sessionId+' and cs.deletedAt IS NULL and l.deletedAt IS NULL and slt.deletedAt IS NULL', { type: db.sequelize.QueryTypes.SELECT}).then(r => [r, course, sessionId])
        })
        .then(([totalDurations, course, sessionId]) => {
            if(totalDurations){
                var tduration = totalDurations[0].duration;
                var coursePercentage = Math.floor(tduration/course.totalDuration*100);
                if (coursePercentage >= 100){
                    coursePercentage = 100;
                    var date = new Date();
                    var completedAt = date.getFullYear()+'-'+(date.getMonth()+1)+'-'+date.getDate()
                    mailer.sendCourseCompletion(coursePercentage, completedAt, courseId, req.user.memberId, 'completion')
                    return db.MemberCourse.update({ percent: 100}, { where: { courseId: courseId, memberId: req.user.memberId } }).then(r => [course,sessionId, coursePercentage]);
                }else 
                    return db.MemberCourse.update({ percent: coursePercentage}, { where: { courseId: courseId, memberId: req.user.memberId } }).then(r => [course,sessionId, coursePercentage]);
            }
        })
        .then(([course, sessionId, coursePercentage]) => {
            if(course)
               return db.CourseUserQuizDetail.findOne({ where: {"courseSectionLectureId": req.body.lectureId, "sessionId":sessionId }, include:[{ model:db.Session }] }).then(r => [r,course, coursePercentage]);
            else  throw new RequestError("Error submiting quiz.!");
        })
        .then(([courseUserDetails, course, coursePercentage]) => {
            if(courseUserDetails)
               return db.MemberCourse.findOne({ where: { courseId: courseId, memberId: req.user.memberId } }).then(r => [r, courseUserDetails, course, coursePercentage]);
            else  throw new RequestError("Error submiting quiz.!");
        })
        .then(([memberCourse, courseUserDetails, course, coursePercentage]) => {
            if(memberCourse){
                var settings = (course.setting ? JSON.parse(course.setting) : undefined);
                var quizCoursePassingPercentage = ( settings && settings.hasOwnProperty('quiz_passing_percentage') ? settings.quiz_passing_percentage : 0); 
                var quiz_fail_reminder = ( settings && settings.hasOwnProperty('quiz_fail_reminder') ? settings.quiz_fail_reminder : false); 
                if(quiz_fail_reminder && Math.round(courseUserDetails.percent) < parseInt(quizCoursePassingPercentage))
                    mailer.CourseQuizFails(courseUserDetails, lectureName, courseId, req.user.memberId, 'quiz_fail')
                res.status(200).json({'success': true, percent: (courseUserDetails && courseUserDetails.percent ? courseUserDetails.percent : 0), coursePercentage:coursePercentage, memberCourseId: memberCourse.id });
            }
            else  throw new RequestError("Error submiting quiz.!");
        })
        .catch(err => {
            next(err);
        })
    },

    async restartUserCourseQuiz(req, res, next){
        var courseId; var courseDuration = 0;
        db.CourseSectionLecture.findOne({ where : { id: req.query.lectureId }, required:true, include:[{ model:db.CourseSection, required:true, include:[{ model:db.Course }] }] })
        .then(CourseSectionLecture => {
            if(CourseSectionLecture){
                courseId = CourseSectionLecture.CourseSection.courseId;
                courseDuration = CourseSectionLecture.CourseSection.Course.totalDuration;
                return  db.Session.findOne({ where : { sessionKey : req.query.sessionKey } })
            }else throw new RequestError("Invalid Lecture received.!");
        })
        .then(Session => {
            if(Session)
                return db.CourseUserQuizAnswer.destroy({ where: { courseSectionLectureId: req.query.lectureId, sessionId:Session.id } }).then(r => Session);
            else throw new RequestError("Invalid Session Key received.!");
        })
        .then(Session => {
            return db.CourseUserQuizDetail.update({ "percent":0, "status":0,"maxMarks": 0, "obtainedMarks":0 }, { where: { courseSectionLectureId: req.query.lectureId, sessionId:Session.id }, returning:true }).then(r => Session);
        })
        .then(Session => {
            return db.SessionCourseLectureTracking.update({ "percent":0, "duration":0 }, { where: { courseSectionLectureId: req.query.lectureId, sessionId:Session.id }, returning:true }).then(r => Session);
        })
        .then(Session => {
            return db.sequelize.query('select SUM(slt.duration) as duration from CourseSections cs, CourseSectionLectures l, SessionCourseLectureTrackings slt where cs.id = l.courseSectionId and l.id = slt.courseSectionLectureId and cs.courseId = '
            +courseId+' and slt.sessionId = '+ Session.id+' and cs.deletedAt IS NULL and l.deletedAt IS NULL and slt.deletedAt IS NULL', { type: db.sequelize.QueryTypes.SELECT})
        })
        .then(totalDurations => {
            if(totalDurations){
                var tduration = totalDurations[0].duration;
                var coursePercentage = Math.floor(tduration/courseDuration*100);
                return db.MemberCourse.update({ percent: coursePercentage, completedAt:null }, { where: { courseId: courseId, memberId: req.user.memberId } }).then(r => coursePercentage)
            }
        })
        .then(coursePercentage => {
            return db.CourseLectureQuestion.findAll({ where: { courseSectionLectureId: req.query.lectureId }, include:[{ model: db.CourseQuizQuestion, include:[{ model: db.CourseQuizQuestionOption, attributes:['id', 'title'] }] }] }).then(r => [r, coursePercentage])
        })
        .then(([result, coursePercentage]) => {
            if(result)
                return db.MemberCourse.findOne({ where: { courseId: courseId, memberId: req.user.memberId } }).then(r => [r, result, coursePercentage]);
            else  throw new RequestError("Error submiting quiz.!");
        })
       .then(([memberCourse, result, coursePercentage]) => {
            if(memberCourse)
                res.status(200).json({'success': true, "result":result, coursePercentage: coursePercentage, memberCourseId: memberCourse.id });
            else  throw new RequestError("Error restart quiz.!");
        })
        .catch(err => {
            next(err);
        })
    },

    async emailUnsubscribe(req, res, next){
        db.Member.findOne({ attributes:['courseMails', 'assessmentMails', 'reminderMails', 'announcementMails'], where: { id: req.user.memberId } })
        .then(member => {
            res.status(200).json({'success': true, "member":member });
        })
        .catch(err => {
            next(err);
        })
    },

    async saveEmailUnsubscribe(req, res, next){
        db.Member.update(req.body, { where: { id: req.user.memberId } })
        .then(member => {
            res.status(200).json({'success': true });
        })
        .catch(err => {
            next(err);
        })
    },

    async unsubscribe(req, res, next){
        db.Member.findOne({ where: { code: req.body.code } })
        .then(member => {
            if(member){
                return db.Member.update({ courseMails: req.body.courseMails, assessmentMails:req.body.assessmentMails, reminderMails:req.body.reminderMails, announcementMails:req.body.announcementMails }, { where:{ id: member.id } })
            }else throw new RequestError("Invalid user received!");
        })
        .then(result =>{
            res.status(200).json({'success': true });
        })
        .catch(err => {
            next(err);
        })
    },
};