import { db } from '../../../models';
import { sanitize } from './../../../functions';

var bulkCreateQuestionOption = function(q, opt){
    return db.QuizQuestion.create(q)
    .then(question =>{
        var addOptions = [];
        for(var j = 0; j < opt.length; j++)
            addOptions.push({ quizQuestionId: question.id, title:opt[j].title, isCorrect:opt[j].isCorrect })
        return db.QuizQuestionOption.bulkCreate(addOptions);
    })
    .catch(err => {
        console.log(err);
    })
}

var addCloneQuestionOption = function(newQuizId, questions){
    return new Promise((resolve, reject) => {
        return questions.reduce((p, data) => {
            var q = { quizId:newQuizId, title: data.title, marks: data.marks,  thumbnail: data.thumbnail }
            return bulkCreateQuestionOption(q, data.QuizQuestionOptions)
        }, Promise.resolve(resolve(true)));
    })
}


export default {
    async fetchQuiz(req, res, next) {
        db.Quiz.findOne({ where : { id : req.query.id, companyId:req.user.companyId} ,
            include: [{ model: db.AssessmentSkill,
                required: false,
                include:[{ model: db.Skill, attributes: ['id', 'name']}]
            },{ model : db.QuizQuestion , required: false , include: [{ model : db.QuizQuestionOption}]}]})
        .then(quiz=>{
            return res.status(200).json({ success: true,'quiz' : quiz })
        })
        .catch(err=>{
            next(err)
        })
    },

    async clone(req, res, next) {
        if(req.query.id){
            db.Quiz.findOne({ where : { id : req.query.id, companyId:req.user.companyId },
                include: [{ model: db.AssessmentSkill, attributes:['skillId', 'quizId'] },{ model : db.QuizQuestion, attributes:['quizId', 'title', 'marks', 'thumbnail'], include: [{ model : db.QuizQuestionOption, attributes:['quizQuestionId', 'title', 'isCorrect'] }]}]})
            .then(quiz=>{
                if(quiz){
                    var search = quiz.name+'-Copy%';
                    return db.Quiz.count({ where: { companyId: req.user.companyId, name: { like: search } } }).then(r =>[r, quiz])
                }else throw new RequestError('Assessment not found!');
            })
            .then(([count, data])=>{
                var originalQuiz = data;
                var newdata  =  {
                    companyId:data.companyId,
                    status: 'Draft',
                    logo: data.logo,
                    description: data.description,
                    languageId: data.languageId,
                    totalMarks:data.totalMarks,
                    totalQuestions: data.totalQuestions,
                    settings:data.settings
                }
                if(count == 0){
                    newdata.name = data.name + '-Copy';
                }else{
                    count = count + 1;
                    newdata.name = data.name + '-Copy('+count+')';
                } 
                return db.Quiz.create(newdata).then(r => [r, originalQuiz]);
            })
            .then(([cloneQuiz, originalQuiz])=>{
                var newQuizId = cloneQuiz.id;
                var questions = originalQuiz.QuizQuestions;
                var assessmentSkills = originalQuiz.AssessmentSkills;
                return addCloneQuestionOption(newQuizId, questions).then(r => [newQuizId, assessmentSkills]);
            })
            .then(([newQuizId,assessmentSkills]) => {
                var skills = [];
                if(assessmentSkills.length > 0){
                    for(var i = 0; i < assessmentSkills.length; i++)
                        skills.push({ 'quizId':newQuizId, 'skillId': assessmentSkills[i].skillId });
                }
                return db.AssessmentSkill.bulkCreate(skills, {returning: true });
            })
            .then(r => {
                return res.status(200).json({ success: true }) 
            })
            .catch(err=>{
                next(err)
            })
        }else throw new RequestError('Invalid Data Received!');
    },

    async listQuiz(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '%%';
        var langCondition = { };
        var skillCondition = { };
        let sort = ['createdAt', 'DESC'];
        if(req.query.limit != undefined){
            limit = parseInt(req.query.limit);
        }
        if (req.query.search) {
            search = '%' + req.query.search + '%';
        }
        if(req.query.language) {
            langCondition = { name : { like: req.query.language }};
        }
        if(req.query.skill) {
            skillCondition = { skillId : req.query.skill };
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        if (req.query.sort) {
            if (req.query.sort == 'name') {
                sort = ['name'];
            }
        }
        var condition = { companyId: req.user.companyId, name: { like: search } };
        if (req.query.fromDate && req.query.toDate) {
            let endDate = new Date(req.query.toDate);
            endDate.setDate(endDate.getDate()+1);
            condition['createdAt'] = { $between: [new Date(req.query.fromDate), endDate] }
        }
        db.Language.findOne({ where : { name : req.query.language } })
        .then(language=>{
            if(language && language.id) condition['languageId'] = language.id;
            return db.Quiz.findAll({
                where: condition,
                attributes: ['id'],
                include: [{
                    model: db.AssessmentSkill,
                    attributes: [],
                    where: skillCondition,
                    required:false
                }]
            })
        })
        .then((count) => {
            count = json(count).length;
            let pages = Math.ceil(count / limit);
            offset = limit * (page - 1);
            db.Quiz.findAll({
                attributes: ['id', 'name', 'logo' , 'description' , 'totalMarks', 'totalQuestions', 'status', 'createdAt' ],
                where: condition,
                include: [{
                        model: db.Language,
                        attributes: ['name'],
                        where: langCondition,
                        required: true
                    },{
                        model: db.AssessmentSkill,
                        attributes: ['skillId', 'quizId'],
                        where: skillCondition,
                        required: false,
                        include:[{ model: db.Skill, attributes: ['id', 'name']}]
                    }],
                limit: limit,
                offset: offset,
                order: [sort],
            })
            .then(quiz => {
                res.status(200).json({ 'quiz': quiz, 'count': count, 'pages': pages });
            })
            .catch(function (error) {
                console.log(error);
                res.status(500).json({ 'errors': ['Error fetching quizzes.'] });
            });
        })
        .catch(function (error) {
            console.log(error);
            res.status(500).json({ 'errors': ['Error fetching quizzes.'] });
        });
    },
    async storeQuiz(req, res, next) { 
        var {  name  , description , languageId } = req.body;
        var uaParser = require('ua-parser-js');
        var ip = req.headers["x-real-ip"] || req.ip;
        var detail = {};
        detail.user_agent = req.headers['user-agent'];
        detail.systemDetail = uaParser(req.headers['user-agent']);
        detail.ip = ip;
        var info = { info: detail};
        var userId = req.user.User.id;
        var type = 'quiz';
        var metaData = JSON.stringify(info);
        var companyId = req.user.companyId;
        var username = req.user.User.firstName+' '+req.user.User.lastName;
        var text = sanitize(name)+' assessment created by '+username;
        db.Quiz.findOne({ where : { name, companyId } })
        .then(quiz=>{
            if(quiz) throw new RequestError("Assessment title already exists.");
            if(req.file){
                req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                    throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                }
            }
            name=sanitize(name);
            return db.Quiz.create({ name ,status : 'Draft',logo: req.file?req.file.filename:null, description , languageId : languageId, companyId : req.user.companyId  })
        })
        .then(quiz=>{
            if(quiz){
                var skills = [];
                var skill = req.body.skills.split(',');
              for(var i=0;i< skill.length; i++){
                skills.push({
                    'quizId' : quiz.id ,
                    'skillId' : skill[i]
                })
              }
              return db.AssessmentSkill.bulkCreate(skills).then(r => quiz);     
            }
        })
        .then(quiz=>{
            if(quiz){
                db.RecentActivity.create({userId, companyId, type, text, metaData});
                return res.status(200).json({ success : 1 ,id: quiz.id });
            }
            throw new RequestError("Failed To Create Recent Activit.");
        })
        .catch(err=>{
        console.log(err)
            next(err);
        }) 
    },
    async storeQuizQuestion(req, res, next) {
        var data = JSON.parse(req.body.data);
        var marks = 5;
        var quizMarks = 0;
        var totalQuestions = 0;
        var { title,quizId } = req.body;
        title=sanitize(title);
        var companyId = req.user.companyId; 
        db.Quiz.findOne({ where :{  id : req.body.quizId, companyId: companyId } })
        .then(quiz => {
            if(quiz){
                quizMarks = quiz.totalMarks;
                totalQuestions = quiz.totalQuestions;
                return db.QuizQuestion.findOne({ where :{ title: title }, include:[{ model: db.Quiz, where :{  id : req.body.quizId, companyId: companyId }, required:true }] })
            }else throw new RequestError("Quiz does not exists.");
        })
        .then(quiz=>{
            if(quiz) throw new RequestError("Question name already exists.");
            else{ 
                if(req.file){
                    req.file.originalname = sanitize((req.file.originalname ? req.file.originalname : ''));
                    var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                    if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                        throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                    }
                }
                return db.QuizQuestion.create({ quizId , title , marks, thumbnail : req.file?req.file.filename:null})
            }
        })
        .then(quizQuestion=>{
            var questionAnswers = [];
            var answers = data;
            Object.keys(answers).reduce((p,a)=>{
                answers[a]["quizQuestionId"] = quizQuestion.id;
                questionAnswers.push(answers[a])
            },Promise.resolve())
            return db.QuizQuestionOption.bulkCreate(questionAnswers).then(r=>[r,quizQuestion]);  
        })
        .then(([options,quizQuestion])=>{
            return db.QuizQuestionOption.findAll({where : {quizQuestionId : quizQuestion.id}}).then(r=>[r,quizQuestion])
        })
        .then(([questionOption,quizQuestion]) =>{
            db.Quiz.update({'totalMarks' : Math.floor(quizMarks+5), 'totalQuestions' : Math.floor(totalQuestions+1)},{where : {id : req.body.quizId}});
            return res.status(200).json({ success : 1, questionId:quizQuestion.id, options:questionOption})
        })
        .catch(err=>{
            console.log(err)
            next(err);
        }) 
     },
     async editQuizQuestion( req, res, next) {
         console.log(req.body);
        var data = JSON.parse(req.body.data);
        var companyId = req.user.companyId;
        db.QuizQuestion.findOne({ where : { id : req.body.questionId } })
        .then(quizQues =>{
            if(quizQues)
                return db.QuizQuestion.findOne({ where : { quizId:quizQues.quizId, title: sanitize(req.body.title), id: { $not : req.body.questionId } }, include:[{ model: db.Quiz, where :{ companyId: companyId }, required:true }] })
            else
                throw new RequestError('Question not Exist');
        })
        .then(isExist=>{
            if(!isExist){
                if(req.file){
                        req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                        var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                        if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                            throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                        }
                        return db.QuizQuestion.update({'title' :sanitize(req.body.title),thumbnail:req.file.filename},{where : { id : req.body.questionId }})
                }else
                    return db.QuizQuestion.update({'title' :sanitize(req.body.title)},{where : { id : req.body.questionId }})
            }else
                throw new RequestError('Question name already exists.')
        }) 
        .then(isUpdated => {
           var opt = data["update"];
           Object.keys(opt).reduce((p,a)=>{
            db.QuizQuestionOption.update({'isCorrect' : opt[a]['isCorrect'],'title' : sanitize(opt[a]['title'])},{where : {'id' : opt[a]['id']}});
           },Promise.resolve())
        }) 
        .then(optionUpdated=>{
            var addedOptions = [];
            var addedOpt = data["add"];
            Object.keys(addedOpt).reduce((p,a)=>{
                addedOpt[a]["quizQuestionId"] = req.body.questionId;
                addedOptions.push(addedOpt[a])
            },Promise.resolve())
         return db.QuizQuestionOption.bulkCreate(addedOpt);  
        })
        .then(addedUpdated=>{
            return db.QuizQuestionOption.findAll({where : {quizQuestionId : req.body.questionId}})
        })
        .then(questionOption => {
              return res.status(200).json({ success : 1,options:questionOption})
        })
        .catch(err=>{
            console.log(err)
                next(err);
            }) 
    },

    async update(req, res, next) {
        var {name, description, languageId, quizId} = req.body;
        name=sanitize(name);
        db.Quiz.findOne({where : { id:quizId, companyId:req.user.companyId }})
        .then(isExist => {
            if(isExist){
                return db.Quiz.findOne({ where: { name: name, companyId: req.user.companyId, id: { $not : quizId }}})
            }else throw new RequestError('Invalid Assessment');
        })
        .then(result => {
            if(result) throw new RequestError('Assessment with same name exists.');
            if(req.file){
                req.file.originalname = (req.file.originalname ? sanitize(req.file.originalname) : '');
                var fileExt=req.file.originalname.split('.').pop().toLowerCase();
                if (fileExt != "jpeg" && fileExt != "jpg" && fileExt != "png") {
                    throw new RequestError('Only .png, .jpg and .jpeg format allowed!',403);
                }
                return db.Quiz.update({name:name,description:description,languageId:languageId,logo:req.file.filename},{where : {id:quizId, companyId: req.user.companyId}});
            }
            return db.Quiz.update({name,description,languageId},{where : {id:quizId, companyId: req.user.companyId}});
        })
        .then(result => {
            return db.AssessmentSkill.destroy({where : {quizId : quizId}}).then(r =>  result);   
        })
        .then(result => {
            var skills = [];
            var skill = req.body.skills.split(',');
            for(var i=0;i< skill.length; i++)
                skills.push({ 'quizId' : quizId, 'skillId' : skill[i] });
            return db.AssessmentSkill.bulkCreate(skills, { updateOnDuplicate: ["courseId", "skillId"] }).then(r => result);
        })
        .then(r => {
            if(r){
                return res.status(200).json({'success' : 1});
            }else throw new RequestError('Failed To Update')
        })
        .catch(err => {
            next(err);
        })
    },

    async deleteQuestion(req, res, next){
        db.QuizQuestion.findOne({where : {id:req.body.questionId}})
        .then(isExist => {
            if(isExist){
              return db.QuizQuestion.destroy({where : {id:req.body.questionId}});
            }else throw new RequestError('Invalid Question Id');
        })
        .then(deleted => {
            if(deleted)
                return db.Quiz.findOne({where : {id : req.body.quizId}})
            else throw new RequestError('Failed to Delete');
        })
        .then(quizData => {
            if(quizData)
               return db.Quiz.update({totalMarks : Math.round(quizData.totalMarks - 5), totalQuestions :  Math.round(quizData.totalQuestions - 1)},{where : {id : req.body.quizId}});
            else  throw new RequestError('Invalid Quiz Id');
        })
        .then(deletedMarks => {
            if(deletedMarks)
                return res.status(200).json({'success' : 1});
            else  throw new RequestError('Failed to calculate total Marks');
        })
        .catch(err => {
            next(err);
      })
    },

    async deleteQuestionOption(req, res, next){
        db.QuizQuestionOption.findOne({where : {id:req.body.optionId}})
        .then(isExist => {
            if(isExist){
              return db.QuizQuestionOption.destroy({where : {id:req.body.optionId}});
            }else throw new RequestError('Invalid Question Option Id');
        })
        .then(deleted => {
            if(deleted)
                return res.status(200).json({'success' : 1});
            else  throw new RequestError('Failed to Delete');
        })
        .catch(err => {
            next(err);
      })
    },

    async settings(req, res, next){
        db.Quiz.findOne({where : {id : req.query.quizId}})
        .then(isExist => {
            if(isExist)
                return db.Quiz.update({settings : req.query.settings,status : req.query.status},{where : {id : req.query.quizId}});
            else throw new RequestError('Invalid Quiz Id');
        })
        .then(isUpdated => {
            if(isUpdated)
                return res.status(200).json({'success' : 1});
            else throw new RequestError('Failed To Update');
        })
        .catch(err => {
            next(err);
        })

    },

    async isQuestionAdded(req, res, next){
        db.QuizQuestion.findAll({where : {quizId : req.query.quizId}})
        .then(quizCount => {
            if(quizCount)
                return res.status(200).json({"success" : 1,'questionCount' : quizCount.length})
            else throw new RequestError('Invalid Quiz Id');
        })
        .catch(err => {
            next(err);
        })
    },

     async quizGuidelines(req, res, next) {
        if(req.params.id > 0 && req.params.sKey){
            db.Quiz.findOne({ where : { id: req.params.id }, attributes:['id', 'name', 'logo', 'settings', 'totalQuestions']})
            .then(quiz => {
                if(quiz)
                    return db.Session.findOne({ where : { sessionKey: req.params.sKey }, attributes:['id', 'email', 'firstName', 'lastName'] });
                else  throw new RequestError("Quiz not found");
            })
            .then(session => {
                if(session)
                    return db.MemberCourse.findOne({ where : { quizId: req.params.id }, attributes:[], include:[{ model:db.Quiz }] });
                else throw new RequestError("Invalid token received");
            })
            .then(memberQuiz => {
                if(memberQuiz)
                    return res.status(200).json({ "success": true, "result":memberQuiz });
                else  throw new RequestError("Invalid Quiz received");
            })
            .catch(err => {
                console.log("TCL: index -> err", err)
                next(err);
            })
        }else throw new RequestError("Invalid data received");
    },
}; 
