import express from 'express';
import quizController from './quiz.controller';
import { sanitize } from '../../../middleware/sanitizer';
import { jwtStrategy } from '../../../middleware/strategy'
import { userPermission } from '../../../middleware/userPermission'
import config from '../../../config';
import multer from 'multer';
import path from 'path';

var attachmentDir = path.join(config.storage.folder, config.storage.quiz_logo);
var attachmentDirContent = path.join(config.storage.folder, config.storage.quiz_content);

var storage = multer.diskStorage({
    destination: function (req, file, cb) {
      if(file.fieldname==="logo") cb(null, attachmentDir)
      if(file.fieldname==="filename") cb(null, attachmentDirContent)
    },
    filename: function (req, file, cb) {
      var originalname = file.originalname;
      if(file.mimetype.includes('image')) originalname = file.originalname +'.png';
      cb(null, Date.now() + path.extname(originalname))
    }
  })
var uploadAttachment = multer({ storage: storage, limits:{ fileSize: 10485760 },
  fileFilter: (req, file, cb) => {
    if (file.mimetype == "image/png" || file.mimetype == "image/jpg" || file.mimetype == "image/jpeg") {
      cb(null, true);
    } else {
      cb(null, false);
      return cb(new RequestError('Only .png, .jpg and .jpeg format allowed!'));
    }
  } 
});

export const quizRouter = express.Router();
const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
    windowMs: 1 * 60 * 1000, // 15 minutes
    max: 100, 
    message: "Too many accounts created from this IP, please try again after an hour"
  });
quizRouter.route('/').get(sanitize(),apiLimiter, jwtStrategy, userPermission('quiz-read'),quizController.fetchQuiz);
quizRouter.route('/clone').get(sanitize(), jwtStrategy, userPermission('quiz-write'),quizController.clone);
quizRouter.route('/list').get(sanitize(),apiLimiter, jwtStrategy, userPermission('quiz-list'),quizController.listQuiz);
quizRouter.route('/delete').delete(sanitize(),apiLimiter, jwtStrategy, userPermission('quiz-delete'),quizController.deleteQuestion);
quizRouter.route('/delete-option').delete(sanitize(),apiLimiter, jwtStrategy, userPermission('quiz-delete'),quizController.deleteQuestionOption);
quizRouter.route('/settings').get(sanitize(), jwtStrategy, userPermission('quiz-read'),quizController.settings);
quizRouter.route('/question-count').get(sanitize(),apiLimiter, jwtStrategy, userPermission('quiz-read'),quizController.isQuestionAdded);
quizRouter.route('/').post(sanitize(),apiLimiter, jwtStrategy, userPermission('quiz-write'),uploadAttachment.single('logo'),quizController.storeQuiz);
quizRouter.route('/').patch(sanitize(),apiLimiter, jwtStrategy, userPermission('quiz-write'),uploadAttachment.single('logo'),quizController.update);
quizRouter.route('/question').post(sanitize(),apiLimiter,  jwtStrategy, userPermission('quiz-write'),uploadAttachment.single('filename'),quizController.storeQuizQuestion);
quizRouter.route('/question').patch(sanitize(),apiLimiter,  jwtStrategy, userPermission('quiz-write'),uploadAttachment.single('filename'),quizController.editQuizQuestion);
quizRouter.route('/guidelines/:id/:sKey').get(jwtStrategy,apiLimiter, userPermission('learner-read'),quizController.quizGuidelines); //change router
