import { db } from '../../../models';
import simulation from '../../../krademy/simulation';

export default {
    async index(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '%%';
        let sort = ['createdAt', 'DESC'];
        if(req.query.limit != undefined)
            limit = parseInt(req.query.limit);
        if (req.query.search) 
            search = '%' + req.query.search + '%';
        if(req.query.language) 
            langCondition = { name : { like: req.query.language }};
        if(req.query.page) {
            page = req.query.page;
            if (page < 1) page = 1;
        }
        if(req.query.sort) {
            if (req.query.sort == 'name') 
                sort = ['name'];
        }
        var condition = { companyId: req.user.companyId, status: 'Published', name: { like: search } };
        if (req.query.fromDate && req.query.toDate) {
            let endDate = new Date(req.query.toDate);
            endDate.setDate(endDate.getDate()+1);
            condition['createdAt'] = { $between: [new Date(req.query.fromDate), endDate] }
        }
        db.Quiz.count({ where: condition })
        .then((count) => {
            let pages = Math.ceil(count / limit);
            offset = limit * (page - 1);
            return db.Quiz.findAll({ where: condition, attributes:['id', 'companyId', 'status', 'name', 'totalMarks', 'totalQuestions', 'settings'], 
                order: [sort],
                limit: limit,
                offset: offset,
            }).then(r => [r, pages, count])
        })
        .then(([quiz, pages, count]) => {
            res.status(200).json({ 'quiz': quiz, 'count': count, 'pages': pages });
        })
        .catch(function (error) {
            console.log(error);
            res.status(500).json({ 'errors': ['Error fetching quizzes.'] });
        });
    },

    async fetch(req, res, next) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '%%';
        var condition = { quizId: req.query.id};
        let sort = ['createdAt', 'DESC'];
        if(req.query.limit != undefined){
            limit = parseInt(req.query.limit);
        }
        if (req.query.search) {
            search = '%' + req.query.search + '%';
        }
        if (req.query.page) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        if (req.query.sort) {
            var percent = req.query.sort;
            condition['percent'] = { $between: [percent.start, percent.end] }
        }
        if(req.query.type =='csv'){
            var uaParser = require('ua-parser-js');
            var ip = req.headers["x-real-ip"] || req.ip;
            var detail = {};
            detail.user_agent = req.headers['user-agent'];
            detail.systemDetail = uaParser(req.headers['user-agent']);
            detail.ip = ip;
            var info = { info: detail};
            var metaData = JSON.stringify(info);
            var username = req.user.User.firstName+' '+req.user.User.lastName;
            var userId = req.user.User.id;
            var companyId = req.user.companyId;
            var type = 'quiz';
            var text = 'CSV file of Assessment result is downloaded by '+username;
            db.RecentActivity.create({userId, companyId , type, text, metaData});
        }
        db.MemberCourse.count({
            where: condition,
            include:[{ model:db.Quiz, where: { id: req.query.id, status: 'Published', companyId: req.user.companyId }, required:true }]
        })
        .then((count) => {
            let pages = Math.ceil(count / limit);
            offset = limit * (page - 1);
            if(req.query.type == 'csv')
                return db.MemberCourse.findAll({
                    where: condition,
                    attributes:['id', 'quizId', 'createdAt', 'sessionId', 'percent', 'completedAt'],
                    include:[
                            { model: db.Session, attributes: ['id', 'firstName', 'lastName', 'email','sessionKey'], 
                                where: { $or: [ 
                                    db.Sequelize.literal("`Session`.`firstName` LIKE '%"+search+"%'"),
                                    db.Sequelize.literal("`Session`.`lastName` LIKE '%"+search+"%'"),
                                    db.Sequelize.literal("`Session`.`email` LIKE '%"+search+"%'"),
                                ]},
                                include:[{ model: db.QuizUserDetail }]
                            }, 
                            { model:db.Quiz, attributes:['id', 'totalMarks', 'settings'], 
                                where: { id: req.query.id, status: 'Published', companyId: req.user.companyId }, 
                                required: true
                            }
                        ],
                    order: [sort],
                }).then(r => [r, count, pages]);
            else
                return db.MemberCourse.findAll({
                    where: condition,
                    attributes:['id', 'quizId', 'createdAt', 'sessionId', 'percent', 'completedAt'],
                    include:[
                            { model: db.Session, attributes: ['id', 'firstName', 'lastName', 'email','sessionKey'], 
                                where: { $or: [ 
                                    db.Sequelize.literal("`Session`.`firstName` LIKE '%"+search+"%'"),
                                    db.Sequelize.literal("`Session`.`lastName` LIKE '%"+search+"%'"),
                                    db.Sequelize.literal("`Session`.`email` LIKE '%"+search+"%'"),
                                ]},
                                include:[{ model: db.QuizUserDetail }]
                            }, 
                            { model:db.Quiz, attributes:['id', 'totalMarks', 'settings'], 
                                where: { id: req.query.id, status: 'Published', companyId: req.user.companyId }, 
                                required: true
                            }
                        ],
                    limit: limit,
                    offset: offset,
                    order: [sort],
                }).then(r => [r, count, pages]);
        })
        .then(([users, count, pages]) => {
            res.status(200).json({ 'users': users, 'count': count, 'pages': pages });
        })
        .catch(function (error) {
            console.log(error);
            res.status(500).json({ 'errors': ['Error fetching users.'] });
        });
    },
}; 
