import express from 'express';
import senderController from './sender.controller';
import { sanitize } from '../../../middleware/sanitizer';
import {jwtStrategy} from '../../../middleware/strategy';
import {userPermission} from '../../../middleware/userPermission';
import { validateBody, schemas } from '../../../middleware/validator';

export const senderRouter = express.Router();
const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
    windowMs: 1 * 60 * 1000, // 15 minutes
    max: 50, 
    message: "Too many accounts created from this IP, please try again after an hour"
  });
senderRouter.route('/').get(sanitize(),apiLimiter, jwtStrategy, userPermission('sender-read'),senderController.index);
senderRouter.route('/').post(sanitize(['fromAddress']),apiLimiter, jwtStrategy, userPermission('sender-write'),senderController.create);
senderRouter.route('/testEmail').post(sanitize(['fromAddress']),apiLimiter, jwtStrategy, userPermission('sender-write'),senderController.testEmail);
senderRouter.route('/:id').patch(sanitize(['fromAddress']),apiLimiter, jwtStrategy, userPermission('sender-write'),senderController.update);
senderRouter.route('/advanced/:id').patch(sanitize(),apiLimiter, jwtStrategy, userPermission('sender-write'),senderController.updateAdvanceSetting);
senderRouter.route('/headers').get(sanitize(),apiLimiter, jwtStrategy, userPermission('sender-read'),senderController.fetch);
senderRouter.route('/headers').post(sanitize(),apiLimiter, validateBody(schemas.headerCreate), jwtStrategy,userPermission('sender-write'), senderController.addHeader);
senderRouter.route('/header/:id').delete(apiLimiter,jwtStrategy, userPermission('sender-delete'),senderController.delete);
