import express from 'express';
import skillController from './skill.controller';
import { sanitize } from '../../../middleware/sanitizer';
import {jwtStrategy} from '../../../middleware/strategy';
import {userPermission} from '../../../middleware/userPermission';

export const skillRouter = express.Router();
const rateLimit = require("express-rate-limit");
const apiLimiter = rateLimit({
    windowMs: 1 * 60 * 1000, // 15 minutes
    max: 50, 
    message: "Too many accounts created from this IP, please try again after an hour"
  });
skillRouter.route('/').get(sanitize(), jwtStrategy, userPermission('skill-list'),skillController.index);
skillRouter.route('/skillsByParent').get(sanitize(), jwtStrategy, userPermission('skill-read'),skillController.skillsByParent);
skillRouter.route('/').post(sanitize(),apiLimiter,jwtStrategy, userPermission('skill-write'),skillController.create);
skillRouter.route('/').patch(sanitize(),apiLimiter, jwtStrategy, userPermission('skill-write'),skillController.update);
skillRouter.route('/:id').get(sanitize(),apiLimiter, jwtStrategy, userPermission('skill-read'),skillController.fetch);
skillRouter.route('/all/:id').get(sanitize(),apiLimiter, jwtStrategy, userPermission('skill-read'),skillController.fetchAll);
skillRouter.route('/:id').delete(sanitize(),apiLimiter, jwtStrategy, userPermission('skill-delete'),skillController.delete);
