import 'dotenv/config';
import { db } from './models';
import { restRouter } from './api';
import { webRouter } from './web';
import config from './config';
import appManager from './app';
import './passport';
import kue from './kue';
import './errors';
import scheduler from './scheduler';
import path from 'path';

global.appRoot = path.resolve(__dirname);

const PORT = config.app.port;

const app = appManager.setup(config, path.join(__dirname, 'web', 'resources', 'views', 'layouts', 'main.hbs'));

// app.use(cors());

/* Route handling */
app.use('/api', restRouter);
app.get('/sitemap.xml', function(req, res) {
    res.sendFile(path.resolve(__dirname) + '/public/sitemap.xml');
});
app.get('/loaderio-8bc680d414d5ec9f91a38f5c028571e5', function(req, res) {
    res.send('loaderio-8bc680d414d5ec9f91a38f5c028571e5');
});
app.get('/favicon.ico', function(req, res) {
    res.sendFile(path.resolve(__dirname) + '/public/images/favicon.ico');
});
app.use('/', webRouter);
app.use((req, res, next) => {
	next(new RequestError('Invalid route', 404));
});
/* app.get('*', function(req, res) {
    res.sendFile(path.resolve(__dirname) + '/public/index.html');
}); */
app.use((req, res, next) => {
	const error = new Error('Not found');
	error.message = 'Invalid route';
	error.status = 404;
	next(error);
});

app.use((error, req, res, next) => {
if (!(error instanceof RequestError)) {
		error = new RequestError('Some Error Occurred', 500, error.message);
	}
	res.status(error.status || 500);
	let contype = req.headers['content-type'];
	var json = !(!contype || contype.indexOf('application/json') !== 0);
	if (json) {
		return res.json({ errors: error.errorList });
	} else {
		if(error.status == 401){
			return res.render('401', { message: error.message })
		} else {
		res.send(error.message)

		}
	}
});

kue.init();

/* Database Connection */
db.sequelize.authenticate().then(function () {
	console.log('Nice! Database looks fine');
	scheduler.init();
}).catch(function (err) {
	console.log(err, "Something went wrong with the Database Update!")
});

/* Start Listening service */
app.listen(PORT, () => {
	console.log(`Server is running at PORT http://localhost:${PORT}`);
});