import { db } from '../models';
import simulation from './simulation';
import { queue } from '../kue';


var createCampaign = (companyId, data) => {
    db.RecentActivity.create({userId:data.userId, companyId:companyId, type:'campaign', text:data.text, metaData:data.metaData});
    return db.Campaign.create({
        companyId: companyId,
        name: data.name,
        status: 'Scheduled',
        departmentCount: data.departmentCount,
        courseCount: (data.courseCount != 0 ? data.courseCount : null),
        quizCount: (data.quizCount != 0 ? data.quizCount : null),
        isManager : (data.isManager == 'yes' ? 1 : 0)
    });
}

var allProcess = (companyId, data) => {
    var simulationId = 0;
    simulation.createSimulation(data)
    .then(simulation => {
        if(simulation){
            simulationId = simulation.id;
            return db.GroupMember.findAll({ where: { groupId: data.groups }, include: [{model: db.Member, required: true, distinct: true}] })
        }else throw new RequestError('Simulation creation failed')
    })
    .then(groups => {
        if(groups.length > 0){
            queue.create('create-sessions', {
                simulationId: simulationId,
                groups: groups,
                results: data,
                isManager: data.isManager,
                companyId: companyId,
                scheduledAt: (data.scheduledAt ? data.scheduledAt : new Date() )
            }).priority('high').removeOnComplete(true).save();
            return groups;
        }else throw new RequestError('Invalid Department is specified for the campaign')
    })
    .then(groups => {
        return db.Campaign.update({ status: 'In Progress', memberCount: groups.length }, { where: { id: data.campaignId, companyId: companyId  },  });
    })
    .then(groups => {
        if(groups && data.quizCount > 0)
            return db.Quiz.update({ status: 'Published'},  { where : { id: { $in: data.quizzes }, companyId: companyId } })
        if(groups && data.courseCount > 0) 
            return db.Course.update({ status: 'Published'},  { where : { id: { $in: data.courses }, companyId: companyId } })
    })
    .then(r => {
        return true;
    })
    .catch(err => {
        db.Campaign.update({status: 'Error'}, { where: {id: data.campaignId} });
        if(data.campaignId){
            db.CampaignError.create({
                campaignId: data.campaignId,
                error: JSON.stringify(err)
            });
        }
        console.log('error - ' + err);
    })
}

export default {
    create : (companyId, data) => {
        return new Promise((resolve, reject) => {
            var type = data.type;
            data.departmentCount = data.groups.length;
            data.courseCount = (data.type == "course" ? data.courses.length : 0);
            data.quizCount = (data.type == "quiz" ? data.quizzes.length : 0);
            createCampaign(companyId, data)
            .then(campaign => {
                data.campaignId = campaign.id;
                var groupEntries = [];
                data.groups.forEach((group) => {
                    groupEntries.push({ campaignId: data.campaignId, groupId:group });
                });
                return db.CampaignGroup.bulkCreate(groupEntries)
            })
            .then(campaignGroups => {
                if(type == "course"){
                    var courseEntries = [];
                    data.courses.forEach((course) => { 
                        courseEntries.push({ campaignId: data.campaignId, courseId:course }); 
                    });
                    return db.CampaignCourse.bulkCreate(courseEntries)
                }else if(type == "quiz"){
                    var quizEntries = [];
                    data.quizzes.forEach((quiz) => {
                        quizEntries.push({ campaignId: data.campaignId, quizId:quiz });
                    });
                    return db.CampaignQuiz.bulkCreate(quizEntries)
                }
            })
            .then(r =>{
                allProcess(companyId, data);
                resolve(data.campaignId);
            })
            .catch(err => {
                db.Campaign.update({status: 'Error'}, { where: {id: data.campaignId} });
                if(data.campaignId){
                    db.CampaignError.create({
                        campaignId: data.campaignId,
                        error: JSON.stringify(err)
                    });
                }
                console.log('error - ' + err);
                reject(err);
            })
        });
    },

}