import nodemailer from 'nodemailer';
import config from './config';
import { db } from './models';
import templateFile from './krademy/template';
import { differentDateFormat, capitalizeFirstLetter } from "./functions";
import { queue } from './kue';
import Blowfish from 'egoroof-blowfish';
import path from 'path';
import fs from 'fs';
const _ = require('lodash'); 

var getTemplateType  = {
    course: '__Send_Course__',
    quiz: '__Send_Assessment__',
    completion: '__Course_Completion__',
    reminder: '__Course_Reminder__',
    quiz_fail: '__Course_Quiz_Fail_Reminder__'
}

const bf = new Blowfish(toString(config.app.secret), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);
const fromHexString = hexString => new Uint8Array(hexString.match(/.{1,2}/g).map(byte => parseInt(byte, 16)));

var storeErrorEmailsForReminder = async (email, course, simulation, reminder) => {
    let date = differentDateFormat(3);
    let filePath = appRoot + '/public/error/error-'+date+'.csv';
    if(!fs.existsSync(filePath)){
        fs.writeFileSync(filePath, `Email,Course,Simulation,Reminder`+ "\n");
    }
    let entry = [email, course, simulation, reminder];
    fs.appendFileSync(filePath, entry.join(',') + "\n");
}

var dcryptPassword = (password) => {
    return new Promise((resolve, reject) => {
        if(password && password.length) resolve(bf.decode(fromHexString(password)));
        else resolve(false);
    })
}

var clientTemplateCheck = function(companyId){
    return db.Company.findOne({ where:{ id:companyId }, attributes:['id', 'name', 'logo'], include:[{ model:db.MetaData, attributes:['key', 'value'], where:{ key: 'SDOMAIN'}, required:false }] })
    .then(company => {
        return company;          
    })
}

var getSender = function(companyId){
    return db.Company.findOne({ where:{ id:companyId } })
    .then(company => {
        if(company && company.smtpStatus == 'custom')
            return db.Sender.findOne({ where: { companyId:companyId }, order:[['id', 'DESC']] })
        else return db.Sender.findOne({ where: { companyId:null }  })
    })
    .then(sender => {
        return sender; 
    })
}

export default {
    getTemplateType,
    sendRegisterationLink: (email, companyKey) => {
        return new Promise((resolve, reject) => {
            try{
                db.User.findOne({where: {email: email}})
                .then((user) => {
                    if(user){
                            var key = Math.random().toString(36).slice(2);
                            db.MetaAuth.update({ verf_key : key }, { where: { email : user.email } })
                            .then(r => {
                                var smtpTransport = nodemailer.createTransport({
                                    host: config.mail.host,
                                    port: config.mail.port,
                                    auth: {
                                        user: config.mail.username,
                                        pass: config.mail.password
                                    },
                                    tls: {rejectUnauthorized: false},
                                });
                                smtpTransport.sendMail({
                                    from: config.mail.from,
                                    to: email,
                                    subject: 'Verify Your Account',
                                    html: '<!DOCTYPE html> <html> <head> <title>Verify account</title> </head> <body cz-shortcut-listen="true" style="margin: 0; padding: 0 !important; margin-top: 20px; mso-line-height-rule: exactly; color: black;" width="100%"> <center style="width: 100%;"> <div style="display: none; font-size: 1px;max-height: 0px; max-width: 0px; opacity: 0; overflow: hidden; mso-hide: all; font-family: sans-serif;">&nbsp;</div> <div class="email-container" style="max-width: 600px; margin: 0 auto;"> <img width="100px" src="'+config.app.url+'/static/images/lms-logo.png"> <div class="overlay">&nbsp;</div> <table align="center" border="0" cellpadding="0" cellspacing="0" role="presentation" style="margin: auto;border: 2px solid #eeeded; border-radius: 10px; padding:0px;" width="100%"> <tbody> <tr> <td class="hero bg_white"  valign="middle"> <table border="0" cellpadding="0" cellspacing="0" width="100%" > <tbody> <!-- center image --> <tr> <td style="text-align: center;"> <img style="margin-bottom: 0px; border-radius: 5px;" src="'+config.app.url+'/static/images/banner-account-verify.png"> </td> </tr> </tbody> </table> <table border="0" cellpadding="10" cellspacing="0" width="100%" > <tbody> <!-- paragraph --> <tr> <td style="text-align: center;"> <p style="font-size:16px;line-height:0px;font-family:Arial;margin-bottom: 28px;color: #4a4848;">Welcome to Krademy LMS!</p> <p style="font-size:20px;line-height:21px;font-family:Arial;margin-bottom:0px;">Thanks for registering with us.</p> </td> </tr> <!-- button --> <tr> <td style="text-align: center;"> <p style="border-top: 2px solid #00000021;"></p> <p style="font-size:16px; line-height: 54px;font-family:Arial;margin-bottom: 20px;color: #4a4848;">Please verify your account to get started</p> <a style="background: #f0803b; color:white; padding: 10px 20px;font-family:Arial; text-decoration: none; border-radius: 22px; letter-spacing: 0.3px;font-size: 16px;    font-weight: 600;" href="'+config.app.url+'/verify/'+email+'/'+key+'" target="_blank">Verify Account</a> </td> </tr> <tr> <td style="text-align: center;"> <p style="font-size:16px;line-height:21px;font-family:Arial;margin-bottom: 25px;">Need help?</p> <p style="font-size:16px;line-height:10px;font-family:Arial;margin-bottom: 15px;color: #4a4848;">Visit the Help Center or Contact us.</p> <p style="font-size:16px;line-height:21px;font-family:Arial;margin-bottom: 10px;"><a href="https://krademylms.com">www.krademylms.com</a></p> </td> </tr> <tr > <td style=" text-align: center;"> <p style="font-size:16px; margin-bottom: 25px; font-family:Arial;">This is a system-generated email, please don\'t reply to this message.</p> </td> </tr> <tr > <td style="border-top: 2px solid #00000021; text-align: center;"> <p style="font-size:16px; margin-bottom: 35px; font-family:Arial;line-height: 26px;">If you’re having trouble clicking the "Verify Account" button, copy and paste the URL below into your web browser:<a href="'+config.app.url+'/verify/'+email+'/'+key+'" style="text-decoration: none;"> '+config.app.url+'/verify/'+email+'/'+key+' </a></p> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </div> </center> </body> </html>' ,
                                }, function(error, info){
                                    if(error){
                                        return reject({
                                            name: "PRAException",
                                            msg: 'Email Sending Failed'
                                        })
                                    }
                                    return resolve(true)
                                });
                            })
                    }else throw {
                        name: "PRAException",
                        msg: 'Email Body not available'
                    }
                })
            }catch(err){
                reject(err);
            }
        });
    },
    sendResetLink: ( email , newPassword, headers) => {
        return new Promise(( resolve, reject) => {
            try{
                db.MetaAuth.findOne({ where : { email : email }})
                .then(meta => {
                    if(meta)
                        return db.MetaData.findOne({ where :{key:'SDOMAIN', value:headers.origin }, attributes:['key', 'value'] })
                    else throw new RequestError('Email not found.');
                })
                .then(metadata =>{
                    if(metadata) return getSender(metadata.companyId).then(r => [r, metadata])
                    else return db.Sender.findOne({ where: { companyId:null }  }).then(r => [r, metadata])
                })
                .then(([sender, metadata]) => {
                    if(metadata) return db.Company.findOne({ where :{ id: metadata.companyId  } }).then(r => [r, sender])
                    else return [false, sender];   
                })
                .then(([company, sender]) =>{
                    var emailconfig = {
                        tls: {rejectUnauthorized: false},
                        host: sender.host.split(':')[0],
                        port: sender.host.split(':')[1],
                    };
                    if(sender.pool){
                        emailconfig['pool'] = true;
                        emailconfig['maxMessages'] = sender.maxMessages;
                        emailconfig['maxConnections'] = sender.maxConnections;
                    }
                    if(sender.requiretls){
                        emailconfig['requireTLS'] = sender.requiretls
                    }else {
                        if((sender.ssl != null) && (sender.ssl == true || sender == false)){
                            emailconfig['ssl'] = sender.ssl
                        }
                    }
                    if(sender.connectionTimeout){
                        emailconfig['connectionTimeout'] = sender.connectionTimeout
                    }
                    if(sender.username){
                        emailconfig['auth'] = { user: sender.username, }
                        if(sender.password)
                            return dcryptPassword(sender.password).then(r => [r, emailconfig, sender.fromAddress, company])
                    }
                    return [false, emailconfig, sender.fromAddress, company]
                })
                .then(([password, emailconfig, fromAddress, company]) =>{
                    if(password) emailconfig.auth['pass'] = password;
                    var smtpTransport = nodemailer.createTransport(emailconfig);
                    var mailOptions = {
                        from: fromAddress,
                        to: email,
                        subject: 'Password Reset Request',
                        html: '<!DOCTYPE html> <html> <head> <title>Reset password</title> </head> <body cz-shortcut-listen="true" style="margin: 0; padding: 0 !important; margin-top: 20px; mso-line-height-rule: exactly; color: black;" width="100%"> <center style="width: 100%;"> <div style="display: none; font-size: 1px;max-height: 0px; max-width: 0px; opacity: 0; overflow: hidden; mso-hide: all; font-family: sans-serif;">&nbsp;</div> <div class="email-container" style="max-width: 600px; margin: 0 auto;"> <img width="100px" src="cid:lms-logo.png"> <div class="overlay">&nbsp;</div> <table align="center" border="0" cellpadding="0" cellspacing="0" role="presentation" style="margin: auto;border: 2px solid #eeeded; border-radius: 10px; padding:0px;" width="100%"> <tbody> <tr> <td class="hero bg_white"  valign="middle"> <table border="0" cellpadding="0" cellspacing="0" width="100%" > <tbody> <tr> <td style="text-align: center;"> <img style="margin-bottom: 0px; border-radius: 5px;" src="cid:banner-password-reset.png"> </td> </tr> </tbody> </table> <table border="0" cellpadding="10" cellspacing="0" width="100%" > <tbody> <!-- paragraph --> <tr> <td style="text-align: center;"> <p style="font-size:16px;line-height:15px;font-family:Arial;margin-bottom: 0px;color: #4a4848;">We have received a password reset request for your account.</p> </td> </tr> <!-- button --> <tr> <td style="text-align: center;"> <p style="border-top: 2px solid #00000021;"></p> <p style="font-size:20px; line-height: 54px;font-family:Arial;margin-bottom: 20px;color: #4a4848;">Click below to reset your password</p> <a style="background: #f0803b; color:white; padding: 10px 20px;font-family:Arial; text-decoration: none; border-radius: 22px; letter-spacing: 0.3px;font-size: 16px;    font-weight: 600;" href="'+(headers ? headers.origin : config.app.url)+'/reset/'+email+'/'+newPassword+'" target="_blank">Reset Password</a> </td> </tr>  <tr > <td style=" text-align: center;"> <p style="font-size:16px; margin-bottom: 25px; font-family:Arial;">This is a system-generated email, please don\'t reply to this message.</p> </td> </tr> <tr > <td style="border-top: 2px solid #00000021; text-align: center;"> <p style="font-size:16px; margin-bottom: 35px; font-family:Arial;">If you’re having trouble clicking the "Reset Password" button, copy and paste the URL below into your web browser:<a href="'+(headers ? headers.origin : config.app.url)+'/reset/'+email+'/'+newPassword+'" style="text-decoration: none;"> '+(headers ? headers.origin : config.app.url)+'/reset/'+email+'/'+newPassword+'</a></p> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </div> </center> </body> </html>',
                    };
                    mailOptions['attachments'] = [];
                    mailOptions['attachments'].push({
                        filename: (company ? company.logo : 'lms-logo.png'),
                        cid: (company ? company.logo : 'lms-logo.png'),
                        path: (company  ? path.join(config.storage.folder, config.storage.company_logo, company.logo) : path.join(__dirname, 'public', 'images', 'lms-logo.png'))
                    })
                    mailOptions['attachments'].push({
                        filename: 'banner-password-reset.png',
                        cid: 'banner-password-reset.png',
                        path: path.join(__dirname, 'public', 'images', 'banner-password-reset.png')
                    })
                    smtpTransport.sendMail(mailOptions, function(error, info){
                        if(error) reject('Email Sending Failed');
                        return resolve(true)
                    });
                })
                .catch(err =>{
                    console.log(err);
                })
            }catch(err){
                console.log("TCL: err", err)
                reject(err);
            }
        });
    },

    sendEnrollLink: async(mailData, simulationId, companyId, type) => { //2
        return new Promise((resolve, reject) => {
            try{
                getSender(companyId)
                .then(sender => {
                    if(sender){
                        var emailconfig = {
                            tls: {rejectUnauthorized: false},
                            host: sender.host.split(':')[0],
                            port: sender.host.split(':')[1],
                        };
                        if(sender.pool){
                            emailconfig['pool'] = true;
                            emailconfig['maxMessages'] = sender.maxMessages;
                            emailconfig['maxConnections'] = sender.maxConnections;
                        }
                        if(sender.requiretls){
                            emailconfig['requireTLS'] = sender.requiretls
                        }else {
                            if((sender.ssl != null) && (sender.ssl == true || sender == false)){
                                emailconfig['ssl'] = sender.ssl
                            }
                        }
                        if(sender.connectionTimeout){
                            emailconfig['connectionTimeout'] = sender.connectionTimeout
                        }
                        if(sender.username){
                            emailconfig['auth'] = { user: sender.username, }
                            if(sender.password)
                                return dcryptPassword(sender.password).then(r => [r, emailconfig, sender.fromAddress])
                        }
                        return [false, emailconfig, sender.fromAddress];
                    }else reject(new RequestError('SMTP Profile is not set.'))
                })
                .then(([password, emailconfig, fromAddress]) => {
                    if(password) emailconfig.auth['pass'] = password;
                    return db.Templates.findOne({ where : { name: getTemplateType[type], type: type}}).then(r =>[r, emailconfig, fromAddress])
                })
                .then(([template, emailconfig, fromAddress]) => {
                    if(template)
                        return db.MetaData.findOne({ where: { companyId: companyId, key: template.name}, attributes:['value'] }).then(r =>[r, template, emailconfig, fromAddress])
                    else reject(new RequestError('No Template Selected.'))
                })
                .then(([metadata, template, emailconfig, fromAddress]) => {
                    return clientTemplateCheck(companyId).then(r =>[r, metadata, template, emailconfig, fromAddress])
                })
                .then(([company, metadata, template, emailconfig, fromAddress]) => {
                    var customData = (metadata != null ? JSON.parse(metadata.value) : '');
                    if(mailData && Object.keys(mailData).length > 0){
                        // var data = Object.keys(mailData);
                        var smtpTransport = nodemailer.createTransport(emailconfig);
                        // for(var i = 0; i < data.length; i++){
                            Object.keys(mailData).forEach(result => {
                                result = mailData[result];
                                if(result.subscribe && result.email && result.memberId){
                                    var mailContent = templateFile.generateTemplateHtml(result, template, company, companyId, customData);
                                    var mailOptions = {
                                        from: fromAddress,
                                        to: result.email,
                                        subject: (customData != '' && customData.enable ? ( companyId == 260 ? 'Hi '+capitalizeFirstLetter(result.firstName)+ (result.type == 'course' ? '. Welcome to '+(company.name && company.name != '' ? company.name : 'Krademy LMS')+' Course Enrollment' : '. Welcome to Your Assessment') : customData.subject):  'Hi '+capitalizeFirstLetter(result.firstName)+(result.type == 'course' ? ', Welcome to '+(company.name && company.name != '' ? company.name : 'Krademy LMS')+(companyId == 71 ? ' CyberSecurity Awareness Programme' : ' Course Enrollment') : ', Welcome to Your Assessment') ),
                                        html: mailContent,
                                    };
                                    mailOptions['attachments'] = [];
                                    if(customData != '' && customData.enable  && customData.type == 'image' && !(_.isEmpty(customData.filename)) ){
                                        mailOptions['attachments'].push({
                                            filename: customData.filename,
                                            cid: customData.filename,
                                            path: path.join(config.storage.folder, config.storage.template_custom, customData.filename)
                                        })
                                    }else {
                                        for(var i =0;i<result.courseOrQuiz.length;i++){
                                            mailOptions['attachments'].push({
                                                filename: result.courseOrQuiz[i].image,
                                                cid: result.courseOrQuiz[i].image,
                                                path: path.join(config.storage.folder, (result.type == 'course' ? config.storage.course_logo : config.storage.quiz_logo), result.courseOrQuiz[i].image)
                                            });
                                        }
                                        mailOptions['attachments'].push({
                                            filename: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                                            cid: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                                            path: (company.logo && company.logo != ''  ? path.join(config.storage.folder, config.storage.company_logo, company.logo) : path.join(__dirname, 'public', 'images', 'lms-logo.png'))
                                        })
                                    }
                                    console.log('Ready to send email:', result.email);
                                    smtpTransport.sendMail(mailOptions, function(error, info) {
                                        if (error || (info && info.rejected.length)) {
                                            console.log("Error : " + JSON.stringify(error));
                                            queue.create('session', { status: "Error", sessionId: result.sessionId }).priority('high').removeOnComplete(true).save();
                                            queue.create('session-event', { status: "Error", sessionId: result.sessionId, simulationId: simulationId, details:JSON.stringify(error) }).priority('high').removeOnComplete(true).save();
                                        }else{
                                            console.log('Message sent: ' + JSON.stringify(info));
                                            queue.create('session', { status: "Email Sent", sessionId: result.sessionId }).priority('high').removeOnComplete(true).save();
                                            queue.create('session-event', { status: "Email Sent", sessionId: result.sessionId, simulationId: simulationId, details:JSON.stringify(info) }).priority('high').removeOnComplete(true).save();
                                        }
                                    })
                                }
                            })
                        // }
                        resolve(true);
                    }else reject(new RequestError('No Data received!'))
                })
                .catch(err => {
                    console.log("Mailer Error:", err);
                    reject(err);
                })
            }catch(err){
                console.log("Mailer Error:", err);
                reject(err);
            }
        })
    },

    sendCourseReminderLink: (email, data, company, template, customData, fromAddress, smtpTransport, simulationId, reminderId) => { //3 done
        return new Promise((resolve, reject) => {
            try{
                var mailContent = templateFile.generateCourseReminderHtml(data, template, company, customData);
                var mailOptions = {
                    from: fromAddress,
                    to: email,
                    subject: (customData != '' && customData.enable ? customData.subject : 'Course Reminder!  '+(data.percent == 0 ? 'Please Begin your ' : 'Pursue your remaining ' )+'"'+data.courseName+'"  Course '),
                    html: mailContent,
                };
                mailOptions['attachments'] = [];
                if(customData != '' && customData.enable  && customData.type == 'image' && !(_.isEmpty(customData.filename)) ){
                    mailOptions['attachments'].push({
                        filename: customData.filename,
                        cid: customData.filename,
                        path: path.join(config.storage.folder, config.storage.template_custom, customData.filename)
                    })
                }else {
                    mailOptions['attachments'].push({
                        filename: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                        cid: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                        path: (company.logo && company.logo != ''  ? path.join(config.storage.folder, config.storage.company_logo, company.logo) : path.join(__dirname, 'public', 'images', 'lms-logo.png'))
                    })
                }
                console.log("Ready to Send Course Reminder Mail:---------------------"+ email);
                smtpTransport.sendMail(mailOptions, function(error, info){
                    if(error){
                        storeErrorEmailsForReminder(email, data.courseId, simulationId, reminderId);
                        console.log("Message Error:- "+ error)
                        reject('Email Sending Failed');
                    }else{
                        console.log("Message Sent:- "+ JSON.stringify(info))
                        return resolve(true)
                    } 
                });
                return resolve(true)  
            }catch(err){
                reject(err);
            }
        });
    },

    sendMailToManagerForRemainingCourseReminder: (mailData, managerData, file) => { //5
        return new Promise((resolve, reject) => {
            try{
                getSender(mailData.companyId)
                .then(sender => {
                    if(sender){
                        var emailconfig = {
                            tls: {rejectUnauthorized: false},
                            host: sender.host.split(':')[0],
                            port: sender.host.split(':')[1],
                        };
                        if(sender.requiretls){
                            emailconfig['requireTLS'] = sender.requiretls
                        }else {
                            if((sender.ssl != null) && (sender.ssl == true || sender == false)){
                                emailconfig['ssl'] = sender.ssl
                            }
                        }
                        if(sender.username){
                            emailconfig['auth'] = { user: sender.username, }
                            if(sender.password)
                                return dcryptPassword(sender.password).then(r => [r, emailconfig, sender.fromAddress])
                        }
                        return [false, emailconfig, fromAddress]
                    }else reject(new RequestError('SMTP Profile is not set.'));
                })
                .then(([password, emailconfig, fromAddress]) => {
                    if(password) emailconfig.auth['pass'] = password;
                    var html = '<!DOCTYPE html> <html> <body>Dear '+managerData.name+', <br><br>Greetings from Krademy LMS! <br>This is a reminder for the course you assigned to your team members. <br><br>We are sending you the learner\'s report who are not meeting the course timeline requirement. <br>Please find the attached CSV file to know their current progress with the course. </body> </html>'
                    var smtpTransport = nodemailer.createTransport(emailconfig);
                    smtpTransport.sendMail({
                        from: fromAddress,
                        to: managerData.email,
                        subject: "Course Reminder! || "+mailData.courseName+" || "+differentDateFormat(4, mailData.scheduledAt)+" || "+mailData.percent+"%", 
                        html: html,
                        attachments: [{
                            filename: 'members-'+mailData.percent+'-percentage'+'.csv',
                            path: file
                        }]
                    }, function(error, info){
                        if(fs.existsSync(file))
                            fs.unlinkSync(file);
                        if(error || (info && info.rejected.length)){
                            console.log("Error : " + JSON.stringify(error));
                            return false;
                        }
                        console.log('Message Sent:- '+JSON.stringify(info));
                        resolve(true)
                    })
                })
            }catch(err){
                reject(err);
            }
        });
    },

    sendMailToManagerForCompleteCourseReminder: (mailData, groups, simulationId) => { //6
        var l = Object.keys(groups);
        return new Promise((resolve, reject) => {
            try{
                db.Campaign.findOne({ attributes:['id', 'companyId'], include:[{ model:db.Simulation, required:true, where:{ id: simulationId } }] })
                .then(campaign => {
                    if(campaign)
                        return getSender(campaign.companyId);
                    else reject(new RequestError('No SMTP Profile Found'));
                })
                .then(sender => {
                    if(sender){
                        var emailconfig = {
                            tls: {rejectUnauthorized: false},
                            host: sender.host.split(':')[0],
                            port: sender.host.split(':')[1],
                        };
                        if(sender.requiretls){
                            emailconfig['requireTLS'] = sender.requiretls
                        }else {
                            if((sender.ssl != null) && (sender.ssl == true || sender == false)){
                                emailconfig['ssl'] = sender.ssl
                            }
                        }
                        if(sender.username){
                            emailconfig['auth'] = { user: sender.username, }
                            if(sender.password)
                                return dcryptPassword(sender.password).then(r => [r, emailconfig, sender.fromAddress])
                        }
                        return [false, emailconfig, fromAddress]
                    }else reject(new RequestError('SMTP Profile is not set.'));
                })
                .then(([password, emailconfig, fromAddress]) => {
                    var smtpTransport = nodemailer.createTransport(emailconfig);
                    for(var i = 0; i < l.length; i++){
                        var html = '<!DOCTYPE html> <html> <body>Dear '+groups[l[i]].name+', <br><br>Greetings from Krademy LMS! <br>This is a reminder for the course you assigned to your team members. <br><br>All your members successfully completed the course within the timeline.  </body> </html>'
                        smtpTransport.sendMail({
                            from: fromAddress,
                            to: groups[l[i]].email,
                            subject: "Course Reminder! || "+mailData.courseName+" || "+differentDateFormat(4, mailData.scheduledAt)+" || "+mailData.percent+"%", 
                            html: html
                        }, function(error, info){
                            if(error){
                                console.log(error);
                            }
                            console.log(JSON.stringify(info));
                            resolve(true)
                        })
                    }
                });

            }catch(err){
                reject(err);
            }
        });
    },

    sendCourseCompletion: (coursePercentage, completedAt, courseId, id, type) => { //4
        return new Promise((resolve, reject) => {
            try{
                db.Member.findOne({ where : { id: id }})
                .then(member => {
                    if(member) return db.MetaAuth.findOne({ where : { email : member.email } , include : [{ model : db.Member, required:true }] }).then(r => [r, member])
                    else throw new RequestError('Invalid Member received');
                })
                .then(([meta, member]) => {
                    if(meta) return db.Course.findOne({ where: { id: courseId } }).then(r => [r, member]);
                    else throw new RequestError('Invalid Member received');
                })
                .then(([course, member]) => {
                    if(course) return db.MemberCourse.findOne({ where: { courseId: course.id, memberId: member.id }, include:[{ model:db.Session }] }).then(r => [r, member, course]);
                    else throw new RequestError('Invalid Course received');
                })
                .then(([memberCourse, member, course]) => {
                    if(memberCourse){
                        return  db.MetaData.findOne({ where: { companyId: course.companyId, key: getTemplateType[type]}, attributes:['value'] }).then(r => [r, memberCourse, member, course])
                    }else throw new RequestError('Invalid Member Course received');
                })
                .then(([metadata, memberCourse, member, course]) => {
                    var customData = (metadata != null ? JSON.parse(metadata.value) : '');
                    if(memberCourse.completedAt == "" || memberCourse.completedAt == null)
                        return db.Templates.findOne({ where : { name: getTemplateType[type], type: type} }).then(r => [r, true, memberCourse, member, course, customData])
                    else return [true, false, memberCourse, member, course, customData];
                })
                .then(([template, check, memberCourse, member, course, customData]) => {
                    if(template){
                        return clientTemplateCheck(course.companyId).then(r => [r, template, check, memberCourse, member, course, customData])
                    }else throw new RequestError('Invalid Template received');
                })
                .then(([company, template, check, memberCourse, member, course, customData]) => {
                    if(check) return db.MemberCourse.update({ percent: 100, completedAt: completedAt }, {where: { id: memberCourse.id, courseId: course.id, memberId: member.id } }).then(r => [true, check, template, member, course, company, memberCourse, customData])
                    else return [false, check, template, member, course, company, memberCourse, customData]
                })
                .then(([result, check, template, member, course, company, memberCourse, customData]) => {
                    company['companyId'] = course.companyId;
                    if(result && check){
                        var data = {
                            firstName: member.firstName,
                            lastName: member.lastName,
                            email: member.email,
                            courseName: course.name,
                            courseId:course.id,
                            sessionKey: memberCourse.Session.sessionKey,
                            url: '/course-auth?'
                        }
                        return getSender(course.companyId).then(r =>[r, data, template, result, company, customData]); 
                    }else return [false, data, template, result, company, customData];
                })
                .then(([sender, data, template, result, company, customData]) => {
                    if(sender){
                        var emailconfig = {
                            tls: {rejectUnauthorized: false},
                            host: sender.host.split(':')[0],
                            port: sender.host.split(':')[1],
                        };
                        if(sender.requiretls){
                            emailconfig['requireTLS'] = sender.requiretls
                        }else {
                            if((sender.ssl != null) && (sender.ssl == true || sender == false)){
                                emailconfig['ssl'] = sender.ssl
                            }
                        }
                        if(sender.username){
                            emailconfig['auth'] = { user: sender.username, }
                            if(sender.password)
                                return dcryptPassword(sender.password).then(r => [r, emailconfig, sender.fromAddress, data, template, result, sender, company, customData])
                        }
                        return [false, emailconfig, sender.fromAddress, data, template, result, sender, company, customData];
                    }else return [false, emailconfig, sender.fromAddress, data, template, result, sender, company, customData];
                })
                .then(([password, emailconfig, fromAddress, data, template, result, sender, company, customData]) => {
                    if(result){
                        if(sender){
                            if(password) emailconfig.auth['pass'] = password;
                            var smtpTransport = nodemailer.createTransport(emailconfig);
                            var mailContent = templateFile.generateCourseCompletionHtml(data, template, company, customData);
                            var mailOptions = {
                                from: fromAddress,
                                to: data.email,
                                subject: (customData != '' && customData.enable ? customData.subject : capitalizeFirstLetter(data.firstName)+ ', Download Your Certificate for the '+'"'+data.courseName+'" Course'),
                                html: mailContent,
                            };
                            mailOptions['attachments'] = [];
                            if(customData != '' && customData.enable  && customData.type == 'image' && !(_.isEmpty(customData.filename)) ){
                                mailOptions['attachments'].push({
                                    filename: customData.filename,
                                    cid: customData.filename,
                                    path: path.join(config.storage.folder, config.storage.template_custom, customData.filename)
                                })
                            }else{
                                mailOptions['attachments'].push({
                                    filename: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                                    cid: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                                    path: (company.logo && company.logo != ''  ? path.join(config.storage.folder, config.storage.company_logo, company.logo) : path.join(__dirname, 'public', 'images', 'lms-logo.png'))
                                })
                            }
                            smtpTransport.sendMail(mailOptions, function(error, info){
                                if(error) reject('Email Sending Failed');
                                return resolve(true)
                            });
                        }else reject(new RequestError('SMTP Profile is not set.'));
                    }else return resolve(true)
                })
            }catch(err){
                reject(err);
            }
        });
    },

    CourseQuizFails: (courseUserDetails, lectureName, courseId, id, type) => { //7
        return new Promise((resolve, reject) => {
            try{
                db.Member.findOne({ where : { id: id }})
                .then(member => {
                    if(member){
                        if(member.reminderMails){
                            db.MetaAuth.findOne({ where : { email : member.email } , include : [{ model : db.Member, required:true }] }).then(r => [r, member])
                            .then(([meta, member]) => {
                                if(meta) return db.Course.findOne({ where: { id: courseId } }).then(r => [r, member]);
                                else throw new RequestError('Invalid Member received');
                            })
                            .then(([course, member]) => {
                                if(course){
                                    return db.Templates.findOne({  where : { name: getTemplateType[type], /* type: 'quiz_fail' */} }).then(r => [r, member, course])
                                } else throw new RequestError('Invalid Member Course received');
                            })
                            .then(([template, member, course]) => {
                                if(template){
                                    return clientTemplateCheck(course.companyId).then(r => [r, template, member, course])
                                } else throw new RequestError('Invalid Template received');
                            })
                            .then(([company, template, member, course]) => {
                                company['companyId'] = course.companyId;
                                var data = {
                                    firstName: member.firstName,
                                    lastName: member.lastName,
                                    email: member.email,
                                    courseName: course.name,
                                    lectureName: lectureName,
                                    sessionKey: courseUserDetails.Session.sessionKey,
                                    code: member.code,
                                    courseOrQuiz: [ {id: course.id,  name: course.name, image: course.logo }]
                                }
                                return getSender(course.companyId).then(r =>[r, data, template, company])
                            })
                            .then(([sender, data, template, company]) => {
                                if(sender){
                                    var emailconfig = {
                                        tls: {rejectUnauthorized: false},
                                        host: sender.host.split(':')[0],
                                        port: sender.host.split(':')[1],
                                    };
                                    if(sender.requiretls){
                                        emailconfig['requireTLS'] = sender.requiretls
                                    }else {
                                        if((sender.ssl != null) && (sender.ssl == true || sender == false)){
                                            emailconfig['ssl'] = sender.ssl
                                        }
                                    }
                                    if(sender.username){
                                        emailconfig['auth'] = { user: sender.username, }
                                        if(sender.password)
                                            return dcryptPassword(sender.password).then(r => [r, emailconfig, sender.fromAddress, data, template, company])
                                    }
                                    return [false, emailconfig, sender.fromAddress, data, template, company];
                                }else reject(new RequestError('SMTP Profile is not set.'));
                            })
                            .then(([password, emailconfig, fromAddress, data, template, company]) => {
                                if(password) emailconfig.auth['pass'] = password;
                                return db.MetaData.findOne({ where: { companyId: company.companyId, key: template.name}, attributes:['value'] }).then(r =>[r, emailconfig, fromAddress, data, template, company])
                            })
                            .then(([metadata, emailconfig, fromAddress, data, template, company]) => {
                                var customData = (metadata != null ? JSON.parse(metadata.value) : '');
                                var smtpTransport = nodemailer.createTransport(emailconfig);
                                var mailContent = templateFile.generateTemplateForCourseQuizFails(data, template, company, customData);
                                var mailOptions = {
                                    from: fromAddress,
                                    to: data.email,
                                    subject:  (customData != '' && customData.enable ? customData.subject : capitalizeFirstLetter(data.firstName)+ ', Your Quiz Result Is Here!'),
                                    html: mailContent,
                                };
                                mailOptions['attachments'] = [];
                                if(customData != '' && customData.enable  && customData.type == 'image' && !(_.isEmpty(customData.filename))){
                                    mailOptions['attachments'].push({
                                        filename: customData.filename,
                                        cid: customData.filename,
                                        path: path.join(config.storage.folder, config.storage.template_custom, customData.filename)
                                    })
                                }else{
                                    for(var i =0;i<data.courseOrQuiz.length;i++){
                                        mailOptions['attachments'].push({
                                            filename: data.courseOrQuiz[i].image,
                                            cid: data.courseOrQuiz[i].image,
                                            path: path.join(config.storage.folder, config.storage.course_logo, data.courseOrQuiz[i].image)
                                        });
                                    } 
                                    mailOptions['attachments'].push({
                                        filename: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                                        cid: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                                        path: (company.logo && company.logo != ''  ? path.join(config.storage.folder, config.storage.company_logo, company.logo) : path.join(appRoot, 'public', 'images', 'lms-logo.png'))
                                    })
                                }
                                smtpTransport.sendMail(mailOptions, function(error, info){
                                    if(error) reject('Email Sending Failed');
                                    return resolve(true)
                                });
                            })
                        }else return resolve(true)  
                    }else throw new RequestError('Invalid Member received');
                    
                })
            }catch(err){
                reject(err);
            }
        });
    },

    resendFailureMail: (data,metadata,template,company,smtpTransport,fromAddress) => { //7
        return new Promise((resolve, reject) => {
            try{        
                let customData = (metadata != null ? JSON.parse(metadata.value) : '');
                let mailContent = templateFile.generateTemplateForCourseQuizFails(data, template, company, customData);
                let mailOptions = {
                    from: fromAddress,
                    to: data.email,
                    subject: (customData != '' && customData.enable ? customData.subject : capitalizeFirstLetter(data.firstName)+ ', Your Quiz Result Is Here!'),
                    html: mailContent,
                };
                mailOptions['attachments'] = [];
                if(customData != '' && customData.enable  && customData.type == 'image' && !(_.isEmpty(customData.filename))){
                    mailOptions['attachments'].push({
                        filename: customData.filename,
                        cid: customData.filename,
                        path: path.join(config.storage.folder, config.storage.template_custom, customData.filename)
                    })
                }else{
                    for(var i =0;i<data.courseOrQuiz.length;i++){
                        mailOptions['attachments'].push({
                            filename: data.courseOrQuiz[i].image,
                            cid: data.courseOrQuiz[i].image,
                            path: path.join(config.storage.folder, config.storage.course_logo, data.courseOrQuiz[i].image)
                        });
                    }
                    mailOptions['attachments'].push({
                        filename: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                        cid: (company.logo && company.logo != '' ? company.logo : 'lms-logo.png'),
                        path: (company.logo && company.logo != ''  ? path.join(config.storage.folder, config.storage.company_logo, company.logo) : path.join(appRoot, 'public', 'images', 'lms-logo.png'))
                    })
                }
                smtpTransport.sendMail(mailOptions, function(error, info){
                    if(error){
                        console.log('Message Error: ' + JSON.stringify(error));
                        reject('Email Sending Failed');
                    }else {
                        console.log('Message sent: ' + JSON.stringify(info));
                        return resolve(true)
                    }
                });
            }catch(err){
                reject(err);
            }
        });
    },

    
    freeTrial: (data) => {
        return new Promise((resolve, reject) => {
            try{
                var html = '<!DOCTYPE html> <html> <body> <label>Firstname:</label> <span>'+(data.firstName?data.firstName:'')+'</span> <br> <label>Lastname:</label> <span>'+(data.lastName?data.lastName:'')+'</span> <br> <label>Email:</label> <span>'+data.email+'</span> <br> <label>Source:</label> <span>'+data.source+'</span> <br> <label>URL:</label>  <span>'+(data.URL?data.URL:'')+'</span>  <br> </body> </html>'
                var smtpTransport = nodemailer.createTransport({
                    host: config.mail.host,
                    port: config.mail.port,
                    auth: {
                        user: config.mail.username,
                        pass: config.mail.password
                    },
                    tls: {rejectUnauthorized: false},
                });
                smtpTransport.sendMail({
                    from: config.mail.from,
                    to: config.mail.to.split(','),
                    subject: "New Lead reported from Free Trial.", 
                    html: html
                }, function(error, info){
                    if(error){
                        console.log(error);
                    }
                    console.log(JSON.stringify(info));
                    resolve(true)
                })
            } catch(err) {
                resolve(true);
            }
        });
    },

    mailSendToUserForFreeTrial: (data) => {
        return new Promise((resolve, reject) => {
            try{
                var html = '<!Doctype html> <html> <body> <p> Dear <b>'+(data ? data.firstName+' '+(data.lastName ? data.lastName : '') : 'Guest')+'</b>, </p> <h2>Greetings from Krademy LMS!</h2> <p>We are delighted to have you on board with us. Kickstart your <b>LMS Free Trial Pack</b>.</p> <p>In the "Free Trial" package, you will get to enjoy all the LMS features free for upto 5 users.</p> <p>Krademy LMS team takes pleasure in delivering well-defined customer resolution. That\'s why we are here to help you in obtaining the maximum benefit from your eLearning platform.</p> <p>To acquire a better understanding of the various features of our LMS and what it can offer to your business, it will be helpful to schedule a Skype/Phone call. Please, let us know what time works for you to have a quick chat.</p> <p>Thank you for being a part of Krademy LMS.</p> <p><b>Enjoy!</b></p></br> <div>Krademy LMS</div> <div>Email- <a href="mailto:support@krademy.com" target="_blank">support@krademy.com</a></div> <div>Phone no- 9319693668</div> <div>Website- <a href="https://krademylms.com/" target="_blank">https://krademylms.com/</a></div> <p>P.S. Do not hesitate to contact me for any suggestions or queries on (support/ helpdesk for customers to leave queries or email ID).</p> <div><b>Best Regards</b></div> <div><b>Krademy LMS Team</b></div> </body> </html>'
                var smtpTransport = nodemailer.createTransport({
                    host: config.mail.host,
                    port: config.mail.port,
                    auth: {
                        user: config.mail.username,
                        pass: config.mail.password
                    },
                    tls: {rejectUnauthorized: false},
                });
                smtpTransport.sendMail({
                    from: config.mail.from,
                    to: data.email,
                    subject: "Welcome to Krademy LMS", 
                    html: html
                }, function(error, info){
                    if(error){
                        console.log(error);
                    }
                    console.log(JSON.stringify(info));
                    resolve(true)
                })
            } catch(err) {
                resolve(true);
            }
        });
    },

    sendAppSumoUsers: (userId, code) => {
        return new Promise((resolve, reject) => {
            try{
               db.User.findOne({ where: { id:userId } })
               .then(data => {
                    var html = '<!DOCTYPE html> <html> <body> <label>Firstname:</label> <span>'+(data.firstName?data.firstName:'')+'</span> <br> <label>Lastname:</label> <span>'+(data.lastName?data.lastName:'')+'</span> <br> <label>Email:</label> <span>'+data.email+'</span> <br> <label>Appsumo Code:</label> <span>'+code+'</span> </body> </html>';
                    var smtpTransport = nodemailer.createTransport({
                        host: config.mail.host,
                        port: config.mail.port,
                        auth: {
                            user: config.mail.username,
                            pass: config.mail.password
                        },
                        tls: {rejectUnauthorized: false},
                    });
                    smtpTransport.sendMail({
                        from: config.mail.from,
                        to: config.mail.to.split(','),
                        subject: "New AppSumo Lead", 
                        html: html
                    }, function(error, info){
                        if(error){
                            return reject({
                                name: "PRAException",
                                msg: 'Email Sending Failed'
                            })
                        }
                        return resolve(true)
                    })
               })
            }catch(err){
                resolve(true);
            }
        });
    },
    
    sendTestMail: (content) => {
        return new Promise((resolve, reject) => {
            try{
                var headers = new Object();
                for(let i = 0; content && content.header && i < content.headers.length; i++){
                    headers[content.headers[i].key] = content.headers[i].value;
                }
                var pass = '';
                if(content.check) pass = content.password;
                dcryptPassword(pass)
                .then( password => {
                    var emailconfig = {
                        host: content.host.split(':')[0],
                        port: content.host.split(':')[1],
                        tls: {rejectUnauthorized: false},
                    };
                    if(content.username){
                        emailconfig['auth'] = {
                            user: content.username,
                            pass: password?password:content.password
                        }
                    }
                    console.log(content.requiretls);
                    if(content.requiretls){
                        emailconfig['requireTLS'] = content.requiretls
                    }else {
                        if((content.ssl != null) && (content.ssl == true || content == false)){
                            emailconfig['ssl'] = content.ssl
                        }
                    }
                    console.log(content, emailconfig);
                    var smtpTransport = nodemailer.createTransport(emailconfig);
                    var mailOptions = {
                        from: content.fromAddress,
                        to: content.email,
                        subject: config.app.name + ' - Test Email',
                        html: "This is a test to verify that your specified SMTP Profile is working!",
                    };
                    if(content && content.header && content.headers.length){
                        mailOptions['headers'] = headers;
                    }
                    smtpTransport.sendMail(mailOptions, function(error, info){
                        if(error || (info && info.rejected.length)){
                            console.log(error);
                            return reject(new RequestError("Invalid SMTP Credentails"))
                        }
                        return resolve(true)
                    })
                })
                .catch(err => {
                    reject(err);
                })
            }catch(err){
                reject(err);
            }
        });
    },

    mailSendToAppsumoUserRoles: (data, adminData) => {
        return new Promise((resolve, reject) => {
            try{
                var html = '<!Doctype html> <html> <body> <p>Hello '+data.firstName+' '+(data.lastName && data.lastName != '' ? data.lastName: '')+',</p> <p><b>Greetings of the day!</b></p> <p>You have been assigned as the “Admin” of Krademy LMS by '+adminData.firstName+' '+(adminData.lastName && adminData.lastName != '' ? adminData.lastName: '')+'. </p> <p>You can now acquire the perks of being an Admin of the Krademy LMS portal.</p> <p>To access your LMS Admin Dashboard, log in to your Krademy LMS portal.</p> <p><a href="https://app.krademylms.com/login" target="_blank">Click here</a> to Login.</p><br> <p>Regards,</p> <p><b>Krademy LMS Team</b></p> </body> </html>'
                var smtpTransport = nodemailer.createTransport({
                    host: config.mail.host,
                    port: config.mail.port,
                    auth: {
                        user: config.mail.username,
                        pass: config.mail.password
                    },
                    tls: {rejectUnauthorized: false},
                });
                smtpTransport.sendMail({
                    from: config.mail.from,
                    to: data.email,
                    subject: "Admin Access to Krademy LMS Portal", 
                    html: html
                }, function(error, info){
                    if(error){
                        console.log(error);
                    }
                    console.log(JSON.stringify(info));
                    resolve(true)
                })
            } catch(err) {
                resolve(true);
            }
        });
    },
    sendAppSumoWelcome: (data) => {
        return new Promise((resolve, reject) => {
            try{
                var html = '<!DOCTYPE html> <html> <head> <title></title> </head> <body cz-shortcut-listen="true" style="margin: 0; padding: 0 !important; margin-top: 20px; mso-line-height-rule: exactly; color: black;" width="100%"> <center style="width: 100%;"> <div style="display: none; font-size: 1px;max-height: 0px; max-width: 0px; opacity: 0; overflow: hidden; mso-hide: all; font-family: sans-serif;">&nbsp;</div> <div class="email-container" style="max-width: 600px; margin: 0 auto;border: 1px solid #d9d9d9;"> <table align="center" border="0" cellpadding="0" cellspacing="0" role="presentation" style="margin: auto; padding:0px;background: #f4f4f4;" width="100%"> <tbody> <tr> <td class="hero bg_white"  valign="middle"> <table border="0" cellpadding="0" cellspacing="0" width="100%" > <tbody> <!-- center image --> <!--  <div class="overlay">&nbsp;</div> --> <tr> <td style="text-align: center;"> <br><br> <img width="120px" style="margin-bottom: 0px; border-radius: 5px;" src="https://www.krademylms.com/assets/img/logo.png"> <br> </td> </tr> </tbody> </table> <table border="0" cellpadding="20" cellspacing="0" width="100%" > <tbody> <!-- paragraph --> <tr> <td style="text-align: center;"> <p style="font-size:26px;margin-bottom:6px;font-family:Arial;color:black;margin-top:0px;"><b>Hello '+data.firstName+' '+(data.lastName && data.lastName !='' ? data.lastName: '')+',</b></p> <p style="font-size:16px;line-height:26px;font-family:Arial;margin-bottom:25px;color:black;margin-top: 0px;">Thank you for signing up at Krademy LMS <br>Get access to your admin dashboard, please click below-</p> <a style="background: #0f56ac; color:white;padding: 12px 40px;font-family:Arial; text-decoration: none; border-radius:4px; letter-spacing: 0.3px;font-size: 16px; font-weight: 600;" href="https://app.krademylms.com/login" target="_blank">Click Here</a> </td> </tr> </tbody> </table> <tr> <td height="10">&nbsp;</td> </tr> <table border="0" cellpadding="10" cellspacing="0" max-width="100%" bgcolor="#f4f4f4" width="100%"> <tbody> <!-- center image --> <tr style="background-image: linear-gradient(rgb(15, 86, 172) 0, rgba(16, 31, 49, 0.7) 100%);opacity: 2.9;"> <td align="center"> <p style="font-size:24px; line-height:26px;font-family:Arial;margin-bottom:4px; margin-top:0px;color:black; color: #fff;text-align: center;">Take a video tour</p> <a href="https://drive.google.com/file/d/1l-wMUf8exTLx6J-EmoibPoY1DyU03vOe/view?usp=sharing" target="_blank" ><img src="https://awareness.threatcop.ai/krademyvideo.png" style="width:60%;max-width:100%;object-fit:cover;padding:0px;outline:0px;border:none;display:block" width="308" class="CToWUd"></a> </td> </tr> <tr> <tr> <td style="text-align: center;"> <p style="font-size:15px; line-height:24px;font-family:Arial;margin-bottom:2px; margin-top:10px;color:black; color:#000000;">  Review us on the <a href="https://appsumo.com/krademy-lms/" target="_blank" style="text-decoration:none;color:#3563ef;">deal page</a> to get an extra code. Hurry Now!</p> <p style="font-size:15px; line-height:24px;font-family:Arial;margin-bottom:2px; margin-top: 0px;color:black; color:#000000;">Reach out to us for any assistance, email us at <a href="mailto:care@krademylms.com" style="text-decoration: none;">care@krademylms.com</a></p> <p style="font-size:15px; line-height:24px;font-family:Arial;margin-bottom:5px; margin-top: 0px;color:black; color:#000000;">Join our <a href="https://www.facebook.com/groups/1264049037309454" target="_blank" style="text-decoration: none;color:#3563ef;"> Facebook cummunity</a> for daily updates.</p> </td> </tr> </tbody> </table> <!-- <div class="overlay">&nbsp;</div> --> </td> </tr> </tbody> <div class="overlay">&nbsp;</div> <table cellpadding="0" cellspacing="0" border="0" width="100%" bgcolor="white"> <tbody> <tr> <td valign="top" colspan="12" width="100.0%" style="width:100.0%;text-align:left;padding:0;font-family:"Helvetica Neue",Helvetica,Arial,sans-serif;font-size:16px;line-height:1.5em;color:#4e524e" align="left"> <div style="line-height:14px"> <p style="margin-bottom:1em;font-family:Geneva,Verdana,Arial,Helvetica,sans-serif;text-align:center;font-size:12px;line-height:1.14em;color:#000000;display:block" align="center">Krademy&nbsp; |&nbsp;&nbsp;A-130, Second Floor, Sector-63, Noida - 201301 </p> <p id="m_-5584124208135577362m_82584675317300226m_3133720979419569615footer" style="margin-bottom:1em;font-family:Geneva,Verdana,Arial,Helvetica,sans-serif;text-align:center;font-size:12px;line-height:1.14em;color:#000000;display:block" align="center">You received this email because you are subscribed to Krademy LMS from Krademy.</p> </div> </td> </tr> </tbody> </table> <!--   <div class="overlay">&nbsp;</div> --> </table> </div> </center> </body> </html>'
                var smtpTransport = nodemailer.createTransport({
                    host: config.mail.host,
                    port: config.mail.port,
                    auth: {
                        user: config.mail.username,
                        pass: config.mail.password
                    },
                    tls: {rejectUnauthorized: false},
                });
                smtpTransport.sendMail({
                    from: config.mail.from,
                    to: data.email,
                    subject: "Welcome to Krademy LMS", 
                    html: html
                }, function(error, info){
                    if(error){
                        console.log(error);
                    }
                    console.log(JSON.stringify(info));
                    resolve(true)
                })
            } catch(err) {
                resolve(true);
            }
        });
    },
}
