import passport from 'passport';
import JWT from 'jsonwebtoken';
import config from '../config';
import { generateQueryString } from '../functions';
// import json from  ;

var JWTSign = function(user, date){
    return JWT.sign({
        iss : config.app.name,
        sub : user.id,
        iam : user.type,
        iat : date.getTime(),
        exp : new Date().setMinutes(date.getMinutes() + 60)
    }, config.app.secret);
}

export var jwtStrategy = (req, res, next) => {
    passport.authenticate('jwt', {session: false}, (err, user, info) => {
        let contype = req.headers['content-type'];
        var json = !(!contype || contype.indexOf('application/json') !== 0);
        if (err && err == 'expired'){ return json?res.status(500).json({ errors: ['Session is expired']}):res.redirect('/logout'); }
        if (err && err == 'invalid'){  return json?res.status(500).json({ errors: ['Invalid token recieved']}):res.redirect('/logout'); }
        if (err && err == 'user'){ return json?res.status(500).json({ errors: ['Invalid user recieved']}):res.redirect('/logout'); }
        if (err && Object.keys(err).length) { return res.status(500).json({ errors: [ err ]}); }
        if (err) {  return res.status(403).json({ errors: [ 'Invalid user recieved' ]}); }
        if (!user) {  return json?res.status(403).json({ errors: ['Invalid user recieved']}):res.redirect('/logout'); }
        //Update Token
        var date = new Date();
        var token = JWTSign(user, date);
        res.cookie('XSRF-token', token, {
            expire: new Date().setMinutes(date.getMinutes() + 60),
            httpOnly: true, secure: config.app.secure
        });
        req.user = user;
        next();
    })(req, res, next);
};

export var jwtLogoutStrategy = (req, res, next) => {
    passport.authenticate('jwt', {session: false}, (err, user, info) => {
        if(user) req.user = user;
        next();
    })(req, res, next);
};

export var rootLoginStrategy = (req, res, next) => { 
        passport.authenticate('root-login', {
            session: false
        }, (err, meta, info) => {
            if (err && err.startsWith("isNewUser")) { var email = err.split("/"); return res.status(200).json({ isnewuser: 1, email: email[1] }); }
            if (err && err == 'invalid') { return res.status(401).json({ errors: ['Email Id not verified'] }); }
            if (err && err == 'attempt') {return res.status(401).json({ errors: ['Too many invalid attempts. Please reset your password.'] }); }
            if (err && err.includes('attempt:')) { return res.status(401).json({ errors: ['Invalid Credentials ' + (err.split(':')[1] != 0 ? err.split(':')[1] + ' Attempt(s) Left' : 'You have exhausted your login attempt limit. Please contact the administrator.') ] }); }
            if (err) { return res.status(401).json({ errors: [err] }); }
            if (!meta) { return res.status(401).json({ errors: ['Invalid Login Credentials '] }); }
            req.user = meta;
            next();
        })(req, res, next);
};

export var singleLoginStrategy = (req, res, next) => {
    passport.authenticate('single-login', { session: false }, (err, meta, skipLogin, info) => {
        if (err && err == 'invalid-session') { return res.status(401).json({ errors: ['Invalid key received.'] }); }
        if (err && err == 'invalid-user') { return res.status(401).json({ errors: ['Invalid user received.'] }); }
        if (err && err == 'invalid-company') { return res.status(401).json({ errors: ['Invalid user received.'] }); }
        if (err && err == 'invalid-course') { return res.status(401).json({ errors: ['Invalid course received.'] }); }
        if (err && err == 'error-key') { return res.status(401).json({ errors: ['Invalid key received.'] }); }
        if (err) { return res.status(401).json({ errors: [err] }); }
        var date = new Date();
        console.log("Skip Login", skipLogin);
        if(!skipLogin){
            if(req.query.pass == undefined && !req.query.pass){
                if(meta && meta.password == null || meta.password == ""){
                    return res.redirect('set-password?'+ generateQueryString({ token: meta.token, redirectTo: req.url }) );
                }else{
                    var token = null;
                    if (req && req.cookies)
                        token = req.cookies['XSRF-token'];
                    if(token == null)
                        return res.redirect('login?'+ generateQueryString({ redirectTo: req.url }) );
                }
            }
        }
        var data = {
            id: meta.id,
            type: meta.Member.type
        }
        token = JWTSign(data, date);
        res.cookie('XSRF-token', token, {
            expire: new Date().setMinutes(date.getMinutes() + 60),
            httpOnly: true, secure: config.app.secure
        });
        req.user = meta; 
        next();
    })(req, res, next);
};