import BaseJoi from 'joi';
import DateExtension from 'joi-date-extensions';
const Joi = BaseJoi.extend(DateExtension);

export var validateBody = (schema) => {
    return (req, res, next) => {
        const result = req.method != 'GET' ? Joi.validate(req.body, schema) : Joi.validate(req.query, schema);
        if (result.error) {
            return res.status(400).json(result.error);
        }

        if (!req.value) { req.value = {}; }
        req.value['body'] = result.value;
        next();
    }
}

export var schemas = {
    appsumoSignUpSchema: Joi.object().keys({
        firstName: Joi.string().required(),
        lastName: Joi.string().required(),
        email: Joi.string().email().required(),
        password: Joi.string().regex(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[$@!%*#?&]).{8,20}$/).min(8).required().error((err) => {
            return '<li style="font-size: 13px;">Password has to be between 8 to 15 characters</li><li style="font-size: 13px;">Password should contain at least one numeric digit</li><li style="font-size: 13px;">Contain at least one uppercase letter</li><li style="font-size: 13px;">Contain at least one lowercase letter</li><li style="font-size: 13px;">Use at least one special character from $, @, !, %, *, #, ?, or &</li>'
        }),
        code: Joi.string().required()
    }),
    registerSchema: Joi.object().keys({
        firstName: Joi.string().required(),
        lastName: Joi.string().required(),
        email: Joi.string().email().required(),
        password: Joi.string().regex(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[$@!%*#?&]).{8,20}$/).min(8).required().error((err) => {
            return '<li style="font-size: 13px;">Password has to be between 8 to 15 characters</li><li style="font-size: 13px;">Password should contain at least one numeric digit</li><li style="font-size: 13px;">Contain at least one uppercase letter</li><li style="font-size: 13px;">Contain at least one lowercase letter</li><li style="font-size: 13px;">Use at least one special character from $, @, !, %, *, #, ?, or &</li>'
        }),
        confhash: Joi.string().valid(Joi.ref('password')).required().error((err) => {
            return "Password does not match."
        }),
        source: Joi.string().optional().allow(null).empty(''),
        URL: Joi.string().optional().allow(null).empty(''),
    }),
    loginSchema: Joi.object().keys({
        email: Joi.string().email().required(),
        password: Joi.string().required()
    }),
    userCheckSchema: Joi.object().keys({
        email: Joi.string().email().required(),
    }),
    rootCheckSchema: Joi.object().keys({
        email: Joi.string().required(),
        alias: Joi.string().optional().allow(null).empty(''),
    }),
    resetPassword: Joi.object().keys({
        email: Joi.string().email().required(),
        token: Joi.string().required(),
        password: Joi.string().regex(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[$@!%*#?&]).{8,20}$/).min(8).required().error((err) => {
            return 'Password has to be between 8 to 15 characters which contain at least one numeric digit, one uppercase and one lowercase letter'
        }),
    }),
    userProfile: Joi.object().keys({
        firstName: Joi.string().required(),
        lastName: Joi.string().optional().allow(null).empty(''),
    }),
    sendResetPassword: Joi.object().keys({
        email: Joi.string().email().required(),
    }),
    setPassword: Joi.object().keys({
        token: Joi.string().required(),
        password: Joi.string().regex(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[@#$%!&]).{8,20}$/).min(8).required().error((err) => {
            return '<li style="font-size: 13px;">Password has to be between 8 to 15 characters</li><li style="font-size: 13px;">Password should contain at least one numeric digit</li><li style="font-size: 13px;">Contain at least one uppercase letter</li><li style="font-size: 13px;">Contain at least one lowercase letter</li><li style="font-size: 13px;">Use at least one special character from $, @, !, %, *, #, ?, or &</li>'
        }),
    }),
    employeePolicyUpdateSchema: Joi.object().keys({
        list:Joi.array().required(),
    }),
    addUserSchema: Joi.object().keys({
        firstName: Joi.string().required(),
        lastName: Joi.string().optional().allow(null).empty(''),
        email: Joi.string().email().required(),
        phone: Joi.string().optional().allow(null).empty(''),
        domainCode: Joi.string().optional().allow(null).empty(''),
    }),
    editUserSchema: Joi.object().keys({
        firstName: Joi.string().required(),
        lastName: Joi.string().optional().allow(null).empty(''),
        email: Joi.string().email().required(),
        phone: Joi.string().optional().allow(null).empty(''),
    }),
     
	changePassword: Joi.object().keys({
        oldPass: Joi.string().required(),
		newPass: Joi.string().regex(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[@#$%!&]).{8,20}$/).min(8).required().error((err) => {
			return 'Password has to be between 8 to 15 characters which contain at least one numeric digit, one uppercase letter, one lowercase letter and one special character.'
		}),
		confirmPass: Joi.string().valid(Joi.ref('newPass')).required().error((err) => {
			return "Password does not match."
		}),
    }),
    headerCreate: Joi.object().keys({
        senderId: Joi.number().min(1).required(),
        key: Joi.string().max(100, 'utf8').required(),
        value: Joi.string().max(100, 'utf8').required(),
      }),

}