'use strict';
import Blowfish from 'egoroof-blowfish';
import config from '../config';

const bf = new Blowfish(toString(config.app.secret), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);

const fromHexString = hexString => new Uint8Array(hexString.match(/.{1,2}/g).map(byte => parseInt(byte, 16)));
const toHexString = bytes => bytes.reduce((str, byte) => str + byte.toString(16).padStart(2, '0'), '');

module.exports = (sequelize, DataTypes) => {
	const Sender = sequelize.define('Sender', {
		companyId: DataTypes.INTEGER,
		name: DataTypes.STRING,
		fromAddress: DataTypes.STRING,
		host: DataTypes.STRING,
		username: DataTypes.TEXT,
		password: DataTypes.STRING,
		ignore_cert_errors: DataTypes.INTEGER,
		ssl: DataTypes.BOOLEAN,
		requiretls: DataTypes.BOOLEAN,
		pool: DataTypes.BOOLEAN,
		limit: DataTypes.INTEGER,
		delay: DataTypes.INTEGER,
		maxMessages: DataTypes.INTEGER,
		maxConnections: DataTypes.INTEGER,
		connectionTimeout: DataTypes.INTEGER,
	}, {
			timestamps: true,
			paranoid: true,
		});
	Sender.associate = function (models) {
		models.Sender.belongsTo(models.Company, { foreignKey: 'companyId' });
		models.Sender.hasMany(models.Header, { foreignKey: 'senderId' });
	};

	Sender.beforeCreate(function (model, options, cb) {
		return new Promise((resolve, reject) => {
			if (model.password && model.password.length) model.password = toHexString(bf.encode(model.password));
			return resolve(model, options);
		});
	});

	Sender.beforeUpdate(function (model, options, cb) {
		return new Promise((resolve, reject) => {
			if (model.password && model.password.length) model.password = toHexString(bf.encode(model.password));
			return resolve(model, options);
		});
	});

	Sender.beforeBulkUpdate(function (model, options, cb) {
		if (model.attributes && model.attributes.password && model.attributes.password.length) model.attributes.password = toHexString(bf.encode(model.attributes.password));
	});

	return Sender;
};